/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_adc.c
* Version      : Code Generator for RL78/G1F V1.00.01.03 [07 Mar 2016]
* Device(s)    : R5F11BLE
* Tool-Chain   : CA78K0R
* Description  : This file implements device driver for ADC module.
* Creation Date: 2016/07/13
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_adc.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_ADC_Create
* Description  : This function initializes the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Create(void)
{
    /********** configure A/D converter **********/
    ADCEN = 1;                                                          /* enables input clock supply for A/D converter */

    /* ADM0 . A/D converter mode register 0 ********************************************************************
    b7 ADCS       - A/D conversion operation control - stops conversion operation
    b6 ADMD       - specification of the A/D conversion channel selection mode - select mode
    b5:b3 FR2:FR0 - conversion time selection - 2.375[us]
    b2:b1 LV1:LV0 - mode - normal 1
    b0 ADCE       - A/D voltage comparator operation control - stops A/D voltage comparator operation
    ***********************************************************************************************************/
    ADM0 = 0x30;

    /* ADM1 . A/D converter mode register 1 ********************************************************************
    b7:b6 ADTMD1:ADTMD0 - selection of the A/D conversion trigger mode - software trigger mode
    b5 ADSCM            - specification of the A/D conversion mode - one-shot conversion mode
    b4:b2               - reseved set to 0
    b1:b0 ADTRS1:ADTRS0 - selection of the hardware trigger signal
                        - end of timer channel 1 count or capture interrupt signal (INTTM01)
    ***********************************************************************************************************/
    ADM1 = 0x20;

    /* ADM2 . A/D converter mode register 2 ********************************************************************
    b7:b6 ADREFP1:ADREFP0 - selection of the "+" side reference voltage source of the A/D converter
                          - supplied from VDD
    b5 ADREFM             - selection of the "-" side reference voltage source of the A/D converter - supplied from VSS
    b4                    - reseved set to 0
    b3 ADCRK              - checking the upper limit and lower limit conversion result values
                          - the interrupt signal is output
                            when the ADLL register <= the ADCR register <= the ADUL register
    b2 AWC                - specification of the wakeup function (SNOOZE mode) - do not use the SNOOZE mode function
    b1                    - reseved set to 0
    b0 ADTYP              - selection of the A/D conversion resolution - 10bit resolution
    ***********************************************************************************************************/
    ADM2 = 0x00;

    /* initialize I/O port *************************************************************************************
    AN0 - temperature of RAJ306000
    AN1 - temperature of MOS
    AN3 - voltage of VM
    AN5 - temperature of board sensor
    AN6 - VR1
    AN7 - voltage of pridriver
    ***********************************************************************************************************/
    PM2.0 = 1;                                                          /* input mode for AN0 */
    PM2.1 = 1;                                                          /* input mode for AN1 */
    PM2.3 = 1;                                                          /* input mode for AN3 */
    PM2.5 = 1;                                                          /* input mode for AN5 */
    PM2.6 = 1;                                                          /* input mode for AN6 */
    PM2.7 = 1;                                                          /* input mode for AN7 */
    PMC2 = 0xEB;

    ADCE = 1;                                                           /* conversion standby mode (only A/D voltage comparator consumes power) */

    ADMK = 1;                                                           /* INTAD(trigger:end of A/D conversion) interrupt servicing disabled */
}
/***********************************************************************************************************************
* Function Name: R_ADC_Start
* Description  : This function starts the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Start(void)
{
    ADIF = 0U;                                                          /* clear INTAD interrupt flag */
    ADMK = 0U;                                                          /* enable INTAD interrupt */
    ADCS = 1U;                                                          /* enables conversion operation */
}
/***********************************************************************************************************************
* Function Name: R_ADC_Stop
* Description  : This function stops the AD converter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Stop(void)
{
    ADCS = 0U;                                                          /* stops conversion operation */
    ADIF = 0U;                                                          /* clear INTAD interrupt flag */
    ADMK = 0U;                                                          /* enable INTAD interrupt */
}
/***********************************************************************************************************************
* Function Name: R_ADC_Set_OperationOn
* Description  : This function enables comparator operation.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Set_OperationOn(void)
{
    ADCE = 1U;                                                          /* enables A/D voltage comparator operation */
}
/***********************************************************************************************************************
* Function Name: R_ADC_Set_OperationOff
* Description  : This function stops comparator operation.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Set_OperationOff(void)
{
    ADCE = 0U;                                                          /* stops A/D voltage comparator operation */
}
/***********************************************************************************************************************
* Function Name: R_ADC_Get_Result
* Description  : This function returns the conversion result in the buffer.
* Arguments    : buffer -
*                    the address where to write the conversion result
* Return Value : None
***********************************************************************************************************************/
void R_ADC_Get_Result(uint16_t * const buffer)
{
    *buffer = (uint16_t)(ADCR >> 6U);
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
