/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_predrv.h
* Version      : 
* Device(s)    : 
* Tool-Chain   : 
* Description  : This file implements device driver for pre-driver module.
* Creation Date: 2016/07/05
***********************************************************************************************************************/
#ifndef PREDRV_H
#define PREDRV_H

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

/* SPI */
#define REG_BUFF_SIZE           (2)
#define SPI_CHK_MAX             (100)                                   /* SPI Access Check Count Max */

/* PreDriver Seq Error */
#define PREDRV_NORMAL           (0)                                     /* No Error */
#define PREDRV_SPI_ERROR        (1)                                     /* SPI Error */
#define PREDRV_ALARM_ERROR      (2)                                     /* ALARM Error */
#define PREDRV_LOGIC_ERROR      (3)                                     /* LOGIC Error */
#define PREDRV_REGRW_ERROR      (4)                                     /* Reg R/W Error */

/* PreDriver Status Check Macro */
#define PREDRV_SPI_ACCESS_OK    (0x6A)
#define PREDRV_NO_RESET_ERROR   (0x01)

/* PreDriver ALARM Raw Status1 */
#define PREDRV_ALMRAW1_OK       (0xEF)

/* PreDriver ALARM Status1 */
#define ALMSTS1_TSD_N           (0x01)                                  /* bit0 = 0 : detect TSD_N */
#define ALMSTS1_OCP_N           (0x02)                                  /* bit1 = 0 : detect OCP_N */
#define ALMSTS1_VGB_UVP_N       (0x04)                                  /* bit2 = 0 : detect VGB_UVP_N */
#define ALMSTS1_VGB_OVP_N       (0x08)                                  /* bit3 = 0 : detect VGB_OVP_N */
#define ALMSTS1_VGT_UVP_N       (0x10)                                  /* bit4 = 0 : detect VGT_UVP_N */
#define ALMSTS1_VGT_OVP2_N      (0x20)                                  /* bit5 = 0 : detect VGT_OVP2_N */
#define ALMSTS1_VGT_OVP1_N      (0x40)                                  /* bit6 = 0 : detect VGT_OVP1_N */
#define ALMSTS1_VREG5_OVP_N     (0x80)                                  /* bit7 = 0 : detect VREG5_OVP_N*/

#define ALMSTS1_NO_ERROR        (0xEF)

#define ALMSTS1_VGT_UVP_MASK    (0xEF)                                  /* ALARM factor(VGT_UVP) mask */

/* PreDriver ALARM Status2 */
#define ALMSTS2_VM_UVP_N        (0x01)                                  /* bit0 = 0 : detect VM_UVP_N */
#define ALMSTS2_DI_SEL_W_CMP_N  (0x20)                                  /* bit5 = 0 : detect DI_SEL_W_CMP_N */
#define ALMSTS2_DI_SEL_V_CMP_N  (0x40)                                  /* bit6 = 0 : detect DI_SEL_V_CMP_N */
#define ALMSTS2_DI_SEL_U_CMP_N  (0x80)                                  /* bit7 = 0 : detect DI_SEL_U_CMP_N */

#define ALMSTS2_NO_ERROR        (0xFF)

/* PreDriver Initail Parameter Macro */
#define INIT_PS_ALL             (0x01)
#define INIT_PS_1ST             (0x3E)
#define INIT_PS_2ND             (0x3F)
#define INIT_PS_3RD             (0xBF)

#define INIT_SELSIG_U           (0x03)
#define INIT_SELSIG_V           (0x14)
#define INIT_SELSIG_W           (0x25)
#define INIT_HALL_SIG           (0xA0)
#ifdef HW_ERR_DET_ON
#define INIT_ALMOPE1            (0x10)
#define INIT_ALMOUT1            (0x10)
#else
#define INIT_ALMOPE1            (0x17)
#define INIT_ALMOUT1            (0xFF)
#endif

#define INIT_CS_SET2            (0x60)

#define INIT_ERROR_WAIT         (0x00)
#define INIT_CS_SET1            (0x08)
#define INIT_HAIC_TH            (0x00)
#define INIT_LD_WAIT            (0x00)

#define INIT_DRIVE_SET          (0x01)

#define INIT_IDRCNT_H           (0x00)
#define INIT_IDRCNT_L           (0x00)
#define INIT_TRCNT_P            (0x00)

#define INIT_CPSET1             (0x01)
#define INIT_CPSET2             (0x02)
#define INIT_CP_TRIM            (0x00U)

#define INIT_VREG5_TRIM         (0x20U)
#define INIT_CSAMP_TRIM         (0x20U)

#define INIT_TRIM_PT            (0x00U)
#define INIT_TRIM_PT_UP         (0x95U)                                 /* _UP: unprotected */

#define INIT_TRIM_EN            (0x00U)
#define INIT_TRIM_EN_EFWD       (0x01U)

#define INIT_BGR_TRIM           (0x00U)
#define INIT_BFAMP_TRIM         (0x00U)

#define MASK_VREG5_TRIM         (0xF0U)
#define MASK_CSAMP_TRIM         (0x0FU)

#define MASK_BGR_TRIM           (0x1FU)
#define MASK_BFAMP_TRIM         (0x3EU)

#ifdef ICS_USE
#define INIT_ICS_PS_ALL         (INIT_PS_ALL)
#define INIT_ICS_PS             (INIT_PS_3RD)
#define INIT_ICS_SW_RESET       (0x00)
#define INIT_ICS_ADC_SEL        (0x00)
#define INIT_ICS_SELSIG_U       (0x03)
#define INIT_ICS_SELSIG_V       (0x14)
#define INIT_ICS_SELSIG_W       (0x25)
#define INIT_ICS_HALL_SIG       (INIT_HALL_SIG)
#define INIT_ICS_ALMSTS1        (0xFF)
#define INIT_ICS_ALMOPE1        (INIT_ALMOPE1)
#define INIT_ICS_ALMOUT1        (INIT_ALMOUT1)
#define INIT_ICS_ALMSTS2        (0xFF)
#define INIT_ICS_CS_SET2        (INIT_CS_SET2)
#define INIT_ICS_ALMOUT2        (0x00)
#define INIT_ICS_ERROR_WAIT     (0x00)

#define INIT_ICS_CS_SET1        (INIT_CS_SET1)
#define INIT_ICS_HAIC_TH        (0x00)
#define INIT_ICS_PDDSTS         (0xF0)
#define INIT_ICS_LD_WAIT        (0x00)
#define INIT_ICS_DRIVE_SET      (INIT_DRIVE_SET)
#define INIT_ICS_DI_TIME        (0x00)
#define INIT_ICS_IDRCNT_H       (0x00)
#define INIT_ICS_IDRCNT_L       (0x00)
#define INIT_ICS_TRCNT_P        (0x00)
#define INIT_ICS_CPSET1         (0x01)
#define INIT_ICS_CPSET2         (0x02)
#define INIT_ICS_CP_TRIM        (INIT_CP_TRIM)

#define INIT_ICS_ALMRAW1        (0xFF)
#define INIT_ICS_TOIN_MONI      (0x00)
#define INIT_ICS_WHO_AM_I       (0x6A)

#define INIT_ICS_TRIM_PT        (INIT_TRIM_PT)
#endif

#define INIT_ICS_VREG5_TRIM     (INIT_VREG5_TRIM)
#define INIT_ICS_CSAMP_TRIM     (INIT_CSAMP_TRIM)

#define INIT_ICS_TRIM_EN        (INIT_TRIM_EN)

#define INIT_ICS_BGR_TRIM       (INIT_BGR_TRIM)
#define INIT_ICS_BFAMP_TRIM     (INIT_BFAMP_TRIM)

/* PreDriver Error Recovery Parameter Macro */
#define ERRRCV_PS_1ST           (0x3C)
#define ERRRCV_PS_2ND           (0x3E)
#define ERRRCV_PS_3RD           (0x3F)
#define ERRRCV_PS_4TH           (0xBF)
#define ERRRCV_MOT_EN_CLR       (0x00)
#define ERRRCV_MOT_EN_SET       (0x01)
#define ERRRCV_ALM_LATCH_CLR    (0x40)

/* wait time */
#define WAITTIME_1_MS           (0x11F8)                                /* (1 / 32MHz) * 7cycles * 0x11F8 = 1[ms] */
#define WAITTIME_3_MS           (0x35E8)                                /* (1 / 32MHz) * 7cycles * 0x35E8 = 3[ms] */

/* PreDriver ADC_SEL Request Macro */
#define REQ_ADC_SEL_ISEN        (0x00)
#define REQ_ADC_SEL_BEMF        (0x01)

/* PreDriver ADC_SEL Setting Parameter Macro */
#define SET_ADC_SEL_ISEN        (0x01)
#define SET_ADC_SEL_BEMF        (0x03)

/* PreDriver MOT_EN bit of DRIVE_SET Request Macro */
#define REQ_MOT_EN_OFF          (0x00)
#define REQ_MOT_EN_ON           (0x01)

/* PreDriver MOT_EN bit of DRIVE_SET Setting Parameter Macro */
#define SET_MOT_EN_CLR          (0x00)
#define SET_MOT_EN_SET          (0x01)

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/
typedef enum
{
    SEQ_INIT = 0,
    SEQ_CHK_SPI,
    SEQ_CHK_TSD_N,
    SEQ_SET_HALL_SIG_PRM,
    SEQ_SET_5VTRIM,
    SEQ_SET_ALMOPE1_PRM,
    SEQ_SET_ALMOUT1_PRM,
    SEQ_SET_CS_SET2_PRM,
    SEQ_SET_CS_SET1_PRM,
    SEQ_SET_SEQINIT_PRM,
    SEQ_SET_PS_ALL_PRM,
    SEQ_SET_PS_1ST_PRM,
    SEQ_SET_PS_2ND_PRM,
    SEQ_CHK_ALMRAW1,
    SEQ_SET_PS_3RD_PRM,
    SEQ_CHK_ALMSTS,
    SEQ_SET_MOT_EN,
    SEQ_END,

    SEQ_NUM_MAX,

} predrv_seq_t;

typedef enum
{
    ERR_RCV_SEQ_INIT = 0,
    ERR_RCV_SEQ_CHK_STS,
    ERR_RCV_SEQ_CLR_MOT_EN,
    ERR_RCV_SEQ_SET_PS_1ST,
    ERR_RCV_SEQ_CHK_ALMSTS_1ST,
    ERR_RCV_SEQ_SET_ALM_LATCH_CLR,
    ERR_RCV_SEQ_SET_PS_2ND,
    ERR_RCV_SEQ_SET_PS_3RD,
    ERR_RCV_SEQ_CHK_ALMRAW1,
    ERR_RCV_SEQ_SET_PS_4TH,
    ERR_RCV_SEQ_CHK_ALMSTS_2ND,
    ERR_RCV_SEQ_SET_MOT_EN,
    ERR_RCV_SEQ_END,

    ERR_RCV_SEQ_NUM_MAX,

} predrv_err_rcv_seq_t;

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
void predriver_hw_init(void);
MD_STATUS R_PREDRV_TRIM_Create(void);
void R_PREDRV_InitSequence(void);
void R_PREDRV_PARAM_Create(void);
void R_PREDRV_ErrorRecoverySequence(uint8_t almsts1, uint8_t almsts2);

uint8_t R_PreDrv_Get_ErrorCode(void);

MD_STATUS R_PreDrvReg_Read(uint8_t address, uint8_t *rd_data);
MD_STATUS R_PreDrvReg_Write(uint8_t address, uint8_t wr_data);
MD_STATUS R_PreDrvReg_Write_Intr_Mode(uint8_t address, uint8_t wr_data);

void R_PreDrv_Set_AdcSel(uint8_t u1_req);
void R_PreDrv_Set_MotEn(uint8_t u1_req);

#endif
