/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_rl78g1f_t2001.h
* Description : A definitions of the processing of a H/W control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_CTRL_RL78G1F_T2001_H
#define MTR_CTRL_RL78G1F_T2001_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define MTR_PWM_TIMER_FREQ      (64.0f)                                 /* PWM timer frequency[MHz] */
#define MTR_CARRIER_FREQ        (10.0f)                                 /* carrier wave frequency[KHz] */

#define MTR_DEADTIME            (0)                                     /* deadtime[ns] */
#define MTR_DEADTIME_SET        ((int16_t)((MTR_DEADTIME * MTR_PWM_TIMER_FREQ) / 1000))
                                                                        /* setting value of deadtime */
#define MTR_CARRIER_SET         ((((MTR_PWM_TIMER_FREQ * 1000) / MTR_CARRIER_FREQ) / 2) + MTR_DEADTIME_SET - 2)
                                                                        /* setting value of carrier */
#define MTR_START_CARRIER_SET   ((MTR_CARRIER_SET * 30) / 100)          /* setting value of 30% carrier */

#define MTR_VR1_ADC_MAX         (802)                                   /* max valule of VR1 ADC */

#define MTR_PWM_DUTY_CALC_COEF1 ((MTR_CARRIER_SET * 100) / MTR_VR1_ADC_MAX)
                                                                        /* coef1 of pwm duty calculation */
#define MTR_PWM_DUTY_CALC_COEF2 (100)                                   /* coef2 of pwm duty calculation */

#define MTR_RPM_CALC_COEF1      (556)                                   /* coef1 of rpm calculation */
#define MTR_RPM_CALC_COEF2      (16888)                                 /* coef2 of rpm calculation */
#define MTR_RPM_CALC_COEF3      (100)                                   /* coef3 of rpm calculation */

#define MTR_PORT_BEMF_U         (P5.2)                                  /* output port of BEMF measure(U) */
#define MTR_PORT_BEMF_V         (P5.3)                                  /* output port of BEMF measure(V) */
#define MTR_PORT_BEMF_W         (P5.4)                                  /* output port of BEMF measure(W) */

#define MTR_PORT_UP             (P1.5)                                  /* output port of PWM timer(Up) */
#define MTR_PORT_UN             (P1.4)                                  /* output port of PWM timer(Un) */
#define MTR_PORT_VP             (P1.3)                                  /* output port of PWM timer(Vp) */
#define MTR_PORT_VN             (P1.1)                                  /* output port of PWM timer(Vn) */
#define MTR_PORT_WP             (P1.2)                                  /* output port of PWM timer(Wp) */
#define MTR_PORT_WN             (P1.0)                                  /* output port of PWM timer(Wn) */

#define MTR_GET_ICS_CTRL_REQ    (P6.3)                                  /* get ICS Control Request */
#define MTR_GET_ROT_DIR_REQ     (P12.2)                                 /* get Rotation Direction Request */

#define MTR_PORT_LED1           (P0.1)                                  /* output port of LED1 */
#define MTR_PORT_LED2           (P0.0)                                  /* output port of LED2 */
#define MTR_PORT_LED3           (P14.1)                                 /* output port of LED3 */
#define MTR_PORT_LED4           (P14.0)                                 /* output port of LED4 */
#define MTR_PORT_LED5           (P4.3)                                  /* output port of LED5 */
#define MTR_PORT_LED6           (P4.2)                                  /* output port of LED6 */

#define MTR_LED_ON              (0)                                     /* active level of LED */
#define MTR_LED_OFF             (1)                                     /* inactive level of LED */

#define MTR_OVERVOLTAGE_LIMIT   ((int16_t)(30 * 128))                   /* over voltage limit [V] */
#define MTR_UNDERVOLTAGE_LIMIT  ((int16_t)( 6 * 128))                   /* under voltage limit [V] */

#define MTR_VDC_SCALING         (1471)                                  /* calculation factor (bus voltage) */

#define MTR_RECIVDC_SCALING     (256)                                   /* calculation factor (reciprocal bus voltage) */
                                                                        /* (MTR_RECIVDC_SCALING) * (2^16) / (vdc * 2^7) = (1 / vdc * 2^15) */
#define MTR_TAU1_CNT            (TCR01)                                 /* TAU1 count register for speed calculation */

/* A/D converter channel number */
#define MTR_ADCCH_RAJ306000_TEMP (0)                                    /* temperature of RAJ306000 */
#define MTR_ADCCH_MOS_TEMP       (1)                                    /* temperature of MOS */
#define MTR_ADCCH_VM             (3)                                    /* voltage of VM */
#define MTR_ADCCH_BOARD_TEMP     (5)                                    /* temperature of board sensor */
#define MTR_ADCCH_VR1            (6)                                    /* VR1 */
#define MTR_ADCCH_PDRV           (7)                                    /* voltage of prideriver */

/******************************************************************************
* Global function definitions
******************************************************************************/
void mtr_ctrl_start(void);                                              /* motor control start */
void mtr_ctrl_stop(void);                                               /* TRD output disable, other interrupt settings, */
                                                                        /* motor output port is on/off for motor control stop */
void mtr_change_pattern(uint8_t);                                       /* change voltage pattern */

int16_t mtr_get_adc(uint8_t);                                           /* A/D conversion */

#endif /* MTR_CTRL_RL78G1F_T2001_H */
