/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : r_less_120_is.h
* Description : A definitions of the processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 27.06.2019  Ver.1.01
******************************************************************************/

/* guard against multiple inclusion */
#ifndef R_LESS_120_IS_H
#define R_LESS_120_IS_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define IS_DIRECTION_CW         (MTR_CW)                                /* direction of rotation: clockwise */
#define IS_DIRECTION_CCW        (MTR_CCW)                               /* direction of rotation: counterclockwise */

#define IS_SECTION_TIME_A       (2)                                     /* time of "a" section: 40[us] = 20[us] x 2 */
#define IS_SECTION_TIME_B       (4)                                     /* time of "b" section: 80[us] = 20[us] x 4 */
#define IS_SECTION_TIME_C       (4)                                     /* time of "c" section: 80[us] = 20[us] x 4 */

#define IS_TIME_RESOLUTION      (20)                                    /* time resolution [us] */

#define IS_PWM_OUT_TIME         (125)                                   /* output time of motor control signal: 2.5[ms] = 20[us] x 125 */
#define IS_PWM_OFF_TIME         (5)                                     /* stop time of motor control signal: 100[us] =  20[us] x 20 */

#define IS_KICK_NUM             (1)                                     /* addition value to stop position */

#define IS_TRIAL_NUM            (4)                                     /* number of trials for inductive sense */

/******************************************************************************
* Typedef definitions
******************************************************************************/
typedef enum
{
    IS_STATE_INIT = 0,                                                  /* initial value */
    IS_STATE_NORMAL,                                                    /* normal */
    IS_STATE_ERROR_PORT,                                                /* error when setting general port */
    IS_STATE_ERROR_WAIT,                                                /* error when waitting for a specified time */
    IS_STATE_ERROR_WDT,                                                 /* error when clearing watchdog timer */
    IS_STATE_ERROR_ADC,                                                 /* error when getting A/D conversion result */

    IS_STATE_MAX
} e_is_status_t;                                                        /* return value */

typedef uint16_t is_revision_t;                                         /* library revision */

typedef struct
{
    uint8_t     u1_direction;                                           /* direction of rotation */

    uint8_t     u1_reserved;

    uint16_t    u2_section_time_a;                                      /* time of "a" section */
    uint16_t    u2_section_time_b;                                      /* time of "b" section */
    uint16_t    u2_section_time_c;                                      /* time of "c" section */
} st_is_prm_t;                                                          /* initial values of inductive sense processing */

typedef uint8_t is_result_t;                                            /* stop position (result of inductive sense processing) */

typedef enum
{
    IS_PATTERN_1 = 0,
    IS_PATTERN_2,
    IS_PATTERN_3,
    IS_PATTERN_4,
    IS_PATTERN_5,
    IS_PATTERN_6,

    IS_PATTERN_MAX
} e_is_pattern_t;                                                       /* current pattern */

/******************************************************************************
* Global function definitions
******************************************************************************/
e_is_status_t R_IS_Init(void);                                          /* initialization of library */
e_is_status_t R_IS_GetRevision(is_revision_t*);                         /* get library revision */
e_is_status_t R_IS_Main(const st_is_prm_t, is_result_t*);               /* main process of inductive sense */

#endif /* R_LESS_120_IS_H */
