/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_spm_less_120_cpm.c
* Description : The processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_spm_less_120_cpm.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "r_cg_macrodriver.h"
#include "r_cg_predrv.h"
#include "r_less_120_isw.h"

/******************************************************************************
* Exported global variables (to be accessed by other files)
******************************************************************************/
volatile uint16_t   g_u2_cnt_boot_mode;                                 /* counter for boot mode time */
volatile uint16_t   g_u2_cnt_wait_stop;                                 /* counter for waiting for motor stop */
volatile uint8_t    g_u1_flg_wait_stop;                                 /* flag for waiting for motor stop */

volatile uint8_t    g_u1_enable_write;                                  /* ICS write enable flag */

volatile int16_t    g_s2_vdc_ad;                                        /* bus voltage : scale(2^7) */
volatile int16_t    g_s2_reci_vdc_ad;                                   /* reciprocal bus voltage : scale(2^15) */
volatile int16_t    g_s2_pdrv_ad;                                       /* predriver voltage : scale(2^0) */

volatile int16_t    g_s2_pwm_duty;                                      /* PWM duty : scale(2^0) */
volatile int16_t    g_s2_ref_speed_rad;                                 /* motor speed reference : scale(2^2) */
volatile int16_t    g_s2_speed_rad;                                     /* motor speed : scale(2^2) */
volatile int16_t    g_s2_speed_lpf_k;                                   /* speed LPF parameter : scale(2^14) */

#ifdef ICS_USE
/****** for ICS ******/
volatile uint8_t    g_u1_cnt_ics;                                       /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
volatile uint16_t   g_u2_run_mode;                                      /* run mode */
volatile uint8_t    g_u1_error_status;                                  /* error status */
volatile uint8_t    g_u1_mode_system;                                   /* system mode */

/******************************************************************************
* Private global variables and functions
******************************************************************************/
volatile uint8_t    g_u1_v_pattern;                                     /* voltage pattern */
volatile uint16_t   g_u2_cnt_timeout;                                   /* counter for timeout error */
volatile uint8_t    g_u1_direction;                                     /* rotational direction (0:CW ,1:CCW) */
volatile uint16_t   g_u2_less_timer_cnt;                                /* value of timer counter per 60 degrees */
volatile uint16_t   g_u2_pre_less_timer_cnt;                            /* previous value of timer counter per 60 degrees */
volatile int16_t    g_s2_timer_cnt_ave;                                 /* counts for 360 degrees */
volatile uint16_t   g_u2_timer_cnt_buf[MTR_TIMER_CNT_BUF_NUM];          /* counts for 60 degrees */
volatile uint16_t   g_u2_timer_cnt_num;                                 /* array element number before 360 degrees */

volatile MTR_ICS_INPUT ics_input_buff;                                  /* struct for ICS input */
volatile uint8_t    g_u1_sensorless_mode;                               /* sensorless mode */
volatile uint8_t    g_u1_bemf_mode;                                     /* BEMF mode */
volatile uint8_t    g_u1_bemf_seq;                                      /* BEMF sequence */
volatile int8_t     g_s1_indsen_signal;                                 /* current pattern */
volatile int16_t    g_s2_bemf_ad_pre;                                   /* BEMF (pre) */
volatile int16_t    g_s2_bemf_ad_now;                                   /* BEMF (now) */
volatile int16_t    g_s2_bemf_time_cnt;                                 /* counts for BEMF wait time */
volatile int16_t    g_s2_bemf_time_fg_calc_pre;                         /* 60 degrees calculation (pre) */
volatile int16_t    g_s2_bemf_time_fg_calc_now;                         /* 60 degrees calculation (now) */
volatile int16_t    g_s2_bemf_time_a_cnt;                               /* counts for BEMF a-time */
volatile int16_t    g_s2_bemf_time_a_cnt_pre;                           /* counts for BEMF a-time (before exceeding the BEMF threshold) */
volatile int16_t    g_s2_bemf_time_b_cnt;                               /* counts for BEMF b-time */
volatile int16_t    g_s2_bemf_time_b_cnt_now;                           /* counts for BEMF b-time (after exceeding the BEMF threshold) */
volatile int16_t    g_s2_bemf_time_fg_cnt;                              /* counts for 30 degrees */
volatile int16_t    g_s2_bemf_fg_th_pre;                                /* threshold of 30 degrees count (pre) */
volatile int16_t    g_s2_bemf_fg_th_now;                                /* threshold of 30 degrees count (now) */
volatile int16_t    g_s2_bemf_calc_pre;                                 /* calculation (pre) */
volatile int16_t    g_s2_bemf_calc_now;                                 /* calculation (now) */
volatile uint8_t    g_u1_bemf_pwm_phase;                                /* type of voltage pattern */
volatile int16_t    g_s2_bemf_timeout_cnt;                              /* counts for BEMF timeout */
volatile int16_t    g_s2_bemf_time_pwmchg_cnt;                          /* counts for Hi-Z detection ADC timing */
volatile uint8_t    g_u1_bemf_chk_hiz;                                  /* type of Hi-Z */
volatile uint8_t    g_u1_bemf_chk_hiz_status;                           /* status of Hi-Z detection */
volatile uint8_t    g_u1_bemf_req_moten_on;                             /* request MOT_EN on */
volatile uint8_t    g_u1_bemf_req_moten_off;                            /* request MOT_EN off */
volatile uint8_t    g_u1_bemf_mode_cnt;                                 /* count for BEMF mode */
volatile uint8_t    g_u1_bemf_th_type;                                  /* type of BEMF */
volatile uint8_t    g_u1_bemf_ad_type;                                  /* type of ADC */
volatile int16_t    g_s2_pdrv_ad_isense;                                /* current */
volatile int16_t    g_s2_bemf_fix_pwm_duty_cnt;                         /* counts for pwm duty fixed */
volatile int16_t    g_s2_bemf_timer_cnt_ave;                            /* counts for 360 degrees */
volatile uint8_t    g_u1_bemf_limit_pwm_duty_flg;                       /* pwm duty clamp flag */
volatile uint8_t    g_u1_bemf_limit_vr_ad_flg;                          /* VR1 ADC limit flag */
volatile uint16_t   g_u2_bemf_limit_vr_ad_cnt;                          /* counts for counts VR1 ADC limit */
volatile int16_t    g_u1_bemf_limit_vr_ad;                              /* VR1 ADC limit calculation */

/* value of current pattern */
const volatile uint8_t g_u1_pwm_pattarn[2][6] = {
    /* CW */
    {
        MTR_PATTERN_U_V,
        MTR_PATTERN_U_W,
        MTR_PATTERN_V_W,
        MTR_PATTERN_V_U,
        MTR_PATTERN_W_U,
        MTR_PATTERN_W_V,
    },

    /* CCW */
    {
        MTR_PATTERN_W_V,
        MTR_PATTERN_W_U,
        MTR_PATTERN_V_U,
        MTR_PATTERN_V_W,
        MTR_PATTERN_U_W,
        MTR_PATTERN_U_V,
    },
};

/* BEMF direction */
const volatile uint8_t g_u1_bemf_dir[2][6] = {
    /* CW */
    {
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
    },

    /* CCW */
    {
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
        MTR_BEMF_DOWM,
        MTR_BEMF_UP,
    },
};

/* current pattern for Hi-Z detection */
const volatile uint8_t g_u1_bemf_v_pattern[2][6][3] = {
    /* CW */
    {
        /* U  V  W */
        {  1, 0, 1 },
        {  1, 0, 0 },
        {  1, 1, 0 },
        {  0, 1, 0 },
        {  0, 1, 1 },
        {  0, 0, 1 },
    },

    /* CCW */
    {
        /* U  V  W */
        {  1, 0, 1 },
        {  0, 0, 1 },
        {  0, 1, 1 },
        {  0, 1, 0 },
        {  1, 1, 0 },
        {  1, 0, 0 },
    },
};

static void mtr_speed_calc(void);
static void mtr_start_init(void);

/* for sequence control */
static uint8_t mtr_act_run(uint8_t);
static uint8_t mtr_act_stop(uint8_t);
static uint8_t mtr_act_reset(uint8_t);
static uint8_t mtr_act_error(uint8_t);
static uint8_t mtr_act_none(uint8_t);

static int16_t    g_s2_bemf_ad[MTR_BEMF_TH_MAX][MTR_BEMF_HIZ_MAX];      /* BEMF threshold */

static const volatile uint8_t g_u1_def_state[MTR_SIZE_EVENT][MTR_SIZE_STATE] = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ MTR_MODE_STOP,   MTR_MODE_STOP,   MTR_MODE_ERROR },
/* 1:MTR_EVENT_RUN   */{ MTR_MODE_RUN,    MTR_MODE_RUN,    MTR_MODE_ERROR },
/* 2:MTR_EVENT_ERROR */{ MTR_MODE_ERROR,  MTR_MODE_ERROR,  MTR_MODE_ERROR },
/* 3:MTR_EVENT_RESET */{ MTR_MODE_STOP,   MTR_MODE_ERROR,  MTR_MODE_STOP  },};

static uint8_t (* gp_u1_def_action[MTR_SIZE_EVENT][MTR_SIZE_STATE])(uint8_t u1_state) = {
/*    state              0:MTR_MODE_STOP, 1:MTR_MODE_RUN,  2:MTR_MODE_ERROR */
/* event */
/* 0:MTR_EVENT_STOP  */{ mtr_act_none,    mtr_act_stop,    mtr_act_none  },
/* 1:MTR_EVENT_RUN   */{ mtr_act_run,     mtr_act_none,    mtr_act_none  },
/* 2:MTR_EVENT_ERROR */{ mtr_act_error,   mtr_act_error,   mtr_act_none  },
/* 3:MTR_EVENT_RESET */{ mtr_act_reset,   mtr_act_error,   mtr_act_reset },};

/******************************************************************************
* Function Name : R_MTR_InitSequence
* Description   : Initialize variables for sequence control
* Arguments     : none
* Return Value  : none
******************************************************************************/
void R_MTR_InitSequence(void)
{
    g_u1_mode_system = MTR_MODE_STOP;                                   /* system mode update : stop mode */
}

/******************************************************************************
* Function Name : R_MTR_ExecEvent
* Description   : Change motor state and select action
* Arguments     : u1_event - event
* Return Value  : none
******************************************************************************/
void R_MTR_ExecEvent(uint8_t u1_event)
{
    uint8_t u1_mode_system;

    u1_mode_system = g_u1_mode_system;

#ifdef FW_ERR_DET_UNKNOWN
    if (u1_event >= MTR_SIZE_EVENT)                                     /* check unknown event */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else if (u1_mode_system >= MTR_SIZE_STATE)                          /* check unknown state */
    {
        g_u1_mode_system = MTR_MODE_ERROR;                              /* system mode update : error mode */

        DI();

        g_u1_error_status |= MTR_UNKNOWN_ERROR;                         /* error status update : unknown error */

        EI();
    }
    else
    {
        /* change motor state and select action */
        g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
    }
#else
    /* change motor state and select action */
    g_u1_mode_system = (gp_u1_def_action[u1_event][u1_mode_system])(g_u1_def_state[u1_event][u1_mode_system]);
#endif
}

/******************************************************************************
* Function Name : mtr_act_run
* Description   : Processing for motor control start
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_run(uint8_t u1_state)
{
    g_u1_flg_wait_stop = MTR_FLG_SET;

    mtr_start_init();                                                   /* initialize variables when motor control start */

    mtr_ctrl_start();                                                   /* motor control start */

    while (g_u1_flg_wait_stop == MTR_FLG_SET);                          /* waiting for motor stop */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_stop
* Description   : Processing for motor control stop
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_stop(uint8_t u1_state)
{
    g_u2_run_mode = MTR_BOOT_MODE;                                      /* boot mode */

    g_u1_sensorless_mode = MTR_SENLESS_INIT;                            /* initialize sensorless mode */
    g_u1_bemf_mode = MTR_BEMF_PRE;                                      /* initialize BEMF mode */
    g_u1_bemf_seq = MTR_BEMF_SEQ_INIT;                                  /* initialize BEMF sequence */

    mtr_ctrl_stop();                                                    /* TRD output disable, other interrupt settings, */
                                                                        /* motor output port is on/off for motor control stop */

    r_isw_wait(MTR_WAITTIME_25_MS);                                     /* 25[ms] */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_none
* Description   : Do Nothing
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_none(uint8_t u1_state)
{
    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_reset
* Description   : Processing for motor control reset
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_reset(uint8_t u1_state)
{
    g_u2_cnt_boot_mode = 0;
    g_u2_cnt_wait_stop = 0;
    g_u1_flg_wait_stop = MTR_FLG_CLR;

    g_u1_enable_write = 0;

    g_s2_vdc_ad = 0;
    g_s2_reci_vdc_ad = 0;
    g_s2_pdrv_ad = 0;

    g_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;
    g_s2_ref_speed_rad = 0;
    g_s2_speed_rad = 0;
    g_s2_speed_lpf_k = MTR_SPEED_LPF_K;                                 /* set MTR_SPEED_LPF_K */

#ifdef ICS_USE
/****** for ICS ******/
    g_u1_cnt_ics = 0;
/*********************/
#endif

    g_u2_run_mode = MTR_BOOT_MODE;                                      /* set MTR_BOOT_MODE */
    g_u1_error_status = 0;
    g_u1_mode_system = MTR_MODE_STOP;                                   /* set MTR_MODE_STOP */

    g_u1_v_pattern = 0;
    g_u2_cnt_timeout = 0;
    g_u1_direction = MTR_CW;                                            /* set MTR_CW */
    g_u2_less_timer_cnt = 0;
    g_u2_pre_less_timer_cnt = 0;
    g_s2_timer_cnt_ave = 0;
    g_u2_timer_cnt_buf[0] = 0;
    g_u2_timer_cnt_buf[1] = 0;
    g_u2_timer_cnt_buf[2] = 0;
    g_u2_timer_cnt_buf[3] = 0;
    g_u2_timer_cnt_buf[4] = 0;
    g_u2_timer_cnt_buf[5] = 0;
    g_u2_timer_cnt_num = 0;

    ics_input_buff.s2_ref_speed = 0;
    ics_input_buff.s2_direction = 0;
    ics_input_buff.s2_speed_lpf_k = 0;

    /* initialize variables for sensorless */
    g_u1_sensorless_mode = MTR_SENLESS_INIT;
    g_u1_bemf_mode = MTR_BEMF_PRE;
    g_u1_bemf_seq = MTR_BEMF_SEQ_INIT;
    g_s1_indsen_signal = 0;
    g_s2_bemf_ad[0][0] = 0;
    g_s2_bemf_ad[0][1] = 0;
    g_s2_bemf_ad[0][2] = 0;
    g_s2_bemf_ad[1][0] = 0;
    g_s2_bemf_ad[1][1] = 0;
    g_s2_bemf_ad[1][2] = 0;
    g_s2_bemf_ad[2][0] = 0;
    g_s2_bemf_ad[2][1] = 0;
    g_s2_bemf_ad[2][2] = 0;
    g_s2_bemf_ad_pre = 0;
    g_s2_bemf_ad_now = 0;
    g_s2_bemf_time_cnt = 0;
    g_s2_bemf_time_fg_calc_pre = 0;
    g_s2_bemf_time_fg_calc_now = 0;
    g_s2_bemf_time_a_cnt = 0;
    g_s2_bemf_time_a_cnt_pre = 0;
    g_s2_bemf_time_b_cnt = 0;
    g_s2_bemf_time_b_cnt_now = 0;
    g_s2_bemf_time_fg_cnt = 0;
    g_s2_bemf_fg_th_pre = 0;
    g_s2_bemf_fg_th_now = 0;
    g_s2_bemf_calc_pre = 0;
    g_s2_bemf_calc_now = 0;
    g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;
    g_s2_bemf_timeout_cnt = 0;
    g_s2_bemf_time_pwmchg_cnt = 0;
    g_u1_bemf_chk_hiz = 0;
    g_u1_bemf_chk_hiz_status = 0;
    g_u1_bemf_req_moten_on = 0;
    g_u1_bemf_req_moten_off = 0;
    g_u1_bemf_mode_cnt = 0;
    g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;
    g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
    g_s2_pdrv_ad_isense = 0;
    g_s2_bemf_fix_pwm_duty_cnt = 0;
    g_s2_bemf_timer_cnt_ave = 0;
    g_u1_bemf_limit_pwm_duty_flg = 0;
    g_u1_bemf_limit_vr_ad_flg = 0;
    g_u2_bemf_limit_vr_ad_cnt = 0;
    g_u1_bemf_limit_vr_ad = 0;

    /* initialize output port of BEMF measure */
    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_act_error
* Description   : Processing when detecting an error
* Arguments     : u1_state - state
* Return Value  : u1_state - state
******************************************************************************/
static uint8_t mtr_act_error(uint8_t u1_state)
{
    g_u2_run_mode = MTR_BOOT_MODE;                                      /* boot mode */

    g_u1_sensorless_mode = MTR_SENLESS_INIT;                            /* initialize sensorless mode */
    g_u1_bemf_mode = MTR_BEMF_PRE;                                      /* initialize BEMF mode */
    g_u1_bemf_seq = MTR_BEMF_SEQ_INIT;                                  /* initialize BEMF sequence */

    mtr_ctrl_stop();                                                    /* TRD output disable, other interrupt settings, */
                                                                        /* motor output port is on/off for motor control stop */

    return (u1_state);
}

/******************************************************************************
* Function Name : mtr_pattern_set
* Description   : Set voltage pattern
* Arguments     : u1_current_signal - value of current pattern
* Return Value  : none
******************************************************************************/
void mtr_pattern_set(uint8_t u1_current_signal)
{
    g_u2_cnt_timeout = 0;                                               /* clear counter for timeout error */

    mtr_speed_calc();                                                   /* speed calculation */

    /*** set voltage pattern ***/
    /*** complementary first half 60 degree chopping ***/
    if (g_u1_direction == MTR_CW)                                       /* check rotational direction */
    {
        switch (u1_current_signal)
        {
            case MTR_PATTERN_CW_W_V:                                    /* from W phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_UN_ON;
            break;

            case MTR_PATTERN_CW_U_V:                                    /* from U phase to V phase */
                g_u1_v_pattern = MTR_WP_ON_V_PWM;
            break;

            case MTR_PATTERN_CW_U_W:                                    /* from U phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_VN_ON;
            break;

            case MTR_PATTERN_CW_V_W:                                    /* from V phase to W phase */
                g_u1_v_pattern = MTR_UP_ON_W_PWM;
            break;

            case MTR_PATTERN_CW_V_U:                                    /* from V phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_WN_ON;
            break;

            case MTR_PATTERN_CW_W_U:                                    /* from W phase to U phase */
                g_u1_v_pattern = MTR_VP_ON_U_PWM;
            break;

            default:                                                    /* current signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else if (g_u1_direction == MTR_CCW)                                 /* check rotational direction */
    {
        switch (u1_current_signal)
        {
            case MTR_PATTERN_CCW_W_V:                                   /* from W phase to V phase */
                g_u1_v_pattern = MTR_WP_ON_U_PWM;
            break;

            case MTR_PATTERN_CCW_W_U:                                   /* from W phase to U phase */
                g_u1_v_pattern = MTR_V_PWM_UN_ON;
            break;

            case MTR_PATTERN_CCW_V_U:                                   /* from V phase to U phase */
                g_u1_v_pattern = MTR_VP_ON_W_PWM;
            break;

            case MTR_PATTERN_CCW_V_W:                                   /* from V phase to W phase */
                g_u1_v_pattern = MTR_U_PWM_WN_ON;
            break;

            case MTR_PATTERN_CCW_U_W:                                   /* from U phase to W phase */
                g_u1_v_pattern = MTR_UP_ON_V_PWM;
            break;

            case MTR_PATTERN_CCW_U_V:                                   /* from U phase to V phase */
                g_u1_v_pattern = MTR_W_PWM_VN_ON;
            break;

            default:                                                    /* current signal pattern error */
                g_u1_v_pattern = MTR_PATTERN_ERROR;
            break;
        }
    }
    else
    {
        /* Do Nothing */
    }

    if (g_u1_mode_system != MTR_MODE_ERROR)                             /* check system mode */
    {
        mtr_change_pattern(g_u1_v_pattern);                             /* change voltage pattern */
    }
}

/******************************************************************************
* Function Name : mtr_convert_indsenpat_to_pwmpat
* Description   : Change voltage pattern
* Arguments     : u1_temp - Voltage pattern
* Return Value  : none
******************************************************************************/
uint8_t mtr_convert_indsenpat_to_pwmpat(uint8_t u1_indsenpat)
{
    uint8_t u1_pat;
    uint8_t u1_temp;

    u1_pat = g_u1_pwm_pattarn[g_u1_direction][u1_indsenpat];

    if (g_u1_direction == MTR_CW)                                       /* check rotational direction */
    {
        switch (u1_pat)
        {
            case MTR_PATTERN_W_V:
                u1_temp = MTR_PATTERN_CW_V_W;
            break;

            case MTR_PATTERN_W_U:
                u1_temp = MTR_PATTERN_CW_U_W;
            break;

            case MTR_PATTERN_V_U:
                u1_temp = MTR_PATTERN_CW_U_V;
            break;

            case MTR_PATTERN_V_W:
                u1_temp = MTR_PATTERN_CW_W_V;
            break;

            case MTR_PATTERN_U_W:
                u1_temp = MTR_PATTERN_CW_W_U;
            break;

            case MTR_PATTERN_U_V:
                u1_temp = MTR_PATTERN_CW_V_U;
            break;

            default:
            break;
        }
    }
    else if (g_u1_direction == MTR_CCW)                                 /* check rotational direction */
    {
        switch (u1_pat)
        {
            case MTR_PATTERN_W_V:
                u1_temp = MTR_PATTERN_CCW_V_W;
            break;

            case MTR_PATTERN_W_U:
                u1_temp = MTR_PATTERN_CCW_U_W;
            break;

            case MTR_PATTERN_V_U:
                u1_temp = MTR_PATTERN_CCW_U_V;
            break;

            case MTR_PATTERN_V_W:
                u1_temp = MTR_PATTERN_CCW_W_V;
            break;

            case MTR_PATTERN_U_W:
                u1_temp = MTR_PATTERN_CCW_W_U;
            break;

            case MTR_PATTERN_U_V:
                u1_temp = MTR_PATTERN_CCW_V_U;
            break;

            default:
            break;
        }
    }
    else
    {
        /* Do Nothing */
    }

    return (u1_temp);
}

/******************************************************************************
* Function Name : mtr_get_bemf_threshold
* Description   : get BEMF threshold
* Arguments     : s2_bemf_th - BEMF threshold
* Return Value  : none
******************************************************************************/
int16_t mtr_get_bemf_threshold(void)
{
    int16_t s2_bemf_th = 0;

    if (MTR_PORT_BEMF_U == 1)
    {
        /* U BEMF Threshold */
        s2_bemf_th = g_s2_bemf_ad[g_u1_bemf_th_type][MTR_BEMF_HIZ_U];
    }
    else if (MTR_PORT_BEMF_V == 1)
    {
        /* V BEMF Threshold */
        s2_bemf_th = g_s2_bemf_ad[g_u1_bemf_th_type][MTR_BEMF_HIZ_V];
    }
    else
    {
        /* W BEMF Threshold */
        s2_bemf_th = g_s2_bemf_ad[g_u1_bemf_th_type][MTR_BEMF_HIZ_W];
    }

    return (s2_bemf_th);
}

/******************************************************************************
* Function Name : mtr_get_bemf_threshold_direct
* Description   : get BEMF threshold
* Arguments     : g_s2_bemf_ad - BEMF threshold
* Return Value  : none
******************************************************************************/
int16_t mtr_get_bemf_threshold_direct(uint8_t u1_th, uint8_t u1_hiz)
{
    return (g_s2_bemf_ad[u1_th][u1_hiz]);
}

/******************************************************************************
* Function Name : mtr_get_bemf_dir
* Description   : get BEMF direction
* Arguments     : g_u1_bemf_dir -  BEMF direction
* Return Value  : none
******************************************************************************/
uint8_t mtr_get_bemf_dir(void)
{
    return (g_u1_bemf_dir[g_u1_direction][g_s1_indsen_signal]);
}

/******************************************************************************
* Function Name : mtr_set_predrv_isense
* Description   : set detection of current
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_set_predrv_isense(void)
{
    R_PreDrv_Set_AdcSel(REQ_ADC_SEL_ISEN);                              /* set ADC_SEL */

    g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
}

/******************************************************************************
* Function Name : mtr_set_predrv_bemf
* Description   : set detection of BEMF
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_set_predrv_bemf(void)
{
    R_PreDrv_Set_AdcSel(REQ_ADC_SEL_BEMF);                              /* set ADC_SEL */

    g_u1_bemf_ad_type = MTR_BEMF_AD_BEMF;
}

/******************************************************************************
* Function Name : mtr_speed_calc
* Description   : Calculate motor speed
* Arguments     : none
* Return Value  : none
******************************************************************************/
static void mtr_speed_calc(void)
{
    int16_t  s2_temp;
    uint16_t u2_temp;

    g_u2_less_timer_cnt = MTR_TAU1_CNT;                                 /* get value of timer counter (TCR01) */

    u2_temp = (g_u2_pre_less_timer_cnt - g_u2_less_timer_cnt);          /* calculate timer count in 60 degrees */

    if (g_u2_run_mode != MTR_BOOT_MODE)                                 /* check run mode */
    {   /*** average 6 times of motor speed timer ***/
        g_s2_timer_cnt_ave -= g_u2_timer_cnt_buf[g_u2_timer_cnt_num];
        g_u2_timer_cnt_buf[g_u2_timer_cnt_num] = u2_temp;
        g_s2_timer_cnt_ave += g_u2_timer_cnt_buf[g_u2_timer_cnt_num];

        g_u2_timer_cnt_num++;

        if (g_u2_timer_cnt_num > (MTR_TIMER_CNT_BUF_NUM - 1))
        {
            g_u2_timer_cnt_num = 0;
        }

        /* *** low speed limit *** */
        if (g_s2_timer_cnt_ave < 0)
        {
            g_s2_timer_cnt_ave = 0x7FFF;
        }

        if (g_u2_timer_cnt_buf[MTR_TIMER_CNT_BUF_NUM - 1] != 0)
        {
            /*** check overflow and calculate RPM & low pass filter ***/
            if (g_s2_timer_cnt_ave > MTR_SPEED_CALC_BASE)
            {
                s2_temp = R_FIX_div_int16(MTR_SPEED_CALC_BASE, g_s2_timer_cnt_ave);
                s2_temp = s2_temp - g_s2_speed_rad;
                g_s2_speed_rad += R_FIX14_mul_int16(g_s2_speed_lpf_k, s2_temp);
                g_s2_bemf_timer_cnt_ave = g_s2_timer_cnt_ave;
            }
#ifdef FW_ERR_DET_OVERSPEED
            else
            {
                if (g_u2_run_mode == MTR_LESS_120_MODE)                 /* check run mode */
                {
                    R_MTR_ExecEvent(MTR_EVENT_ERROR);                   /* execute error event */

                    DI();

                    g_u1_error_status |= MTR_OVER_SPEED_ERROR;          /* error status update : over speed error */

                    EI();
                }
            }
#endif
        }
    }

    g_u2_pre_less_timer_cnt = g_u2_less_timer_cnt;                      /* keep timer counter value */
}

/******************************************************************************
* Function Name : mtr_start_init
* Description   : Initialize variables when motor control start
* Arguments     : none
* Return Value  : none
******************************************************************************/
static void mtr_start_init(void)
{
    int16_t s2_temp;

    g_u2_cnt_boot_mode = 0;
    g_u2_cnt_wait_stop = 0;
    g_u1_flg_wait_stop = MTR_FLG_CLR;

    DI();

    s2_temp = mtr_get_adc(MTR_ADCCH_VM);

    EI();

    g_s2_vdc_ad = R_FIX8_mul_int16(s2_temp, MTR_VDC_SCALING);
    g_s2_reci_vdc_ad = R_FIX_div_int16(MTR_RECIVDC_SCALING, g_s2_vdc_ad);

    /* duty 30% */
    g_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;

    g_s2_speed_rad = 0;

    g_u2_cnt_timeout = 0;
    g_u2_less_timer_cnt = 0;
    g_u2_pre_less_timer_cnt = 0;
    g_s2_timer_cnt_ave = 0;
    g_u2_timer_cnt_buf[0] = 0;
    g_u2_timer_cnt_buf[1] = 0;
    g_u2_timer_cnt_buf[2] = 0;
    g_u2_timer_cnt_buf[3] = 0;
    g_u2_timer_cnt_buf[4] = 0;
    g_u2_timer_cnt_buf[5] = 0;
    g_u2_timer_cnt_num = 0;

    /* initialize variables for sensorless */
    g_u1_sensorless_mode = MTR_SENLESS_INDSEN;
    g_u1_bemf_mode = MTR_BEMF_PRE;
    g_u1_bemf_seq = MTR_BEMF_SEQ_INIT;
    g_s1_indsen_signal = 0;
    g_s2_bemf_ad_pre = 0;
    g_s2_bemf_ad_now = 0;
    g_s2_bemf_time_cnt = 0;
    g_s2_bemf_time_fg_calc_pre = 0;
    g_s2_bemf_time_fg_calc_now = 0;
    g_s2_bemf_time_a_cnt = 0;
    g_s2_bemf_time_a_cnt_pre = 0;
    g_s2_bemf_time_b_cnt = 0;
    g_s2_bemf_time_b_cnt_now = 0;
    g_s2_bemf_time_fg_cnt = 0;
    g_s2_bemf_fg_th_pre = 0;
    g_s2_bemf_fg_th_now = 0;
    g_s2_bemf_calc_pre = 0;
    g_s2_bemf_calc_now = 0;
    g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;
    g_s2_bemf_timeout_cnt = 0;
    g_s2_bemf_time_pwmchg_cnt = 0;
    g_u1_bemf_chk_hiz = 0;
    g_u1_bemf_chk_hiz_status = 0;
    g_u1_bemf_req_moten_on = 0;
    g_u1_bemf_req_moten_off = 0;
    g_u1_bemf_mode_cnt = 0;
    g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;
    g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
    g_s2_pdrv_ad_isense = 0;
    g_s2_bemf_fix_pwm_duty_cnt = 0;
    g_s2_bemf_timer_cnt_ave = 0;
    g_u1_bemf_limit_pwm_duty_flg = 0;
    g_u1_bemf_limit_vr_ad_flg = 0;
    g_u2_bemf_limit_vr_ad_cnt = 0;
    g_u1_bemf_limit_vr_ad = 0;

    mtr_set_predrv_bemf();                                              /* set detection of BEMF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    DI();

    /* get BEMF threshold for high side regeneration*/
    r_isw_set_port(IS_PORT_P_ON);                                       /* set voltage pattern (high side on) */

    MTR_PORT_BEMF_U = 1;                                                /* BEMF measure(U) ON */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HSIDE][MTR_BEMF_HIZ_U] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 1;                                                /* BEMF measure(V) ON */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HSIDE][MTR_BEMF_HIZ_V] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 1;                                                /* BEMF measure(W) ON */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HSIDE][MTR_BEMF_HIZ_W] = mtr_get_adc(MTR_ADCCH_PDRV);

    /* get BEMF threshold for Hi-Z */
    r_isw_set_port(IS_PORT_OFF);                                        /* set voltage pattern (all off) */

    MTR_PORT_BEMF_U = 1;                                                /* BEMF measure(U) ON */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HIZ][MTR_BEMF_HIZ_U] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 1;                                                /* BEMF measure(V) ON */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HIZ][MTR_BEMF_HIZ_V] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 1;                                                /* BEMF measure(W) ON */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_HIZ][MTR_BEMF_HIZ_W] = mtr_get_adc(MTR_ADCCH_PDRV);

    /* get BEMF threshold for low side regeneration */
    r_isw_set_port(IS_PORT_N_ON);                                       /* set voltage pattern (low side on) */

    MTR_PORT_BEMF_U = 1;                                                /* BEMF measure(U) ON */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_LSIDE][MTR_BEMF_HIZ_U] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 1;                                                /* BEMF measure(V) ON */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_LSIDE][MTR_BEMF_HIZ_V] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 1;                                                /* BEMF measure(W) ON */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */

    g_s2_bemf_ad[MTR_BEMF_TH_LSIDE][MTR_BEMF_HIZ_W] = mtr_get_adc(MTR_ADCCH_PDRV);

    MTR_PORT_BEMF_U = 0;                                                /* BEMF measure(U) OFF */
    MTR_PORT_BEMF_V = 0;                                                /* BEMF measure(V) OFF */
    MTR_PORT_BEMF_W = 0;                                                /* BEMF measure(W) OFF */

    r_isw_set_port(IS_PORT_OFF);                                        /* set voltage pattern (all off) */

    EI();

    mtr_set_predrv_isense();                                            /* set detection of current */

    r_isw_wait(MTR_WAITTIME_100_US);                                    /* 100[us] */
}

/******************************************************************************
* Function Name : mtr_set_variables
* Description   : Set variables (conversion of unit)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_set_variables(void)
{
    int16_t s2_temp;

    /* reference speed */
    s2_temp = R_FIX12_mul_int16(MTR_RPM_RAD, ics_input_buff.s2_ref_speed);
    g_s2_ref_speed_rad = s2_temp * MTR_POLE_PAIRS;                      /* [rpm]->[rad/s] */

    /* rotation direction */
    g_u1_direction = (uint8_t)ics_input_buff.s2_direction;

    /* speed lpf parameter */
    g_s2_speed_lpf_k = ics_input_buff.s2_speed_lpf_k;
}

/******************************************************************************
* Function Name : R_MTR_IcsInput
* Description   : Set value to buffer
* Arguments     : ICS input structure
* Return Value  : none
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *ics_input)
{
    /* reference speed */
    ics_input_buff.s2_ref_speed = ics_input->s2_ref_speed;

    /* rotation direction */
    ics_input_buff.s2_direction = ics_input->s2_direction;

    /* speed lpf parameter */
    ics_input_buff.s2_speed_lpf_k = ics_input->s2_speed_lpf_k;

    g_u1_enable_write = MTR_FLG_SET;
}

/******************************************************************************
* Function Name : R_MTR_SetSpeed
* Description   : Set speed reference
* Arguments     : ref_speed - Speed reference[rpm]
* Return Value  : none
******************************************************************************/
void R_MTR_SetSpeed(int16_t ref_speed)
{
    int16_t s2_temp;

    s2_temp = R_FIX12_mul_int16(MTR_RPM_RAD, ref_speed);
    g_s2_ref_speed_rad = s2_temp * MTR_POLE_PAIRS;
}

/******************************************************************************
* Function Name : R_MTR_SetDir
* Description   : Set rotational direction
* Arguments     : dir - Rotational direction
* Return Value  : none
******************************************************************************/
void R_MTR_SetDir(uint8_t dir)
{
    g_u1_direction = dir;
}

/******************************************************************************
* Function Name : R_MTR_GetSpeed
* Description   : Get speed value
* Arguments     : none
* Return Value  : s2_speed_rpm - Speed
******************************************************************************/
int16_t R_MTR_GetSpeed(void)
{
    int16_t s2_speed_rpm;

    s2_speed_rpm = R_FIX10_mul_int16(MTR_RAD_RPM, g_s2_speed_rad);

    return (s2_speed_rpm);
}

/******************************************************************************
* Function Name : R_MTR_GetDir
* Description   : Get rotation direction
* Arguments     : none
* Return Value  : g_u1_direction - Rotational direction
******************************************************************************/
uint8_t R_MTR_GetDir(void)
{
    return (g_u1_direction);
}

/******************************************************************************
* Function Name : R_MTR_GetStatus
* Description   : Get status of motor control system
* Arguments     : none
* Return Value  : g_u1_mode_system - Motor status
******************************************************************************/
uint8_t R_MTR_GetStatus(void)
{
    return (g_u1_mode_system);
}

/******************************************************************************
* Function Name : mtr_error_check
* Description   : Check error status
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_error_check(void)
{
    /*==================================*/
    /*     over voltage error check     */
    /*==================================*/

#ifdef FW_ERR_DET_OVERVOLTAGE
    if (g_s2_vdc_ad > MTR_OVERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_VOLTAGE_ERROR;                    /* error status update : over voltage error */

        EI();
    }
#endif

    /*===================================*/
    /*     under voltage error check     */
    /*===================================*/

#ifdef FW_ERR_DET_UNDERVOLTAGE
    if (g_s2_vdc_ad < MTR_UNDERVOLTAGE_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_UNDER_VOLTAGE_ERROR;                   /* under voltage error */

        EI();
    }
#endif

    /*================================*/
    /*     over speed error check     */
    /*================================*/

#ifdef FW_ERR_DET_OVERSPEED
    if (g_s2_speed_rad > MTR_SPEED_LIMIT)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        DI();

        g_u1_error_status |= MTR_OVER_SPEED_ERROR;                      /* error status update : over speed error */

        EI();
    }
#endif

    /*===============================================================*/
    /*     timeout error check (undetected changing pwm pattern)     */
    /*===============================================================*/

#ifdef FW_ERR_DET_TIMEOUT
    if (g_u2_run_mode == MTR_LESS_120_MODE)                             /* check run mode */
    {
        if (g_u1_mode_system == MTR_MODE_RUN)                           /* check system mode */
        {
            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                g_u2_cnt_timeout++;
            }
            else
            {
                g_u2_cnt_timeout = 0;
            }
        }
        else
        {
            g_u2_cnt_timeout = 0;
        }

        if (g_u2_cnt_timeout > MTR_TIMEOUT_CNT)                         /* check undetected time */
        {
            R_MTR_ExecEvent(MTR_EVENT_ERROR);                           /* execute error event */

            DI();

            g_u1_error_status |= MTR_TIMEOUT_ERROR;                     /* error status update : timeout error */

            EI();
        }
    }
#endif
}

