/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_predrv.c
* Version      : 
* Device(s)    : 
* Tool-Chain   : 
* Description  : This file implements device driver for pre-driver module.
* Creation Date: 2016/07/05
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "mtr_common.h"
#include "r_cg_macrodriver.h"
#include "r_cg_predrv_reg.h"
#include "r_cg_predrv.h"
#include "r_cg_predrv_prm.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
volatile uint8_t g_u1_predriver_error;                                  /* error status */
static void wait(uint16_t count);

extern uint8_t com_u1_pdrvreg_vreg5_trim_now;
extern uint8_t com_u1_pdrvreg_csamp_trim_now;

extern uint8_t com_u1_pdrvreg_trim_en_now;

extern uint8_t com_u1_pdrvreg_bgr_trim_now;
extern uint8_t com_u1_pdrvreg_bfamp_trim_now;

/***********************************************************************************************************************
* Function Name: predriver_hw_init
* Description  : 
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void predriver_hw_init(void)
{
    EI();                                                               /* interrupt enable */

    /* Init Sequence */
    R_PREDRV_InitSequence();

    DI();                                                               /* interrupt disable */
}

/***********************************************************************************************************************
* Function Name: R_PREDRV_TRIM_Create
* Description  : 
* Arguments    : None
* Return Value : 
***********************************************************************************************************************/
MD_STATUS R_PREDRV_TRIM_Create(void)
{
    MD_STATUS status = MD_OK;

    uint8_t u1_temp[3];                                                 /* trimming data from flash */
    uint8_t u1_data_status = FALSE;                                     /* invalid data */

    /* BFA3 flash */
    u1_temp[0] = *(__far const uint8_t*)(0xEFFECU);                     /* VREG5, CSAMP_TRIM */
    u1_temp[1] = *(__far const uint8_t*)(0xEFFEDU);                     /* BGR_TRIM */
    u1_temp[2] = *(__far const uint8_t*)(0xEFFEEU);                     /* BFAMP_TRIM */

    com_u1_pdrvreg_vreg5_trim_now = INIT_ICS_VREG5_TRIM;
    com_u1_pdrvreg_csamp_trim_now = INIT_ICS_CSAMP_TRIM;

    com_u1_pdrvreg_trim_en_now    = INIT_ICS_TRIM_EN;

    com_u1_pdrvreg_bgr_trim_now   = INIT_ICS_BGR_TRIM;
    com_u1_pdrvreg_bfamp_trim_now = INIT_ICS_BFAMP_TRIM;

    if ((u1_temp[1] == 0xFFU) && (u1_temp[2] == 0x24U))
    {
        /* invalid data */
        u1_temp[0] = 0x00U;
    }
    else
    {
        /* valid data */
        u1_data_status = TRUE;
    }

    if (u1_data_status == TRUE)
    {
        status = R_PreDrvReg_Write(PREDRVREG_TRIM_PT, INIT_TRIM_PT_UP);

        if (status == MD_OK)
        {
            com_u1_pdrvreg_bgr_trim_now = INIT_BGR_TRIM | (u1_temp[1] & MASK_BGR_TRIM);

            status = R_PreDrvReg_Write(PREDRVREG_BGR_TRIM, com_u1_pdrvreg_bgr_trim_now);
        }

        if (status == MD_OK)
        {
            com_u1_pdrvreg_bfamp_trim_now = INIT_BFAMP_TRIM | ((u1_temp[2] << 1) & MASK_BFAMP_TRIM);

            status = R_PreDrvReg_Write(PREDRVREG_BFAMP_TRIM, com_u1_pdrvreg_bfamp_trim_now);
        }

        if (status == MD_OK)
        {
            com_u1_pdrvreg_trim_en_now = INIT_TRIM_EN_EFWD;

            status = R_PreDrvReg_Write(PREDRVREG_TRIM_EN, com_u1_pdrvreg_trim_en_now);
        }

        if (status == MD_OK)
        {
            status = R_PreDrvReg_Write(PREDRVREG_TRIM_PT, INIT_TRIM_PT);
        }
    }

    if (status == MD_OK)
    {
        com_u1_pdrvreg_vreg5_trim_now = INIT_VREG5_TRIM | ((u1_temp[0] & MASK_VREG5_TRIM) >> 4);

        status = R_PreDrvReg_Write(PREDRVREG_VREG5_TRIM, com_u1_pdrvreg_vreg5_trim_now);
    }

    if (status == MD_OK)
    {
        com_u1_pdrvreg_csamp_trim_now = INIT_CSAMP_TRIM | (u1_temp[0] & MASK_CSAMP_TRIM);

        status = R_PreDrvReg_Write(PREDRVREG_CSAMP_TRIM, com_u1_pdrvreg_csamp_trim_now);
    }

    if (status == MD_OK)
    {
        status = R_PreDrvReg_Write(PREDRVREG_CP_TRIM, INIT_CP_TRIM);
    }

    return (status);
}

/***********************************************************************************************************************
* Function Name: R_PREDRV_InitSequence
* Description  : 
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_PREDRV_InitSequence(void)
{
    MD_STATUS status = MD_OK;

    uint8_t seq     = SEQ_INIT;
    uint8_t endflag = 0;
    uint8_t data    = 0;
    uint8_t count   = 0;
    uint8_t error   = PREDRV_NORMAL;

    PREDRV_PRM reg;

    do
    {
        switch (seq)
        {
            case SEQ_INIT:
                g_u1_predriver_error = PREDRV_NORMAL;
                seq = SEQ_CHK_SPI;
            break;

            case SEQ_CHK_SPI:
                /* *** check SPI *** */
                status = R_PreDrvReg_Read(PREDRVREG_WHO_AM_I, &data);

                if (data == PREDRV_SPI_ACCESS_OK)
                {
                    /* OK */
                    seq = SEQ_CHK_TSD_N;
                }
                else
                {
                    if ((count++) > SPI_CHK_MAX)
                    {
                        /* NG */
                        error = PREDRV_SPI_ERROR;
                        seq   = SEQ_END;
                    }
                }
            break;

            case SEQ_CHK_TSD_N:
                /* *** check ALARM Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &data);

                if (((~data & ALMSTS1_TSD_N) == ALMSTS1_TSD_N))
                {
                    /* error */
                    error = PREDRV_ALARM_ERROR;
                    seq   = SEQ_END;
                }
                else
                {
                    /* no error */
                    seq   = SEQ_SET_HALL_SIG_PRM;
                }
            break;

            case SEQ_SET_HALL_SIG_PRM:
                /* *** set Hall Signal Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_HALL_SIG, INIT_HALL_SIG);

                seq = SEQ_SET_5VTRIM;
            break;

            case SEQ_SET_5VTRIM:
                /* *** set 5V Trimming Data *** */
                status = R_PREDRV_TRIM_Create();

                seq = SEQ_SET_ALMOPE1_PRM;
            break;

            case SEQ_SET_ALMOPE1_PRM:
                /* *** set ALARM Operation Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_ALMOPE1, INIT_ALMOPE1);

                seq = SEQ_SET_ALMOUT1_PRM;
            break;

            case SEQ_SET_ALMOUT1_PRM:
                /* *** set ALARM Pin Output Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_ALMOUT1, INIT_ALMOUT1);

                seq = SEQ_SET_CS_SET2_PRM;
            break;

            case SEQ_SET_CS_SET2_PRM:
                /* *** set Current Sense Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_CS_SET2, INIT_CS_SET2);

                seq = SEQ_SET_CS_SET1_PRM;
            break;

            case SEQ_SET_CS_SET1_PRM:
                /* *** set Current Sense Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_CS_SET1, INIT_CS_SET1);

                seq = SEQ_SET_PS_ALL_PRM;

#ifdef CS_SET1_TO_SEQINIT
                count = 0;
                seq = SEQ_SET_SEQINIT_PRM;
#endif
            break;

            case SEQ_SET_SEQINIT_PRM:
                /* *** init Sequence Parameter *** */
                reg.adr = predrv_reg_seqinit_parameter[count].adr;
                reg.prm = predrv_reg_seqinit_parameter[count].prm;

                status = R_PreDrvReg_Write(reg.adr, reg.prm);

                if ((count++) >= PREDRVREG_SEQINIT_RPM_NUM)
                {
                    count = 0;
                    seq   = SEQ_SET_PS_ALL_PRM;
                }
            break;

            case SEQ_SET_PS_ALL_PRM:
                /* *** set Power Save Control *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS_ALL, INIT_PS_ALL);

                seq = SEQ_SET_PS_1ST_PRM;
            break;

            case SEQ_SET_PS_1ST_PRM:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, INIT_PS_1ST);

                /* *** wait 1[ms] *** */
                wait(WAITTIME_1_MS);

                seq = SEQ_SET_PS_2ND_PRM;
            break;

            case SEQ_SET_PS_2ND_PRM:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, INIT_PS_2ND);

                /* *** wait 3[ms] *** */
                wait(WAITTIME_3_MS);

                seq = SEQ_CHK_ALMRAW1;
            break;

            case SEQ_CHK_ALMRAW1:
                /* *** check ALARM Raw Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMRAW1, &data);

                data &= ALMSTS1_VGT_UVP_MASK;                           /* ALARM factor(VGT_UVP) mask */

                if (data != PREDRV_ALMRAW1_OK)
                {
                    /* error */
                    seq = SEQ_CHK_ALMRAW1;
                }
                else
                {
                    /* no error */
                    seq = SEQ_SET_PS_3RD_PRM;
                }
            break;

            case SEQ_SET_PS_3RD_PRM:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, INIT_PS_3RD);

                seq = SEQ_CHK_ALMSTS;
            break;

            case SEQ_CHK_ALMSTS:
                /* *** check ALARM Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &data);

                if (status == MD_OK)
                {
                    data &= ALMSTS1_VGT_UVP_MASK;                       /* ALARM factor(VGT_UVP) mask */

                    if (data != ALMSTS1_NO_ERROR)
                    {
                        /* error */
                        error = PREDRV_ALARM_ERROR;
                        seq   = SEQ_END;
                    }
                    else
                    {
                        /* *** check ALARM Status *** */
                        status = R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &data);

                        if (data != ALMSTS2_NO_ERROR)
                        {
                            /* error */
                            error = PREDRV_ALARM_ERROR;
                            seq   = SEQ_END;
                        }
                        else
                        {
                            /* no error */
                            seq   = SEQ_SET_MOT_EN;
                        }
                    }
                }
            break;

            case SEQ_SET_MOT_EN:
                /* *** enable Motor Drive Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_DRIVE_SET, INIT_DRIVE_SET);

                seq = SEQ_END;
            break;

            case SEQ_END:
                seq = SEQ_END;
                endflag = 1;
            break;

            default:
            break;
        }

        if (status != MD_OK)
        {
            error = PREDRV_REGRW_ERROR;
            seq   = SEQ_END;
            endflag = 1;
        }
    } while (endflag == 0);

    g_u1_predriver_error = error;
}

/***********************************************************************************************************************
* Function Name: R_PREDRV_PARAM_Create
* Description  : 
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_PREDRV_PARAM_Create(void)
{
    uint8_t i;

    PREDRV_PRM reg;

    for (i = 0; i < PREDRVREG_NUM_MAX; i++)
    {
        reg.adr = predrv_reg_init_parameter[i].adr;
        reg.prm = predrv_reg_init_parameter[i].prm;

        R_PreDrvReg_Write(reg.adr, reg.prm);
    }
}

/***********************************************************************************************************************
* Function Name: R_PREDRV_ErrorRecoverySequence
* Description  : 
* Arguments    : 
* Return Value : None
***********************************************************************************************************************/
void R_PREDRV_ErrorRecoverySequence(uint8_t almsts1, uint8_t almsts2)
{
    MD_STATUS status = MD_OK;

    uint8_t seq     = ERR_RCV_SEQ_INIT;
    uint8_t endflag = 0;
    uint8_t data    = 0;
    uint8_t error   = PREDRV_NORMAL;

    do
    {
        switch (seq)
        {
            case ERR_RCV_SEQ_INIT:
                g_u1_predriver_error = PREDRV_NORMAL;
                seq = ERR_RCV_SEQ_CHK_STS;
            break;

            case ERR_RCV_SEQ_CHK_STS:
                if (((~almsts1 & ALMSTS1_TSD_N    ) == ALMSTS1_TSD_N    ) ||
                    ((~almsts1 & ALMSTS1_OCP_N    ) == ALMSTS1_OCP_N    ) ||
                    ((~almsts1 & ALMSTS1_VGB_UVP_N) == ALMSTS1_VGB_UVP_N)
                                                                             )
                {
                    /* error */
                    error = PREDRV_ALARM_ERROR;
                    seq   = ERR_RCV_SEQ_CLR_MOT_EN;
                }
                else
                {
                    if (almsts2 != ALMSTS2_NO_ERROR)
                    {
                        /* error */
                        error = PREDRV_ALARM_ERROR;
                        seq   = ERR_RCV_SEQ_CLR_MOT_EN;
                    }
                    else
                    {
                        /* no error */
                        seq   = ERR_RCV_SEQ_END;
                    }
                }
            break;

            case ERR_RCV_SEQ_CLR_MOT_EN:
                /* *** disable Motor Drive Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_DRIVE_SET, ERRRCV_MOT_EN_CLR);

                seq = ERR_RCV_SEQ_SET_PS_1ST;
            break;

            case ERR_RCV_SEQ_SET_PS_1ST:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, ERRRCV_PS_1ST);

                seq = ERR_RCV_SEQ_CHK_ALMSTS_1ST;
            break;

            case ERR_RCV_SEQ_CHK_ALMSTS_1ST:
                /* *** check ALARM Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &data);

                if (status == MD_OK)
                {
                    if (((~data & ALMSTS1_TSD_N    ) == ALMSTS1_TSD_N    ) ||
                        ((~data & ALMSTS1_OCP_N    ) == ALMSTS1_OCP_N    ) ||
                        ((~data & ALMSTS1_VGB_UVP_N) == ALMSTS1_VGB_UVP_N)
                                                                              )
                    {
                        /* error */
                        seq = ERR_RCV_SEQ_CHK_ALMSTS_1ST;
                    }
                    else
                    {
                        /* *** check ALARM Status *** */
                        status = R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &data);

                        if (data != ALMSTS2_NO_ERROR)
                        {
                            /* error */
                            seq = ERR_RCV_SEQ_CHK_ALMSTS_1ST;
                        }
                        else
                        {
                            /* no error */
                            error = PREDRV_NORMAL;
                            seq   = ERR_RCV_SEQ_SET_ALM_LATCH_CLR;
                        }
                    }
                }
            break;

            case ERR_RCV_SEQ_SET_ALM_LATCH_CLR:
                /* *** set Motor Drive Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_DRIVE_SET, ERRRCV_ALM_LATCH_CLR);

                seq = ERR_RCV_SEQ_SET_PS_2ND;
            break;

            case ERR_RCV_SEQ_SET_PS_2ND:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, ERRRCV_PS_2ND);

                /* *** wait 1[ms] *** */
                wait(WAITTIME_1_MS);

                seq = ERR_RCV_SEQ_SET_PS_3RD;
            break;

            case ERR_RCV_SEQ_SET_PS_3RD:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, ERRRCV_PS_3RD);

                /* *** wait 3[ms] *** */
                wait(WAITTIME_3_MS);

                seq = ERR_RCV_SEQ_CHK_ALMRAW1;
            break;

            case ERR_RCV_SEQ_CHK_ALMRAW1:
                /* *** check ALARM Raw Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMRAW1, &data);

                data &= ALMSTS1_VGT_UVP_MASK;                           /* ALARM factor(VGT_UVP) mask */

                if (data != PREDRV_ALMRAW1_OK)
                {
                    /* error */
                    seq = ERR_RCV_SEQ_CHK_ALMRAW1;
                }
                else
                {
                    /* no error */
                    seq = ERR_RCV_SEQ_SET_PS_4TH;
                }
            break;

            case ERR_RCV_SEQ_SET_PS_4TH:
                /* *** set Power Save Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_PS, ERRRCV_PS_4TH);

                seq = ERR_RCV_SEQ_CHK_ALMSTS_2ND;
            break;

            case ERR_RCV_SEQ_CHK_ALMSTS_2ND:
                /* *** check ALARM Status *** */
                status = R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &data);

                if (status == MD_OK)
                {
                    data &= ALMSTS1_VGT_UVP_MASK;                       /* ALARM factor(VGT_UVP) mask */

                    if (data != ALMSTS1_NO_ERROR)
                    {
                        /* error */
                        error = PREDRV_ALARM_ERROR;
                        seq   = ERR_RCV_SEQ_END;
                    }
                    else
                    {
                        /* *** check ALARM Status *** */
                        status = R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &data);

                        if (data != ALMSTS2_NO_ERROR)
                        {
                            /* error */
                            error = PREDRV_ALARM_ERROR;
                            seq   = ERR_RCV_SEQ_END;
                        }
                        else
                        {
                            /* no error */
                            seq   = ERR_RCV_SEQ_SET_MOT_EN;
                        }
                    }
                }
            break;

            case ERR_RCV_SEQ_SET_MOT_EN:
                /* *** enable Motor Drive Control Setting *** */
                status = R_PreDrvReg_Write(PREDRVREG_DRIVE_SET, ERRRCV_MOT_EN_SET);

                seq = ERR_RCV_SEQ_END;
            break;

            case ERR_RCV_SEQ_END:
                seq = ERR_RCV_SEQ_END;
                endflag = 1;
            break;

            default:
            break;
        }

        if (status != MD_OK)
        {
            error = PREDRV_REGRW_ERROR;
            seq   = ERR_RCV_SEQ_END;
            endflag = 1;
        }
    } while (endflag == 0);

    g_u1_predriver_error = error;
}

/***********************************************************************************************************************
* Function Name: R_PreDrv_Get_ErrorCode
* Description  : 
* Arguments    : None
* Return Value : 
***********************************************************************************************************************/
uint8_t R_PreDrv_Get_ErrorCode(void)
{
    return (g_u1_predriver_error);
}

/******************************************************************************
* Function Name: wait
* Description  : wait
* Arguments    : count
* Return Value : None
******************************************************************************/
static void wait(uint16_t count)
{
    do
    {
        NOP();
        NOP();
    } while ((count--) > 0);
}

