/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_tau.c
* Version      : Code Generator for RL78/G1F V1.00.01.03 [07 Mar 2016]
* Device(s)    : R5F11BLE
* Tool-Chain   : CA78K0R
* Description  : This file implements device driver for TAU module.
* Creation Date: 2016/07/07
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_tau.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_TAU0_Create
* Description  : This function initializes the TAU0 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_TAU0_Create(void)
{
    int16_t i;

    /*************** basic configuration ***************/
    TAU0EN = 1;                                                         /* enables input clock supply for TAU */
    TPS0 = 0x0050;                                                      /* CK00 = 32MHz, CK01 =   1MHz */
                                                                        /* CK02 = 16MHz, CK03 = 125KHz */

    /*************** configure TAU0 ***************/
    /* TMR00 . timer mode register 00 **************************************************************************
    b15:b14 CKS001:CKS000 - selection of operation clock (fMCK) of channel 0 - operation clock CK00(32MHz)
    b13                   - reserved set to 0
    b12 CCS00             - selection of count clock (fTCLK) of channel 0
                          - operation clock (fMCK) specified by the CKS000 and CKS001 bits
    b11                   - reserved set to 0
    b10:b8 STS002:STS000  - getting of start trigger or capture trigger of channel 0
                          - only software trigger start is valid
    b7:b6 CIS001:CIS000   - selection of TI00 pin input valid edge - falling edge
    b5:b4                 - reserved set to 0
    b3:b0 MD003:MD000     - operation mode of channel 0 - interval timer mode
    ***********************************************************************************************************/
    TMR00 = 0x0000;

    /* TDR00 . timer data register 00 **************************************************************************
    b15:b0 - set count down start value - create 0.5[ms] cycle interrupt
    set value = (frequency of TAU0 count clock) * (interrupt period) = 32M * 0.5[ms] - 1 = 15999
    ***********************************************************************************************************/
    TDR00 = 15999;

    /* TS0 . timer channel start register 0 ********************************************************************
    b15:b12         - reserved set to 0
    b11 TSH03       - trigger to enable operation of the higher 8bit timer when channel 3 is in the 8bit timer mode
                    - no trigger operation
    b10             - reserved set to 0
    b9 TSH01        - trigger to enable operation of the higher 8bit timer when channel 1 is in the 8bit timer mode
                    - no trigger operation
    b8:b4           - reserved set to 0
    b3:b0 TS03:TS00 - operation enable trigger of channel 0 to 3
                    - the TE00 bit is set to 1 and the count operation becomes enabled
    ***********************************************************************************************************/
    TS0 = 0x0001;

    TMPR100 = 1;                                                        /* INTTM00(trigger:end of timer channel 0 count) priority level is 3 */
    TMPR000 = 1;
    TMMK00 = 0;                                                         /* INTTM00 interrupt servicing enabled */

    for (i = 0; i < 0x9; i++)
    {
        NOP();
    }

    /*************** configure TAU1 ***************/
    /* TMR01 . timer mode register 01 **************************************************************************
    b15:b14 CKS011:CKS010 - selection of operation clock (fMCK) of channel 1 - operation clock CK01(1MHz)
    b13                   - reserved set to 0
    b12 CCS01             - selection of count clock (fTCLK) of channel 1
                          - operation clock (fMCK) specified by the CKS010 and CKS011 bits
    b11                   - reserved set to 0
    b10:b8 STS012:STS010  - getting of start trigger or capture trigger of channel 1
                          - only software trigger start is valid
    b7:b6 CIS011:CIS010   - selection of TI01 pin input valid edge - falling edge
    b5:b4                 - reserved set to 0
    b3:b0 MD013:MD010     - operation mode of channel 1 - interval timer mode
    ***********************************************************************************************************/
    TMR01 = 0x8000;

    /* TDR01 . timer data register 01 **************************************************************************
    b15:b0 - set count down start value - TAU1 is free run timer
    ***********************************************************************************************************/
    TDR01 = 0xFFFF;

    /* TS0 . timer channel start register 0 ********************************************************************
    b15:b12         - reserved set to 0
    b11 TSH03       - trigger to enable operation of the higher 8bit timer when channel 3 is in the 8bit timer mode
                    - no trigger operation
    b10             - reserved set to 0
    b9 TSH01        - trigger to enable operation of the higher 8bit timer when channel 1 is in the 8bit timer mode
                    - no trigger operation
    b8:b4           - reserved set to 0
    b3:b0 TS03:TS00 - operation enable trigger of channel 0 to 3
                    - the TE01 bit is set to 1 and the count operation becomes enabled
    ***********************************************************************************************************/
    TS0 = 0x0002;

    TMMK01 = 1;                                                         /* INTTM01(trigger:end of timer channel 1 count) interrupt servicing disabled */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
