/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_tmrd.c
* Version      : Code Generator for RL78/G1F V1.00.01.03 [07 Mar 2016]
* Device(s)    : R5F11BLE
* Tool-Chain   : CA78K0R
* Description  : This file implements device driver for TMRD module.
* Creation Date: 2016/07/13
***********************************************************************************************************************/

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_tmrd.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_TMRD0_Create
* Description  : This function initializes the TMRD0 module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_TMRD0_Create(void)
{
    /*************** configure TRD ***************/
    TRD0EN = 1;                                                         /* enables input clock supply for TRD */

    /* TRDFCR . timer RD function control register *************************************************************
    b7 PWM3         - PWM3 mode select - disabled in reset synchronous PWM mode.
    b6 STCLK        - external clock input select - external clock input disabled
    b5:b4           - reseved set to 0
    b3 OLS1         - counter phase output level select - low initial output and high active level
    b2 OLS0         - phase output level select - low initial output and high active level
    b1:b0 CMD1:CMD0 - combination mode select
                    - in reset synchronous PWM mode, set to 01B (reset synchronous PWM mode)
    ***********************************************************************************************************/
    TRDFCR = 0x0C;
    TRDFCR = 0x0E;

    /* TRDMR . timer RD mode register **************************************************************************
    b7 BFD1 - TRDGRD1 register function select - general register
    b6 BFC1 - TRDGRC1 register function select - general register
    b5 BFD0 - TRDGRD0 register function select - general register
    b4 BFC0 - TRDGRC0 register function select - general register
    b3:b1   - nothing is assigned
    b0 SYNC - timer RD synchronous - TRD0 and TRD1 operate independently
    ***********************************************************************************************************/
    TRDMR = 0xE0;

    /* TRDOER1 . timer RD output master enable register 1*******************************************************
    b7 ED1 - TRDIOD1 output disable - output disabled (TRDIOD1 pin functions as an I/O port)
    b6 EC1 - TRDIOC1 output disable - output disabled (TRDIOC1 pin functions as an I/O port)
    b5 EB1 - TRDIOB1 output disable - output disabled (TRDIOB1 pin functions as an I/O port)
    b4 EA1 - TRDIOA1 output disable - output disabled (TRDIOA1 pin functions as an I/O port)
    b3 ED0 - TRDIOD0 output disable - output disabled (TRDIOD0 pin functions as an I/O port)
    b2 EC0 - TRDIOC0 output disable - output disabled (TRDIOC0 pin functions as an I/O port)
    b1 EB0 - TRDIOB0 output disable - output disabled (TRDIOB0 pin functions as an I/O port)
    b0 EA0 - TRDIOA0 output disable - output disabled (TRDIOA0 pin functions as an I/O port)
    ***********************************************************************************************************/
    TRDOER1 = 0xFF;

    /* TRDOER2 . timer RD output master enable register 2 ******************************************************
    b7 PTO   - INTP0 of pulse output forced cutoff signal input enabled - pulse output forced cutoff input enabled
    b6:b1    - nothing is assigned - the write value must be 0
    b0 SHUTS - forced cutoff flag - not forcibly cut off
    ***********************************************************************************************************/
    TRDOER2 = 0x00;

    /* TRDDF0 . timer RD digital filter function select register 0 *********************************************
    b7:b6 DFCK1:DFCK0 - TRDIOA pin pulse forced cutoff control - forced cutoff disabled
    b5:b4 PENB1:PENB0 - TRDIOB pin pulse forced cutoff control - high-impedance output
    b3:b2 DFD:DFC     - TRDIOC pin pulse forced cutoff control - forced cutoff disabled
    b1:b0 DFB DFA     - TRDIOD pin pulse forced cutoff control - high-impedance output
    ***********************************************************************************************************/
    TRDDF0 = 0x11;

    /* TRDDF1 . timer RD digital filter function select register 1 *********************************************
    b7:b6 DFCK1:DFCK0 - TRDIOA pin pulse forced cutoff control - high-impedance output
    b5:b4 PENB1:PENB0 - TRDIOB pin pulse forced cutoff control - high-impedance output
    b3:b2 DFD:DFC     - TRDIOC pin pulse forced cutoff control - high-impedance output
    b1:b0 DFB DFA     - TRDIOD pin pulse forced cutoff control - high-impedance output
    ***********************************************************************************************************/
    TRDDF1 = 0x55;

    /* TRDIER0 . timer RD interrupt enable register 0 **********************************************************
    b7:b5    - nothing is assigned - the write value must be 0
    b4 OVIE  - overflow/underflow interrupt enable - interrupt (OVI) by bits OVF and UDF disabled
    b3 IMIED - input capture/compare match interrupt enable D - interrupt (IMID) by the IMFD bit is disabled
    b2 IMIEC - input capture/compare match interrupt enable C - interrupt (IMIC) by the IMFC bit is disabled
    b1 IMIEB - input capture/compare match interrupt enable B - interrupt (IMIB) by the IMFB bit is disabled
    b0 IMIEA - input capture/compare match interrupt enable A - interrupt (IMIA) by the IMFA bit is enabled
    ***********************************************************************************************************/
    TRDIER0 = 0x01;

    /* TRDIER1 . timer RD interrupt enable register 1 **********************************************************
    b7:b5    - nothing is assigned - the write value must be 0
    b4 OVIE  - overflow/underflow interrupt enable - interrupt (OVI) by bits OVF and UDF enabled
    b3 IMIED - input capture/compare match interrupt enable D - interrupt (IMID) by the IMFD bit is disabled
    b2 IMIEC - input capture/compare match interrupt enable C - interrupt (IMIC) by the IMFC bit is disabled
    b1 IMIEB - input capture/compare match interrupt enable B - interrupt (IMIB) by the IMFB bit is disabled
    b0 IMIEA - input capture/compare match interrupt enable A - interrupt (IMIA) by the IMFA bit is disabled
    ***********************************************************************************************************/
    TRDIER1 = 0x10;

    /* TRDCR0 . timer RD control register 0 ********************************************************************
    b7:b5 CCLR2:CCLR0 - TRD0 counter clear select
                      - TRD0 register is cleared at compare match with TRDGRA0 register
    b4:b3 CKEG1:CKEG0 - external clock edge select - count at the rising edge
    b2:b0 TCK2:TCK0   - count source select - fHOCO
    ***********************************************************************************************************/
    TRDCR0 = 0x00;

    /* TRD0 . timer RD counter 0 *******************************************************************************
    b15:b0 - counter value - clear counter
    ***********************************************************************************************************/
    TRD0 = (uint16_t)MTR_DEADTIME_SET;
    TRD1 = 0;

    TRDGRA0 = (uint16_t)MTR_CARRIER_SET;
    TRDGRD0 = (uint16_t)MTR_START_CARRIER_SET;
    TRDGRB0 = (uint16_t)MTR_START_CARRIER_SET;
    TRDGRC1 = (uint16_t)MTR_START_CARRIER_SET;
    TRDGRA1 = (uint16_t)MTR_START_CARRIER_SET;
    TRDGRD1 = (uint16_t)MTR_START_CARRIER_SET;
    TRDGRB1 = (uint16_t)MTR_START_CARRIER_SET;

    /* initialize I/O port *************************************************************************************
    ***********************************************************************************************************/
    /***** PWM U,V,W ***************
    P10 - Wn   P11 - Vn   P12 - Wp
    P13 - Vp   P14 - Un   P15 - Up
    ********************************/
    /*R_PORT_Create*/

    /* TRDSTR . timer RD start register ************************************************************************
    b7:b4      - nothing is assigned - the write value must be 0
    b3 CSEL1   - TRD1 count operation select - count continues after compare match with TRDGRA1 register
    b2 CSEL0   - TRD0 count operation select - count continues after compare match with TRDGRA0 register
    b1 TSTART1 - TRD1 count start flag - count start
    b0 TSTART0 - TRD0 count start flag - count start(for ICS)
    ***********************************************************************************************************/
    TRDSTR = 0x0F;

    TRDPR10 = 1;                                                        /* INTTRD0(trigger : timer RD0 compare match A) priority level is 2 */
    TRDPR00 = 0;

    TRDMK0 = 1;                                                         /* INTTRD0 interrupt servicing disable */

    TRDPR11 = 1;                                                        /* INTTRD1(trigger : timer RD1 compare match A) priority level is 2 */
    TRDPR01 = 0;

    TRDMK1 = 1;                                                         /* INTTRD1 interrupt servicing disable */

    PMC1 = 0xE0;
}
/***********************************************************************************************************************
* Function Name: R_TMRD0_Start
* Description  : This function starts TMRD0 counter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_TMRD0_Start(void)
{
    volatile uint8_t trdsr_dummy;

    TRDIF0 = 0U;                                                        /* clear INTTRD0 interrupt flag */
    TRDIF1 = 0U;                                                        /* clear INTTRD1 interrupt flag */
    trdsr_dummy = TRDSR0;                                               /* read TRDSR0 before write 0 */
    TRDSR0 = 0x00U;                                                     /* clear TRD0 each interrupt request */
    trdsr_dummy = TRDSR1;                                               /* read TRDSR1 before write 0 */
    TRDSR1 = 0x00U;                                                     /* clear TRD1 each interrupt request */
    TRDMK0 = 0U;                                                        /* enable INTTRD0 interrupt */
    TRDMK1 = 0U;                                                        /* enable INTTRD1 interrupt */
    TRDSTR |= _04_TMRD_TRD0_COUNT_CONTINUES | _08_TMRD_TRD1_COUNT_CONTINUES;
    TRDSTR |= _02_TMRD_TRD1_COUNT_START | _01_TMRD_TRD0_COUNT_START;
}
/***********************************************************************************************************************
* Function Name: R_TMRD0_Stop
* Description  : This function stops TMRD0 counter.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_TMRD0_Stop(void)
{
    volatile uint8_t trdsr_dummy;

    TRDSTR |= _04_TMRD_TRD0_COUNT_CONTINUES | _08_TMRD_TRD1_COUNT_CONTINUES;
    TRDSTR &= (uint8_t)~(_02_TMRD_TRD1_COUNT_START | _01_TMRD_TRD0_COUNT_START);
    TRDMK0 = 1U;                                                        /* disable INTTRD0 interrupt */
    TRDMK1 = 1U;                                                        /* disable INTTRD1 interrupt */
    TRDIF0 = 0U;                                                        /* clear INTTRD0 interrupt flag */
    TRDIF1 = 0U;                                                        /* clear INTTRD1 interrupt flag */
    trdsr_dummy = TRDSR0;                                               /* read TRDSR0 before write 0 */
    TRDSR0 = 0x00U;                                                     /* clear TRD0 each interrupt request */
    trdsr_dummy = TRDSR1;                                               /* read TRDSR1 before write 0 */
    TRDSR1 = 0x00U;                                                     /* clear TRD1 each interrupt request */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
