/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_tmrj.h
* Version      : Code Generator for RL78/G1F V1.00.01.03 [07 Mar 2016]
* Device(s)    : R5F11BLE
* Tool-Chain   : CA78K0R
* Description  : This file implements device driver for TMRJ module.
* Creation Date: 2016/07/13
***********************************************************************************************************************/
#ifndef TMRJ_H
#define TMRJ_H

/***********************************************************************************************************************
Macro definitions (Register bit)
***********************************************************************************************************************/
/*
    Timer RJ Control Register 0 (TRJCR0) 
*/
/* Timer RJ underflow flag (TUNDF) */
#define _00_TMRJ_UNDERFLOW_NOT_OCCUR                  (0x00U)           /* no underflow */
#define _20_TMRJ_UNDERFLOW_OCCUR                      (0x20U)           /* underflow */
/* Active edge judgment flag (TEDGF) */
#define _00_TMRJ_ACTIVE_EDGE_RECEIVED                 (0x00U)           /* active edge not received */
#define _10_TMRJ_ACTIVE_EDGE_UNRECEIVED               (0x10U)           /* active edge received (end of measurement period) */
/* Timer RJ count forcible stop bit (TSTOP) */
#define _00_TMRJ_FORCIBLE_STOP_DISABLE                (0x00U)           /* the count is not forcibly stopped */
#define _04_TMRJ_FORCIBLE_STOP_ENABLE                 (0x04U)           /* the count is forcibly stopped */
/* Timer RJ count status flag (TCSTF) */
#define _00_TMRJ_STATUS_STOP                          (0x00U)           /* count stops */
#define _02_TMRJ_STATUS_COUNT                         (0x02U)           /* during count */
/* Timer RJ count start bit (TSTART) */
#define _00_TMRJ_COUNT_STOP                           (0x00U)           /* count stops */
#define _01_TMRJ_COUNT_START                          (0x01U)           /* count starts */

/*
    Timer RJ I/O Control Register 0 (TRJIOC0) 
*/
/* TRJIO event input control bit (TIOGT1,TIOGT0) */
#define _00_TMRJ_EVENT_ENABLE_ALWAYS                  (0x00U)           /* event is counted */
#define _40_TMRJ_EVENT_ENABLE_INTP4                   (0x40U)           /* event is counted during INTP4 specified period */
#define _80_TMRJ_EVENT_ENABLE_TIMEROUTSIGNAL          (0x80U)           /* event is counted during polarity period specified for timer output signal */
/* TRJIO input filter select bit (TIPF1,TIPF0) */
#define _00_TMRJ_TRJIO_FILTER_UNUSED                  (0x00U)           /* no filter */
#define _10_TMRJ_TRJIO_FILTER_FCLK                    (0x10U)           /* filter with fCLK sampling */
#define _20_TMRJ_TRJIO_FILTER_FCLK8                   (0x20U)           /* filter with fCLK/8 sampling */
#define _30_TMRJ_TRJIO_FILTER_FCLK32                  (0x30U)           /* filter with fCLK/32 sampling */
/* TRJO output enable bit (TOENA) */
#define _00_TMRJ_TRJO_OUTPUT_DISABLE                  (0x00U)           /* TRJO output disable */
#define _04_TMRJ_TRJO_OUTPUT_ENABLE                   (0x04U)           /* TRJO output */
/* TRJIO polarity switch bit (TEDGSEL) */
#define _00_TMRJ_TRJIO_POLARITY_0                     (0x00U)           /* TRJIO polarity switch bit = 0 */
#define _01_TMRJ_TRJIO_POLARITY_1                     (0x01U)           /* TRJIO polarity switch bit = 1 */

/*
    Timer RJ Mode Register 0 (TRJMR0) 
*/
/* Timer RJ count source select bit (TCK2,TCK1,TCK0) */
#define _00_TMRJ_COUNT_SOURCE_FCLK                    (0x00U)           /* fCLK */
#define _10_TMRJ_COUNT_SOURCE_FCLK8                   (0x10U)           /* fCLK/8 */
#define _30_TMRJ_COUNT_SOURCE_FCLK2                   (0x30U)           /* fCLK/2 */
#define _40_TMRJ_COUNT_SOURCE_FIL                     (0x40U)           /* fIL */
#define _50_TMRJ_COUNT_SOURCE_FELC                    (0x50U)           /* event input from event link controller (ELC) */
#define _60_TMRJ_COUNT_SOURCE_FSUB                    (0x60U)           /* fSUB */
/* TRJIO input polarity select bit (TEDGPL) */
#define _00_TMRJ_TRJIO_POLARITY_ONE                   (0x00U)           /* one edge */
#define _08_TMRJ_TRJIO_POLARITY_BOTH                  (0x08U)           /* both edges */
/* Timer RJ operating mode select bit (TMOD2,TMOD1,TMOD0) */
#define _00_TMRJ_MODE_TIMER                           (0x00U)           /* timer mode */
#define _01_TMRJ_MODE_PULSE_OUTPUT                    (0x01U)           /* pulse output mode */
#define _02_TMRJ_MODE_EVENT_COUNTER                   (0x02U)           /* event counter mode */
#define _03_TMRJ_MODE_PULSE_WIDTH                     (0x03U)           /* pulse width measurement mode */
#define _04_TMRJ_MODE_PULSE_PERIOD                    (0x04U)           /* pulse period measurement mode */

/*
    Timer RJ event pin selection register 0 (TRJISR0) 
*/
/* PWM signal and INTP4 polarity selection (RCCPSEL2) */
#define _00_TMRJ_PWM_POLARITY_L                       (0x00U)           /* L period is counted */
#define _04_TMRJ_PWM_POLARITY_H                       (0x04U)           /* H period is counted */
/* PWM signal selection (RCCPSEL1,RCCPSEL0) */
#define _00_TMRJ_PWM_TRDIOD1                          (0x00U)           /* TRDIOD1 */
#define _01_TMRJ_PWM_TRDIOC1                          (0x01U)           /* TRDIOC1 */
#define _02_TMRJ_PWM_TO02                             (0x02U)           /* TO02 */
#define _03_TMRJ_PWM_TO03                             (0x03U)           /* TO03 */


/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/
#define _0003_TMRJ_TRJ0_VALUE                         (0x0003U)         /* Timer RJ counter register 0 */

/***********************************************************************************************************************
Typedef definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Global functions
***********************************************************************************************************************/
void R_TMRJ0_Create(void);
void R_TMRJ0_Start(void);
void R_TMRJ0_Stop(void);
/* Start user code for function. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#endif
