/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_spm_less_120_cpm.h
* Description : A definitions of the processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_SPM_LESS_120_CPM_H
#define MTR_SPM_LESS_120_CPM_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define MTR_TWOPI               (2 * 3.14159265f)                       /* 2 * pi */
#define MTR_POLE_PAIRS          (MP_POLE_PAIRS)                         /* number of pole pairs */
#define MTR_RPM_RAD             (1716)                                  /* [rpm]->[rad/s] : scale(2^12 * 2^2 = 2^14) */
#define MTR_RAD_RPM             (2445 / MTR_POLE_PAIRS)                 /* [rad/s]->[rpm] : scale(2^10 / 2^2 = 2^8) */

#define MTR_SPEED_LIMIT_RPM     (4290)                                  /* over speed limit [rpm](mechanical angle) : scale(2^0) */
#define MTR_SPEED_LIMIT         ((int16_t)((MTR_SPEED_LIMIT_RPM / 60) * MTR_POLE_PAIRS * MTR_TWOPI * 4))
                                                                        /* over speed limit [rad/s](electrical angle) : scale(2^2) */
#define MTR_SPEED_LPF_K         ((int16_t)(CP_SPEED_LPF_K * 16384))     /* speed LPF gain : scale(2^14) */

#define MTR_SPEED_CALC_BASE     (383)                                   /* conversion factor (rad/s <-> timer count) */
                                                                        /* 1000000 * 2PI / 2^16 * 2^2 : scale(2^-14) */

#define MTR_TIMER_CNT_BUF_NUM   (6)                                     /* buffer number of counts for 60 degrees */

#define MTR_TIMEOUT_CNT         (200)                                   /* undetected time = MTR_TIMEOUT_CNT * 100[us] */
#define MTR_START_CNT           (2000)                                  /* boot mode time = MTR_START_CNT * 500[us] */

/* value of current pattern (CW) */
#define MTR_PATTERN_CW_V_U      (5)                                     /* V phase to U phase */
#define MTR_PATTERN_CW_W_U      (4)                                     /* W phase to U phase */
#define MTR_PATTERN_CW_W_V      (6)                                     /* W phase to V phase */
#define MTR_PATTERN_CW_U_V      (2)                                     /* U phase to V phase */
#define MTR_PATTERN_CW_U_W      (3)                                     /* U phase to W phase */
#define MTR_PATTERN_CW_V_W      (1)                                     /* V phase to W phase */

/* value of current pattern (CCW) */
#define MTR_PATTERN_CCW_V_U     (2)                                     /* V phase to U phase */
#define MTR_PATTERN_CCW_V_W     (6)                                     /* V phase to W phase */
#define MTR_PATTERN_CCW_U_W     (4)                                     /* U phase to W phase */
#define MTR_PATTERN_CCW_U_V     (5)                                     /* U phase to V phase */
#define MTR_PATTERN_CCW_W_V     (1)                                     /* W phase to V phase */
#define MTR_PATTERN_CCW_W_U     (3)                                     /* W phase to U phase */

/* voltage pattern */
#define MTR_PATTERN_ERROR       (0)                                     /* voltage pattern error */
#define MTR_U_PWM_VN_ON         (1)                                     /* U(PWM) to Vn(on) */
#define MTR_U_PWM_WN_ON         (2)                                     /* U(PWM) to Wn(on) */
#define MTR_V_PWM_UN_ON         (3)                                     /* V(PWM) to Un(on) */
#define MTR_V_PWM_WN_ON         (4)                                     /* V(PWM) to Wn(on) */
#define MTR_W_PWM_UN_ON         (5)                                     /* W(PWM) to Un(on) */
#define MTR_W_PWM_VN_ON         (6)                                     /* W(PWM) to Vn(on) */
#define MTR_UP_ON_V_PWM         (7)                                     /* Up(on) to V(PWM) */
#define MTR_UP_ON_W_PWM         (8)                                     /* Up(on) to W(PWM) */
#define MTR_VP_ON_U_PWM         (9)                                     /* Vp(on) to U(PWM) */
#define MTR_VP_ON_W_PWM         (10)                                    /* Vp(on) to W(PWM) */
#define MTR_WP_ON_U_PWM         (11)                                    /* Wp(on) to U(PWM) */
#define MTR_WP_ON_V_PWM         (12)                                    /* Wp(on) to V(PWM) */

/* rotational direction */
#define MTR_CW                  (0)
#define MTR_CCW                 (1)

#define MTR_FLG_CLR             (0)                                     /* for flag clear */
#define MTR_FLG_SET             (1)                                     /* for flag set */
#define MTR_STOP_WAIT_CNT       (100)
/* Timer RD interrupt period(100[us]) * MTR_STOP_WAIT_CNT = motor stop waiting time */

#define MTR_ICS_DECIMATION      (2)                                     /* ICS decimation count */

/* sensorless mode */
#define MTR_SENLESS_INIT        (0)
#define MTR_SENLESS_INDSEN      (1)
#define MTR_SENLESS_BEMF        (2)

/* BEMF mode */
#define MTR_BEMF_PRE            (0)
#define MTR_BEMF_1ST            (1)
#define MTR_BEMF_2ND            (2)
#define MTR_BEMF_3RD            (3)

/* BEMF direction */
#define MTR_BEMF_UP             (0)                                     /* Up */
#define MTR_BEMF_DOWM           (1)                                     /* Down */

/* BEMF wait time */
#define MTR_BEMF_WAIT_1ST_1     (25)                                    /* 100[us] * 25 = 2.5[ms] */
#define MTR_BEMF_WAIT_1ST_2     (10)                                    /* 100[us] * 10 = 1.0[ms] */
#define MTR_BEMF_WAIT_1ST_3     (0)                                     /* 100[us] *  0 = 0.0[ms] */
#define MTR_BEMF_WAIT_2ND       (5)                                     /* 100[us] *  5 = 0.5[ms] */
#define MTR_BEMF_WAIT_3RD       (50)                                    /* 100[us] * 50 = 5.0[ms] */
#define MTR_BEMF_PWMCHG_WAIT    (1)                                     /* 100[us] *  1 = 0.1[ms] */

/* BEMF timeout */
#define MTR_BEMF_TIMEOUT        (2500)                                  /* 100[us] * 2500 = 250[ms] */

#define MTR_BEMF_HIZ_KICK_NUM   (1)                                     /* Hi-Z detection : add to current pattern */

#define MTR_BEMF_TH_LIMIT_MAX   (1024)                                  /* BEMF max level */
#define MTR_BEMF_TH_LIMIT_MIN   (0)                                     /* BEMF min level */
#define MTR_BEMF_TH_MARGIN      (300)                                   /* BEMF margin level */

/* type of voltage pattern */
#define MTR_PWM_PHASE_P         (0)                                     /* regeneration : high side */
#define MTR_PWM_PHASE_N         (1)                                     /* regeneration : low side */

#define MTR_BEMF_MODE_CNT       (2)                                     /* BEMF mode count */

/* switching threshold of BEMF detection */
#define MTR_BEMF_TIMING_TH      (50)                                    /* pwm duty 50% */

/* parameter of pwm duty fixed function */
#define MTR_BEMF_FIX_PWMDUTY           (35)                             /* pwm duty 35% */
#define MTR_BEMF_FIX_PWMDUTY_CNT_LIMIT (32)                             /* 60 degrees * 32 -> electrica degrees is 5.3 times */

/* parameter of pwm duty clamp function */
#define MTR_BEMF_LIMIT_PWMDUTY             (55)                         /* pwm duty 55% */
#define MTR_BEMF_LIMIT_PWMDUTY_ELEROT_TIME (1875)                       /* electrica 360 degrees : 1.875[ms] */

/* parameter of VR1 ADC limit function */
#define MTR_BEMF_LIMIT_VRAD_CNT_LIMIT (1000)                            /* 500[us] * 1000 = 500[ms] */
#define MTR_BEMF_LIMIT_VRAD_COEF1     (8)                               /* coef1 of VR1 ADC limit calculation */
#define MTR_BEMF_LIMIT_VRAD_COEF2     (10)                              /* coef2 of VR1 ADC limit calculation */

/* wait time */
#define MTR_WAITTIME_100_US     (100)                                   /* 100[us] */
#define MTR_WAITTIME_25_MS      (25000)                                 /* 25[ms] */

/* value of current pattern */
typedef enum
{
    MTR_PATTERN_W_V = 0,
    MTR_PATTERN_W_U,
    MTR_PATTERN_V_U,
    MTR_PATTERN_V_W,
    MTR_PATTERN_U_W,
    MTR_PATTERN_U_V,

    MTR_PATTERN_MAX,

} mtr_v_pattern_t;

/* type of BEMF */
typedef enum
{
    MTR_BEMF_TH_HSIDE = 0,
    MTR_BEMF_TH_HIZ,
    MTR_BEMF_TH_LSIDE,

    MTR_BEMF_TH_MAX,

} mtr_bemf_th_t;

/* type of Hi-Z */
typedef enum
{
    MTR_BEMF_HIZ_U = 0,
    MTR_BEMF_HIZ_V,
    MTR_BEMF_HIZ_W,

    MTR_BEMF_HIZ_MAX,

} mtr_bemf_hiz_t;

/* type of ADC */
typedef enum
{
    MTR_BEMF_AD_ISENSE = 0,
    MTR_BEMF_AD_BEMF,

    MTR_BEMF_AD_MAX,

} mtr_bemf_ad_t;

/* BEMF sequence */
typedef enum
{
    MTR_BEMF_SEQ_INIT = 0,
    MTR_BEMF_SEQ_WAIT,
    MTR_BEMF_SEQ_SET_MOTEN_OFF,
    MTR_BEMF_SEQ_WAIT_2ND,
    MTR_BEMF_SEQ_CHK_HIZ,
    MTR_BEMF_SEQ_SET_MOTEN_ON,
    MTR_BEMF_SEQ_WAIT_3RD,
    MTR_BEMF_SEQ_CHK_TH_RANGE,
    MTR_BEMF_SEQ_DET_BEMF_LEVEL,
    MTR_BEMF_SEQ_SET_PWM,
    MTR_BEMF_SEQ_OVF_COUNT,

    MTR_BEMF_SEQ_NUM_MAX,

} mtr_bemf_seq_t;

/******************************************************************************
* Macro definitions for sequence control
******************************************************************************/
/* run mode */
#define MTR_BOOT_MODE           (0x00)
#define MTR_OPENLOOP_MODE       (0x01)
#define MTR_START_MODE          (0x02)
#define MTR_LESS_120_MODE       (0x03)
#define MTR_BEMF_120_MODE       (0x04)
#define MTR_ENCD_FOC_MODE       (0x05)
#define MTR_LESS_FOC_MODE       (0x06)

/* error status */
#define MTR_ALARM_ERROR         (0x01)
#define MTR_OVER_VOLTAGE_ERROR  (0x02)
#define MTR_OVER_SPEED_ERROR    (0x04)
#define MTR_TIMEOUT_ERROR       (0x08)
#define MTR_LESS_ERROR          (0x10)
#define MTR_BEMF_ERROR          (0x20)
#define MTR_UNDER_VOLTAGE_ERROR (0x40)
#define MTR_UNKNOWN_ERROR       (0x80)
#define MTR_HW_ERROR            (MTR_ALARM_ERROR)
#define MTR_FW_ERROR            (MTR_OVER_VOLTAGE_ERROR | MTR_OVER_SPEED_ERROR    | MTR_TIMEOUT_ERROR |\
                                 MTR_LESS_ERROR         | MTR_UNDER_VOLTAGE_ERROR | MTR_UNKNOWN_ERROR)

/* system mode */
#define MTR_MODE_STOP           (0x00)
#define MTR_MODE_RUN            (0x01)
#define MTR_MODE_ERROR          (0x02)
#define MTR_SIZE_STATE          (3)                                     /* number of state */

/* event */
#define MTR_EVENT_STOP          (0x00)
#define MTR_EVENT_RUN           (0x01)
#define MTR_EVENT_ERROR         (0x02)
#define MTR_EVENT_RESET         (0x03)
#define MTR_SIZE_EVENT          (4)                                     /* number of event */

typedef struct
{
    int16_t    s2_ref_speed;
    int16_t    s2_direction;
    int16_t    s2_speed_lpf_k;
} MTR_ICS_INPUT;

/******************************************************************************
* Global function definitions
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *);                                   /* set value to buffer */
void R_MTR_SetSpeed(int16_t);                                           /* set speed reference */
void R_MTR_SetDir(uint8_t);                                             /* set rotational direction */
int16_t R_MTR_GetSpeed(void);                                           /* get speed value */
uint8_t R_MTR_GetDir(void);                                             /* get rotation direction */
uint8_t R_MTR_GetStatus(void);                                          /* get status of motor control system */

void mtr_pattern_set(uint8_t);                                          /* set voltage pattern */

uint8_t mtr_convert_indsenpat_to_pwmpat(uint8_t);                       /* convert current pattern */

int16_t mtr_get_bemf_threshold(void);                                   /* get BEMF threshold */
int16_t mtr_get_bemf_threshold_direct(uint8_t, uint8_t);                /* get BEMF threshold */
uint8_t mtr_get_bemf_dir(void);                                         /* get BEMF direction */

void mtr_set_predrv_isense(void);                                       /* set detection of current */
void mtr_set_predrv_bemf(void);                                         /* set detection of BEMF */
void mtr_set_variables(void);                                           /* set variables (conversion of unit) */
void mtr_error_check(void);                                             /* check error status */

/* for sequence control */
void R_MTR_InitSequence(void);                                          /* initialize variables for sequence control */
void R_MTR_ExecEvent(uint8_t);                                          /* change motor state and select action */

#endif /* MTR_SPM_LESS_120_CPM_H */
