/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_interrupt.c
* Description : The interrupt handler
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Interrupt vector define
******************************************************************************/
#pragma interrupt mtr_alarm_interrupt(vect=INTP4)
#pragma interrupt mtr_tau0_interrupt(vect=INTTM00)
#pragma interrupt mtr_carrier_interrupt(vect=INTTRD0)
#pragma interrupt mtr_carrier_udf_interrupt(vect=INTTRD1)

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_spm_less_120_cpm.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "r_cg_macrodriver.h"
#include "r_cg_predrv_reg.h"
#include "r_cg_predrv.h"
#include "r_cg_userdefine.h"
#ifdef ICS_USE
/****** for ICS ******/
#include "ics2_RL78G1F.h"
/*********************/
#endif

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile uint16_t   g_u2_cnt_boot_mode;                          /* counter for boot mode time */
extern volatile uint16_t   g_u2_cnt_wait_stop;                          /* counter for waiting for motor stop */
extern volatile uint8_t    g_u1_flg_wait_stop;                          /* flag for waiting for motor stop */

extern volatile uint8_t    g_u1_enable_write;                           /* ICS write enable flag */

extern volatile int16_t    g_s2_vdc_ad;                                 /* bus voltage : scale(2^7) */
extern volatile int16_t    g_s2_pdrv_ad;                                /* predriver voltage : scale(2^0) */

#ifdef ICS_USE
/****** for ICS ******/
extern volatile uint8_t    g_u1_cnt_ics;                                /* counter for period of calling "scope_watchpoint" */
/*********************/
#endif

/* for sequence control */
extern volatile uint16_t   g_u2_run_mode;                               /* run mode */
extern volatile uint8_t    g_u1_mode_system;                            /* system mode */
extern volatile uint8_t    g_u1_error_status;                           /* error status */
extern volatile uint8_t    g_u1_direction;                              /* rotational direction (0:CW ,1:CCW) */
extern volatile uint8_t    g_spi00_comend_flag;                         /* FALSE:SPI00 is communicating */

extern volatile uint8_t    g_u1_sensorless_mode;                        /* sensorless mode */
extern volatile uint8_t    g_u1_bemf_mode;                              /* BEMF mode */
extern volatile uint8_t    g_u1_bemf_seq;                               /* BEMF sequence */
extern volatile int8_t     g_s1_indsen_signal;                          /* current pattern */
extern volatile int16_t    g_s2_bemf_ad_pre;                            /* BEMF (pre) */
extern volatile int16_t    g_s2_bemf_ad_now;                            /* BEMF (now) */
extern volatile int16_t    g_s2_bemf_time_cnt;                          /* counts for BEMF wait time */
extern volatile int16_t    g_s2_bemf_time_fg_calc_pre;                  /* 60 degrees calculation (pre) */
extern volatile int16_t    g_s2_bemf_time_fg_calc_now;                  /* 60 degrees calculation (now) */
extern volatile int16_t    g_s2_bemf_time_a_cnt;                        /* counts for BEMF a-time */
extern volatile int16_t    g_s2_bemf_time_a_cnt_pre;                    /* counts for BEMF a-time (before exceeding the BEMF threshold) */
extern volatile int16_t    g_s2_bemf_time_b_cnt;                        /* counts for BEMF b-time */
extern volatile int16_t    g_s2_bemf_time_b_cnt_now;                    /* counts for BEMF b-time (after exceeding the BEMF threshold) */
extern volatile int16_t    g_s2_bemf_time_fg_cnt;                       /* counts for 30 degrees */
extern volatile int16_t    g_s2_bemf_fg_th_pre;                         /* threshold of 30 degrees count (pre) */
extern volatile int16_t    g_s2_bemf_fg_th_now;                         /* threshold of 30 degrees count (now) */
extern volatile int16_t    g_s2_bemf_calc_pre;                          /* calculation (pre) */
extern volatile int16_t    g_s2_bemf_calc_now;                          /* calculation (now) */
extern volatile uint8_t    g_u1_bemf_pwm_phase;                         /* type of voltage pattern */
extern volatile int16_t    g_s2_bemf_timeout_cnt;                       /* counts for BEMF timeout */
extern volatile int16_t    g_s2_bemf_time_pwmchg_cnt;                   /* counts for Hi-Z detection ADC timing */
extern volatile uint8_t    g_u1_bemf_chk_hiz;                           /* type of Hi-Z */
extern volatile uint8_t    g_u1_bemf_chk_hiz_status;                    /* status of Hi-Z detection */
extern volatile uint8_t    g_u1_bemf_req_moten_on;                      /* request MOT_EN on */
extern volatile uint8_t    g_u1_bemf_req_moten_off;                     /* request MOT_EN off */
extern volatile uint8_t    g_u1_bemf_mode_cnt;                          /* count for BEMF mode */
extern volatile uint8_t    g_u1_bemf_th_type;                           /* type of BEMF */
extern volatile uint8_t    g_u1_bemf_ad_type;                           /* type of ADC */
extern volatile int16_t    g_s2_pdrv_ad_isense;                         /* current */
extern volatile uint16_t   g_u2_bemf_limit_vr_ad_cnt;                   /* counts for pwm duty fixed */

extern const volatile uint8_t g_u1_bemf_v_pattern[2][6][3];             /* current pattern for Hi-Z detection */

/******************************************************************************
* Private global variables and functions
******************************************************************************/
static int16_t    g_s2_bemf_chk_ad[MTR_PATTERN_MAX][MTR_BEMF_HIZ_MAX];  /* BEMF for Hi-Z detection */

/******************************************************************************
* Function Name : mtr_alarm_interrupt
* Description   : Alarm interrupt(INTP4)
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_alarm_interrupt(void)
{
#ifdef HW_ERR_DET_ON
    R_MTR_ExecEvent(MTR_EVENT_ERROR);                                   /* execute error event */

    DI();

    g_u1_error_status |= MTR_ALARM_ERROR;                               /* error status update : alarm error */

    EI();

    mtr_clear_oc_flag();                                                /* clear forced cutoff flag (detect alarm) */
#endif
}

/******************************************************************************
* Function Name : mtr_tau0_interrupt
* Description   : TAU0 interrupt (period: 500[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_tau0_interrupt(void)
{
    EI();

    /* check counts for pwm duty fixed */
    if (g_u2_bemf_limit_vr_ad_cnt < (MTR_BEMF_LIMIT_VRAD_CNT_LIMIT * 2))
    {
        g_u2_bemf_limit_vr_ad_cnt++;
    }

    /*==============================*/
    /*     count boot mode time     */
    /*==============================*/

    if (g_u1_flg_wait_stop == MTR_FLG_CLR)                              /* check wait stop flag */
    {
        if (g_u1_mode_system == MTR_MODE_RUN)                           /* check system mode */
        {
            if (g_u2_run_mode == MTR_BOOT_MODE)
            {
                g_u2_cnt_boot_mode++;

                if (g_u2_cnt_boot_mode >= MTR_START_CNT)
                {
                    g_u2_cnt_boot_mode = 0;
                    g_u2_run_mode = MTR_LESS_120_MODE;                  /* normal control mode */
                }
            }
        }
    }
}

/******************************************************************************
* Function Name : mtr_carrier_interrupt
* Description   : Carrier interrupt (period: 100[us])
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_carrier_interrupt(void)
{
    int16_t  s2_temp             = 0;
    int16_t  s2_bemf             = 0;
    int16_t  s2_bemf_th          = 0;
    int16_t  s2_bemf_th_margin_p = 0;
    int16_t  s2_bemf_th_margin_n = 0;
    int16_t  s2_a                = 0;
    int16_t  s2_b                = 0;
    int16_t  s2_x                = 0;
    int16_t  s2_y                = 0;
    int16_t  s2_a_b              = 0;
    int16_t  s2_x_y              = 0;
    int16_t  s2_calc             = 0;
    int16_t  s2_timeout          = 0;
    uint8_t  u1_bemf_dir         = 0;
    uint8_t  u1_bemf_flag        = 0;
    uint8_t  u1_setptflag        = 0;
    uint8_t  u1_v_pat[MTR_BEMF_HIZ_MAX] = {0,0,0};
    uint8_t  u1_v_flg[MTR_BEMF_HIZ_MAX] = {0,0,0};
    uint8_t  u1_v_pat_cnt        = 0;
    uint8_t  u1_hiz_cnt          = 0;
    uint8_t  u1_chg_calc_flag    = 0;

    EI();

    if (g_u1_sensorless_mode == MTR_SENLESS_BEMF)
    {
        /*==============================*/
        /*           set pdrv           */
        /*==============================*/

        if (g_u1_bemf_th_type == MTR_BEMF_TH_HSIDE)
        {
            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                while (g_spi00_comend_flag == FALSE)
                {
                    NOP();
                }
            }

            if (g_u1_bemf_ad_type == MTR_BEMF_AD_BEMF)
            {
                DI();

                /* BEMF A/D */
                g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

                EI();
            }

            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                /* PreDriver Output Signal : Current */
                R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_ISEN);

                g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
            }
        }
        else if (g_u1_bemf_th_type == MTR_BEMF_TH_LSIDE)
        {
            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                while (g_spi00_comend_flag == FALSE)
                {
                    NOP();
                }
            }

            if (g_u1_bemf_ad_type == MTR_BEMF_AD_ISENSE)
            {
                DI();

                /* Current A/D */
                g_s2_pdrv_ad_isense = mtr_get_adc(MTR_ADCCH_PDRV);

                EI();
            }

            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                /* PreDriver Output Signal : BEMF */
                R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_BEMF);

                g_u1_bemf_ad_type = MTR_BEMF_AD_BEMF;
            }
        }
        else if (g_u1_bemf_th_type == MTR_BEMF_TH_HIZ)
        {
            if (g_u1_bemf_pwm_phase == MTR_PWM_PHASE_N)
            {
                if (g_u1_bemf_mode == MTR_BEMF_3RD)
                {
                    while (g_spi00_comend_flag == FALSE)
                    {
                        NOP();
                    }
                }

                if (g_u1_bemf_ad_type == MTR_BEMF_AD_BEMF)
                {
                    DI();

                    /* BEMF A/D */
                    g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

                    EI();
                }

                if (g_u1_bemf_mode == MTR_BEMF_3RD)
                {
                    /* PreDriver Output Signal : Current */
                    R_PreDrvReg_Write(PREDRVREG_ADC_SEL, SET_ADC_SEL_ISEN);

                    g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
                }

                if (g_u1_bemf_ad_type == MTR_BEMF_AD_ISENSE)
                {
                    DI();

                    /* Current A/D */
                    g_s2_pdrv_ad_isense = mtr_get_adc(MTR_ADCCH_PDRV);

                    EI();
                }

                if (g_u1_bemf_mode == MTR_BEMF_3RD)
                {
                    /* PreDriver Output Signal : BEMF */
                    R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_BEMF);

                    g_u1_bemf_ad_type = MTR_BEMF_AD_BEMF;
                }
            }
        }
        else
        {
            /* Do Nothing */
        }

        g_s2_bemf_time_a_cnt++;
        g_s2_bemf_time_b_cnt++;
        g_s2_bemf_timeout_cnt++;
        g_s2_bemf_time_pwmchg_cnt++;

        if ((g_s2_bemf_time_a_cnt < 0) || (g_s2_bemf_time_b_cnt < 0))
        {
            mtr_ctrl_stop();

            g_u1_bemf_seq = MTR_BEMF_SEQ_OVF_COUNT;
        }

        if (g_s2_bemf_timeout_cnt > MTR_BEMF_TIMEOUT)
        {
            mtr_ctrl_stop();

            g_u1_bemf_seq = MTR_BEMF_SEQ_OVF_COUNT;
        }

        switch (g_u1_bemf_seq)
        {
            case MTR_BEMF_SEQ_INIT:                                     /* initialize */
                for (u1_v_pat_cnt = 0; u1_v_pat_cnt < MTR_PATTERN_MAX; u1_v_pat_cnt++)
                {
                    for (u1_hiz_cnt = 0; u1_hiz_cnt < MTR_BEMF_HIZ_MAX; u1_hiz_cnt++)
                    {
                        g_s2_bemf_chk_ad[u1_v_pat_cnt][u1_hiz_cnt] = 0;
                    }
                }

                g_s2_bemf_time_cnt = 0;
                g_u1_bemf_seq      = MTR_BEMF_SEQ_WAIT;
            break;

            case MTR_BEMF_SEQ_WAIT:                                     /* wait */
                g_s2_bemf_time_cnt++;

                if (g_u1_bemf_chk_hiz_status == 0)
                {
                    s2_timeout = MTR_BEMF_WAIT_1ST_1;
                }
                else
                {
                    if (g_u1_bemf_th_type == MTR_BEMF_TH_HIZ)
                    {
                        s2_timeout = MTR_BEMF_WAIT_1ST_3;
                    }
                    else
                    {
                        s2_timeout = MTR_BEMF_WAIT_1ST_2;
                    }
                }

                if (g_s2_bemf_time_cnt > s2_timeout)
                {
                    g_s2_bemf_timeout_cnt = 0;

                    if (g_u1_bemf_chk_hiz_status == 0)
                    {
                        g_u1_bemf_req_moten_off = 1;
                        g_u1_bemf_seq           = MTR_BEMF_SEQ_SET_MOTEN_OFF;
                    }
                    else
                    {
                        g_u1_bemf_req_moten_off  = 0;
                        g_s2_bemf_time_a_cnt_pre = g_s2_bemf_time_a_cnt;
                        g_u1_bemf_seq            = MTR_BEMF_SEQ_DET_BEMF_LEVEL;
                    }
                }
            break;

            case MTR_BEMF_SEQ_SET_MOTEN_OFF:                            /* motor control : off */
                if (g_u1_bemf_req_moten_off == 0)
                {
                    g_s2_bemf_time_cnt    = 0;
                    g_s2_bemf_timeout_cnt = 0;
                    g_u1_bemf_seq         = MTR_BEMF_SEQ_WAIT_2ND;
                }
            break;

            case MTR_BEMF_SEQ_WAIT_2ND:                                 /* Hi-Z detection : wait for start */
                g_s2_bemf_time_cnt++;

                if (g_s2_bemf_time_cnt > MTR_BEMF_WAIT_2ND)
                {
                    g_s2_bemf_timeout_cnt = 0;
                    g_u1_bemf_seq         = MTR_BEMF_SEQ_CHK_HIZ;
                }
            break;

            case MTR_BEMF_SEQ_CHK_HIZ:                                  /* Hi-Z detection : main */
                switch (g_u1_bemf_chk_hiz)
                {
                    case MTR_BEMF_HIZ_U:
                        MTR_PORT_BEMF_U = 1;                            /* BEMF measure(U) ON */
                        MTR_PORT_BEMF_V = 0;                            /* BEMF measure(V) OFF */
                        MTR_PORT_BEMF_W = 0;                            /* BEMF measure(W) OFF */
                    break;

                    case MTR_BEMF_HIZ_V:
                        MTR_PORT_BEMF_U = 0;                            /* BEMF measure(U) OFF */
                        MTR_PORT_BEMF_V = 1;                            /* BEMF measure(V) ON */
                        MTR_PORT_BEMF_W = 0;                            /* BEMF measure(W) OFF */
                    break;

                    case MTR_BEMF_HIZ_W:
                        MTR_PORT_BEMF_U = 0;                            /* BEMF measure(U) OFF */
                        MTR_PORT_BEMF_V = 0;                            /* BEMF measure(V) OFF */
                        MTR_PORT_BEMF_W = 1;                            /* BEMF measure(W) ON */
                    break;

                    default:
                    break;
                }

                if (g_s2_bemf_time_pwmchg_cnt > MTR_BEMF_PWMCHG_WAIT)
                {
                    g_s2_bemf_chk_ad[g_s1_indsen_signal][g_u1_bemf_chk_hiz] = g_s2_pdrv_ad;
                    g_s2_bemf_time_pwmchg_cnt = 0;
                    g_u1_bemf_chk_hiz++;

                    if (g_u1_bemf_chk_hiz > MTR_BEMF_HIZ_W)
                    {
                        g_u1_bemf_chk_hiz = 0;

                        for (u1_hiz_cnt = 0; u1_hiz_cnt < MTR_BEMF_HIZ_MAX; u1_hiz_cnt++)
                        {
                            u1_v_pat[u1_hiz_cnt] = g_u1_bemf_v_pattern[g_u1_direction][g_s1_indsen_signal][u1_hiz_cnt];
                            u1_v_flg[u1_hiz_cnt] = 0;
                            s2_bemf_th           = mtr_get_bemf_threshold_direct(MTR_BEMF_TH_HIZ, u1_hiz_cnt);

                            /* compare Hi-Z pattern */
                            if (g_s2_bemf_chk_ad[g_s1_indsen_signal][u1_hiz_cnt] > s2_bemf_th)
                            {
                                if (u1_v_pat[u1_hiz_cnt] == 1)
                                {
                                    u1_v_flg[u1_hiz_cnt] = 1;           /* pattern match */
                                }
                            }
                            else
                            {
                                if (u1_v_pat[u1_hiz_cnt] == 0)
                                {
                                    u1_v_flg[u1_hiz_cnt] = 1;           /* pattern match */
                                }
                            }
                        }

                        /* check UVW pattern match */
                        if ((u1_v_flg[0] == 1) && (u1_v_flg[1] == 1) && (u1_v_flg[2] == 1))
                        {
                            /* add to current pattern */
                            g_s1_indsen_signal += MTR_BEMF_HIZ_KICK_NUM;

                            if (g_s1_indsen_signal > (MTR_PATTERN_MAX - 1))
                            {
                                g_s1_indsen_signal -= MTR_PATTERN_MAX;
                            }

                            /* set voltage pattern */
                            mtr_pattern_set(mtr_convert_indsenpat_to_pwmpat(g_s1_indsen_signal));

                            g_u1_bemf_chk_hiz_status = 1;
                            g_u1_bemf_req_moten_on   = 1;
                            g_s2_bemf_timeout_cnt    = 0;
                            g_u1_bemf_seq            = MTR_BEMF_SEQ_SET_MOTEN_ON;
                        }
                        else
                        {
                            /* next current pattern */
                            g_s1_indsen_signal++;

                            if (g_s1_indsen_signal > (MTR_PATTERN_MAX - 1))
                            {
                                g_s1_indsen_signal = 0;
                            }

                            /* set voltage pattern */
                            mtr_pattern_set(mtr_convert_indsenpat_to_pwmpat(g_s1_indsen_signal));
                        }
                    }
                }
            break;

            case MTR_BEMF_SEQ_SET_MOTEN_ON:                             /* motor control : on */
                if (g_u1_bemf_req_moten_on == 0)
                {
                    g_s2_bemf_time_cnt    = 0;
                    g_s2_bemf_timeout_cnt = 0;
                    g_s2_bemf_time_a_cnt  = 0;
                    g_s2_bemf_time_b_cnt  = 0;
                    g_u1_bemf_seq         = MTR_BEMF_SEQ_WAIT_3RD;
                }
            break;

            case MTR_BEMF_SEQ_WAIT_3RD:                                 /* Hi-Z detection : wait for start */
                g_s2_bemf_time_cnt++;

                if (g_s2_bemf_time_cnt > MTR_BEMF_WAIT_3RD)
                {
                    g_s2_bemf_timeout_cnt = 0;
                    g_u1_bemf_seq         = MTR_BEMF_SEQ_CHK_TH_RANGE;
                }
            break;

            case MTR_BEMF_SEQ_CHK_TH_RANGE:                             /* check margin of BEMF level */
                s2_bemf             = g_s2_pdrv_ad;
                s2_bemf_th          = mtr_get_bemf_threshold();
                s2_bemf_th_margin_p = s2_bemf_th + MTR_BEMF_TH_MARGIN;
                s2_bemf_th_margin_n = s2_bemf_th - MTR_BEMF_TH_MARGIN;

                if (s2_bemf_th_margin_p > MTR_BEMF_TH_LIMIT_MAX)
                {
                    s2_bemf_th_margin_p = MTR_BEMF_TH_LIMIT_MAX;
                }

                if (s2_bemf_th_margin_n < MTR_BEMF_TH_LIMIT_MIN)
                {
                    s2_bemf_th_margin_n = MTR_BEMF_TH_LIMIT_MIN;
                }

                if ((s2_bemf < s2_bemf_th_margin_p) && (s2_bemf > s2_bemf_th_margin_n))
                {
                    g_s2_bemf_timeout_cnt = 0;
                    g_u1_bemf_seq         = MTR_BEMF_SEQ_DET_BEMF_LEVEL;
                }

                g_s2_bemf_time_a_cnt_pre = g_s2_bemf_time_a_cnt;
            break;

            case MTR_BEMF_SEQ_DET_BEMF_LEVEL:                           /* detect BEMF level */
                s2_bemf     = g_s2_pdrv_ad;
                s2_bemf_th  = mtr_get_bemf_threshold();
                u1_bemf_dir = mtr_get_bemf_dir();

                /* compare BEMF level and BEMF threshold */
                if (u1_bemf_dir == MTR_BEMF_UP)
                {
                    if (s2_bemf > s2_bemf_th)
                    {
                        u1_bemf_flag = 1;                               /* detect BEMF zerocross */
                    }
                    else
                    {
                        g_s2_bemf_time_a_cnt_pre = g_s2_bemf_time_a_cnt;
                        g_s2_bemf_ad_pre         = s2_bemf;
                    }
                }
                else
                {
                    if (s2_bemf < s2_bemf_th)
                    {
                        u1_bemf_flag = 1;                               /* detect BEMF zerocross */
                    }
                    else
                    {
                        g_s2_bemf_time_a_cnt_pre = g_s2_bemf_time_a_cnt;
                        g_s2_bemf_ad_pre         = s2_bemf;
                    }
                }

                if (u1_bemf_flag == 1)
                {
                    if (g_u1_bemf_mode == MTR_BEMF_PRE)
                    {
                        g_s2_bemf_time_a_cnt  = 0;
                        g_s2_bemf_time_b_cnt  = 0;
                        g_s2_bemf_fg_th_now   = 0;
                        g_s2_bemf_time_fg_cnt = 0;
                        g_s2_bemf_timeout_cnt = 0;
                        g_u1_bemf_seq         = MTR_BEMF_SEQ_SET_PWM;
                    }
                    else
                    {
                        g_s2_bemf_time_b_cnt_now = g_s2_bemf_time_b_cnt;
                        g_s2_bemf_ad_now         = s2_bemf;
                        g_s2_bemf_time_a_cnt     = 0;
                        g_s2_bemf_time_b_cnt     = 0;

                        s2_bemf_th = mtr_get_bemf_threshold();
                        s2_a       = g_s2_bemf_time_a_cnt_pre;
                        s2_b       = g_s2_bemf_time_b_cnt_now;
                        s2_x       = g_s2_bemf_ad_pre;
                        s2_y       = g_s2_bemf_ad_now;
                        s2_a_b  = (s2_b - s2_a) * 100;
                        s2_x_y  = s2_y - s2_x;
                        s2_calc = ((s2_a_b / s2_x_y) * (s2_bemf_th - s2_y)) / 100;

                        if (s2_calc < 0)
                        {
                            s2_calc *= (-1);
                        }

                        g_s2_bemf_calc_now = s2_calc;

                        if ((s2_bemf_th < s2_x) && (s2_bemf_th < s2_y))
                        {
                            u1_chg_calc_flag = 1;
                        }
                        else if ((s2_bemf_th > s2_x) && (s2_bemf_th > s2_y))
                        {
                            u1_chg_calc_flag = 1;
                        }
                        else
                        {
                            /* Do Nothing */
                        }

                        switch (g_u1_bemf_mode)
                        {
                            case MTR_BEMF_1ST:
                                g_s2_bemf_time_fg_calc_now = g_s2_bemf_time_b_cnt_now;
                                g_s2_bemf_fg_th_now        = g_s2_bemf_time_fg_calc_now / 2;
                            break;

                            case MTR_BEMF_2ND:
                                g_s2_bemf_time_fg_calc_now = g_s2_bemf_time_b_cnt_now;
                                g_s2_bemf_fg_th_now        = g_s2_bemf_time_fg_calc_now / 2;
                            break;

                            case MTR_BEMF_3RD:
                                g_s2_bemf_time_fg_calc_now = g_s2_bemf_time_b_cnt_now + g_s2_bemf_calc_pre - g_s2_bemf_calc_now;

                                if (g_u1_bemf_th_type == MTR_BEMF_TH_HIZ)
                                {
                                    g_s2_bemf_fg_th_now = (g_s2_bemf_time_fg_calc_now - g_s2_bemf_calc_now) / 4;
                                }
                                else
                                {
                                    if (u1_chg_calc_flag == 1)
                                    {
                                        g_s2_bemf_fg_th_now = (g_s2_bemf_time_fg_calc_now - g_s2_bemf_calc_now) * 2 / 5;

                                        if (g_s2_bemf_fg_th_now >= 0)
                                        {
                                            g_s2_bemf_fg_th_now += 1;
                                        }
                                    }
                                    else
                                    {
                                        g_s2_bemf_fg_th_now = (g_s2_bemf_time_fg_calc_now - g_s2_bemf_calc_now) / 3;
                                    }
                                }

                                if (g_s2_bemf_fg_th_now < 0)
                                {
                                    g_s2_bemf_calc_now  = g_s2_bemf_calc_pre;
                                    g_s2_bemf_fg_th_now = g_s2_bemf_fg_th_pre;
                                }
                            break;

                            default:
                            break;
                        }

                        g_s2_bemf_time_fg_calc_pre = g_s2_bemf_time_fg_calc_now;
                        g_s2_bemf_calc_pre         = g_s2_bemf_calc_now;
                        g_s2_bemf_fg_th_pre        = g_s2_bemf_fg_th_now;
                        g_s2_bemf_time_fg_cnt      = 0;
                        g_s2_bemf_timeout_cnt      = 0;
                        g_u1_bemf_seq              = MTR_BEMF_SEQ_SET_PWM;
                    }
                }
            break;

            case MTR_BEMF_SEQ_SET_PWM:                                  /* wait 30 degrees and set PWM and change FG */
                g_s2_bemf_time_fg_cnt++;

                if (g_s2_bemf_time_fg_cnt >= g_s2_bemf_fg_th_now)
                {
                    u1_setptflag = 1;
                }

                if (u1_setptflag == 1)
                {
                    /* next current pattern */
                    g_s1_indsen_signal++;

                    if (g_s1_indsen_signal > (MTR_PATTERN_MAX - 1))
                    {
                        g_s1_indsen_signal = 0;
                    }

                    /* set voltage pattern */
                    mtr_pattern_set(mtr_convert_indsenpat_to_pwmpat(g_s1_indsen_signal));

                    switch (g_u1_bemf_mode)
                    {
                        case MTR_BEMF_PRE:
                            g_u1_bemf_mode = MTR_BEMF_1ST;
                        break;

                        case MTR_BEMF_1ST:
                            g_u1_bemf_mode = MTR_BEMF_2ND;
                        break;

                        case MTR_BEMF_2ND:
                            g_u1_bemf_mode_cnt++;

                            if (g_u1_bemf_mode_cnt > MTR_BEMF_MODE_CNT)
                            {
                                g_u1_bemf_mode = MTR_BEMF_3RD;
                            }
                        break;

                        default:
                        break;
                    }

                    g_u1_bemf_seq         = MTR_BEMF_SEQ_WAIT;
                    g_s2_bemf_timeout_cnt = 0;
                    g_s2_bemf_time_cnt    = 0;
                }
            break;

            case MTR_BEMF_SEQ_OVF_COUNT:                                /* counter overflow (BEMF error) */
                g_u1_sensorless_mode    = MTR_SENLESS_INIT;
                g_u1_bemf_mode          = MTR_BEMF_PRE;
                g_u1_bemf_seq           = MTR_BEMF_SEQ_INIT;
                g_u1_bemf_req_moten_on  = 1;
                g_u1_bemf_req_moten_off = 0;

                R_MTR_ExecEvent(MTR_EVENT_ERROR);                       /* execute error event */

                DI();

                g_u1_error_status |= MTR_BEMF_ERROR;                    /* error status update : BEMF error */

                EI();
            break;

            default:
            break;
        }
    }

    /*==============================*/
    /*           set vdc            */
    /*==============================*/

    DI();

    s2_temp = mtr_get_adc(MTR_ADCCH_VM);

    EI();

    g_s2_vdc_ad = R_FIX8_mul_int16(s2_temp, MTR_VDC_SCALING);           /* g_s2_vdc_ad : scale(2^7) */

    /*============================*/
    /*      check error           */
    /*============================*/

    mtr_error_check();                                                  /* check error status */

    /*** wait motor stop ***/
    if (g_u1_flg_wait_stop == MTR_FLG_SET)                              /* waiting for motor stop */
    {
        g_u2_cnt_wait_stop++;

        if (g_u2_cnt_wait_stop >= MTR_STOP_WAIT_CNT)
        {
            g_u1_flg_wait_stop = MTR_FLG_CLR;
            g_u2_cnt_wait_stop = 0;
        }
    }

#ifdef ICS_USE
    /****** for ICS ******/
    g_u1_cnt_ics++;

    if (g_u1_cnt_ics > MTR_ICS_DECIMATION)                              /* period : 300[us] */
    {
        DI();                                                           /* interrupt disable */

        ics2_watchpoint();

        EI();                                                           /* interrupt enable */

        g_u1_cnt_ics = 0;
    }
    /*********************/
#endif

    /*** set variables ***/
    if (MTR_FLG_SET == g_u1_enable_write)
    {
        mtr_set_variables();
        g_u1_enable_write = MTR_FLG_CLR;
    }

    mtr_clear_trd0_imfa();                                              /* clear compare match flag A */
}

/******************************************************************************
* Function Name : mtr_carrier_udf_interrupt
* Description   : Carrier underflow interrupt
* Arguments     : none
* Return Value  : none
******************************************************************************/
__interrupt static void mtr_carrier_udf_interrupt(void)
{
    EI();

    if (g_u1_bemf_th_type == MTR_BEMF_TH_HSIDE)
    {
        if (g_u1_bemf_mode == MTR_BEMF_3RD)
        {
            while (g_spi00_comend_flag == FALSE)
            {
                NOP();
            }
        }

        if (g_u1_bemf_ad_type == MTR_BEMF_AD_ISENSE)
        {
            DI();

            /* Current A/D */
            g_s2_pdrv_ad_isense = mtr_get_adc(MTR_ADCCH_PDRV);

            EI();
        }

        if (g_u1_bemf_mode == MTR_BEMF_3RD)
        {
            /* PreDriver Output Signal : BEMF */
            R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_BEMF);

            g_u1_bemf_ad_type = MTR_BEMF_AD_BEMF;
        }
    }
    else if (g_u1_bemf_th_type == MTR_BEMF_TH_LSIDE)
    {
        if (g_u1_bemf_mode == MTR_BEMF_3RD)
        {
            while (g_spi00_comend_flag == FALSE)
            {
                NOP();
            }
        }

        if (g_u1_bemf_ad_type == MTR_BEMF_AD_BEMF)
        {
            DI();

            /* BEMF A/D */
            g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

            EI();
        }

        if (g_u1_bemf_mode == MTR_BEMF_3RD)
        {
            /* PreDriver Output Signal : Current */
            R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_ISEN);

            g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
        }
    }
    else if (g_u1_bemf_th_type == MTR_BEMF_TH_HIZ)
    {
        if (g_u1_bemf_pwm_phase == MTR_PWM_PHASE_P)
        {
            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                while (g_spi00_comend_flag == FALSE)
                {
                    NOP();
                }
            }

            if (g_u1_bemf_ad_type == MTR_BEMF_AD_BEMF)
            {
                DI();

                /* BEMF A/D */
                g_s2_pdrv_ad = mtr_get_adc(MTR_ADCCH_PDRV);

                EI();
            }

            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                /* PreDriver Output Signal : Current */
                R_PreDrvReg_Write(PREDRVREG_ADC_SEL, SET_ADC_SEL_ISEN);

                g_u1_bemf_ad_type = MTR_BEMF_AD_ISENSE;
            }

            if (g_u1_bemf_ad_type == MTR_BEMF_AD_ISENSE)
            {
                DI();

                /* Current A/D */
                g_s2_pdrv_ad_isense = mtr_get_adc(MTR_ADCCH_PDRV);

                EI();
            }

            if (g_u1_bemf_mode == MTR_BEMF_3RD)
            {
                /* PreDriver Output Signal : BEMF */
                R_PreDrvReg_Write_Intr_Mode(PREDRVREG_ADC_SEL, SET_ADC_SEL_BEMF);

                g_u1_bemf_ad_type = MTR_BEMF_AD_BEMF;
            }
        }
    }
    else
    {
        /* Do Nothing */
    }

    mtr_clear_trd1_udf();                                               /* clear underflow flag */
}

