/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_main.c
* Description : The main function and the processing of an application layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "version.h"
#include "r_dsp.h"
#include "mtr_main.h"
#include "control_parameter.h"
#include "motor_parameter.h"
#include "mtr_spm_less_120_cpm.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "mtr_ctrl_t2001.h"
#include "r_cg_macrodriver.h"
#include "r_cg_predrv_reg.h"
#include "r_cg_predrv.h"
#include "r_cg_sau.h"
#include "r_cg_main.h"
#include "r_cg_tmrd.h"
#include "r_less_120_is.h"
#include "r_less_120_isw.h"

#ifdef ICS_USE
/****** for ICS ******/
#include "ics2_RL78G1F.h"
/*********************/
/*********** KEEP DTC TABLE AREA ***********/
#pragma address dtc_tbl = 0xFFE00
char dtc_tbl[0xD0];
#endif

/******************************************************************************
* Private global variables and functions
******************************************************************************/
static int16_t    g_s2_max_speed;                                       /* max speed [rpm] (mechanical angle) */
static int16_t    g_s2_min_speed;                                       /* min speed [rpm] (mechanical angle) */
static int16_t    g_s2_margin_min_speed;                                /* min speed (include margin) [rpm] (mechanical angle) */
static int16_t    g_s2_ref_speed;                                       /* speed reference */
       uint16_t   g_u2_speed_rpm;                                       /* current speed [rpm] (mechanical angle) */
static uint8_t    g_u1_rot_dir;                                         /* rotational direction */
static uint8_t    g_u1_motor_status;                                    /* motor status */
static uint8_t    g_u1_stop_req;                                        /* stop request flag */

#ifdef ICS_USE
       MD_STATUS  g_u1_pdrv_status;
#endif

static uint8_t    g_u1_err_recovery_req;                                /* error recovery request flag */
static uint8_t    g_u1_get_alarm_sts1;                                  /* alarm status1 */
static uint8_t    g_u1_store_alarm_sts1;                                /* alarm status1 */
static uint8_t    g_u1_get_alarm_sts2;                                  /* alarm status2 */
static uint8_t    g_u1_store_alarm_sts2;                                /* alarm status2 */

uint16_t   g_u2_fw_revision;                                            /* FW Revision */
uint16_t   g_u2_is_lib_revision;                                        /* Library Revision */
int16_t    com_s2_sw_userif;                                            /* user interface sw */
int16_t    g_s2_sw_userif;                                              /* user interface sw */
int16_t    com_s2_mode_system;                                          /* system mode */
int16_t    g_s2_mode_system;                                            /* system mode */
int16_t    com_s2_direction;                                            /* rotational direction (0:CW ,1:CCW) */
int16_t    com_s2_ref_speed_rpm;                                        /* motor speed reference [rpm] (mechanical angle) */
int16_t    com_s2_enable_write;                                         /* ICS write enable flag */
int16_t    g_s2_enable_write;                                           /* ICS write enable flag */
int16_t    com_s2_speed_lpf_k;                                          /* speed LPF parameter */

MTR_ICS_INPUT ics_input;                                                /* struct for ICS input */

#ifdef ICS_USE
uint8_t    com_u1_pdrvreg_ctrl;
uint8_t    com_u1_pdrvreg_ps_all_pre;
uint8_t    com_u1_pdrvreg_ps_all_now;
uint8_t    com_u1_pdrvreg_ps_pre;
uint8_t    com_u1_pdrvreg_ps_now;
uint8_t    com_u1_pdrvreg_sw_reset_pre;
uint8_t    com_u1_pdrvreg_sw_reset_now;
uint8_t    com_u1_pdrvreg_adc_sel_pre;
uint8_t    com_u1_pdrvreg_adc_sel_now;
uint8_t    com_u1_pdrvreg_selsig_u_pre;
uint8_t    com_u1_pdrvreg_selsig_u_now;
uint8_t    com_u1_pdrvreg_selsig_v_pre;
uint8_t    com_u1_pdrvreg_selsig_v_now;
uint8_t    com_u1_pdrvreg_selsig_w_pre;
uint8_t    com_u1_pdrvreg_selsig_w_now;
uint8_t    com_u1_pdrvreg_hall_sig_pre;
uint8_t    com_u1_pdrvreg_hall_sig_now;
uint8_t    com_u1_pdrvreg_almsts1_pre;
uint8_t    com_u1_pdrvreg_almsts1_now;
uint8_t    com_u1_pdrvreg_almope1_pre;
uint8_t    com_u1_pdrvreg_almope1_now;
uint8_t    com_u1_pdrvreg_almout1_pre;
uint8_t    com_u1_pdrvreg_almout1_now;
uint8_t    com_u1_pdrvreg_almsts2_pre;
uint8_t    com_u1_pdrvreg_almsts2_now;
uint8_t    com_u1_pdrvreg_cs_set2_pre;
uint8_t    com_u1_pdrvreg_cs_set2_now;
uint8_t    com_u1_pdrvreg_almout2_pre;
uint8_t    com_u1_pdrvreg_almout2_now;
uint8_t    com_u1_pdrvreg_error_wait_pre;
uint8_t    com_u1_pdrvreg_error_wait_now;

uint8_t    com_u1_pdrvreg_cs_set1_pre;
uint8_t    com_u1_pdrvreg_cs_set1_now;
uint8_t    com_u1_pdrvreg_haic_th_pre;
uint8_t    com_u1_pdrvreg_haic_th_now;
uint8_t    com_u1_pdrvreg_pddsts_pre;
uint8_t    com_u1_pdrvreg_pddsts_now;
uint8_t    com_u1_pdrvreg_ld_wait_pre;
uint8_t    com_u1_pdrvreg_ld_wait_now;
uint8_t    com_u1_pdrvreg_drive_set_pre;
uint8_t    com_u1_pdrvreg_drive_set_now;
uint8_t    com_u1_pdrvreg_di_time_pre;
uint8_t    com_u1_pdrvreg_di_time_now;
uint8_t    com_u1_pdrvreg_idrcnt_h_pre;
uint8_t    com_u1_pdrvreg_idrcnt_h_now;
uint8_t    com_u1_pdrvreg_idrcnt_l_pre;
uint8_t    com_u1_pdrvreg_idrcnt_l_now;
uint8_t    com_u1_pdrvreg_trcnt_p_pre;
uint8_t    com_u1_pdrvreg_trcnt_p_now;
uint8_t    com_u1_pdrvreg_cpset1_pre;
uint8_t    com_u1_pdrvreg_cpset1_now;
uint8_t    com_u1_pdrvreg_cpset2_pre;
uint8_t    com_u1_pdrvreg_cpset2_now;
uint8_t    com_u1_pdrvreg_cp_trim_pre;
uint8_t    com_u1_pdrvreg_cp_trim_now;

uint8_t    com_u1_pdrvreg_vreg5_trim_pre;

uint8_t    com_u1_pdrvreg_csamp_trim_pre;

uint8_t    com_u1_pdrvreg_almraw1_pre;
uint8_t    com_u1_pdrvreg_almraw1_now;
uint8_t    com_u1_pdrvreg_toin_moni_pre;
uint8_t    com_u1_pdrvreg_toin_moni_now;
uint8_t    com_u1_pdrvreg_who_am_i_pre;
uint8_t    com_u1_pdrvreg_who_am_i_now;

uint8_t    com_u1_pdrvreg_trim_pt_pre;
uint8_t    com_u1_pdrvreg_trim_pt_now;
uint8_t    com_u1_pdrvreg_trim_en_pre;

uint8_t    com_u1_pdrvreg_bgr_trim_pre;

uint8_t    com_u1_pdrvreg_bfamp_trim_pre;

#endif

uint8_t    com_u1_pdrvreg_vreg5_trim_now;
uint8_t    com_u1_pdrvreg_csamp_trim_now;

uint8_t    com_u1_pdrvreg_trim_en_now;

uint8_t    com_u1_pdrvreg_bgr_trim_now;
uint8_t    com_u1_pdrvreg_bfamp_trim_now;

int16_t    g_s2_ref_speed_rpm_vr1;
uint8_t    g_u1_alarm_sts1;
uint8_t    g_u1_alarm_sts2;

int16_t    com_s2_pwm_duty;

extern volatile int16_t    g_s2_pwm_duty;                               /* PWM duty (setting value of PWM timer compare register) */
extern volatile uint8_t    g_u1_error_status;                           /* error status */
extern volatile uint8_t    g_u1_direction;                              /* rotational direction (0:CW ,1:CCW) */
extern volatile uint8_t    g_u1_sensorless_mode;                        /* sensorless mode */
extern volatile uint8_t    g_u1_bemf_mode;                              /* BEMF mode */
extern volatile uint8_t    g_u1_bemf_seq;                               /* BEMF sequence */
extern volatile uint8_t    g_u1_bemf_req_moten_off;                     /* request MOT_EN on */
extern volatile uint8_t    g_u1_bemf_req_moten_on;                      /* request MOT_EN off */
extern volatile uint8_t    g_u1_bemf_limit_vr_ad_flg;                   /* VR1 ADC limit flag */
extern volatile uint16_t   g_u2_bemf_limit_vr_ad_cnt;                   /* counts for counts VR1 ADC limit */
extern volatile int16_t    g_u1_bemf_limit_vr_ad;                       /* VR1 ADC limit calculation */

static void ctrl_ui(void);
static void ics_ui(void);
static void ctrl_sensorless(void);
static void ctrl_led(void);
static void mcu_sw_init(void);
static void software_init(void);

#ifdef ICS_USE
static void ics_predrv_reg_ctrl(void);
#endif

/******************************************************************************
* Function Name : main
* Description   : Initialization and main routine
* Arguments     : none
* Return Value  : none
******************************************************************************/
void main(void)
{
    /*** mcu ***/
    R_MAIN_UserInit();

    /*** predriver hw init ***/
    predriver_hw_init();

    /*** mcu sw init ***/
    mcu_sw_init();

    /*** main routine ***/
    while (1)
    {
        if (g_s2_sw_userif != com_s2_sw_userif)
        {
            g_s2_sw_userif = com_s2_sw_userif;

            if (g_s2_sw_userif == ICS_UI)
            {
                g_s2_mode_system = (int16_t)g_u1_motor_status;
            }
        }

        if (BOARD_UI == g_s2_sw_userif)
        {
            ctrl_ui();                                                  /* user interface control routine */
        }
        else if (ICS_UI == g_s2_sw_userif)
        {
            ics_ui();                                                   /* user interface using ICS */
        }
        else
        {
            /* Do Nothing */
        }

        ctrl_sensorless();                                              /* sensorless */

        g_u2_speed_rpm = R_MTR_GetSpeed();                              /* watch current speed */

        ctrl_led();                                                     /* led on off */

        clear_wdt();                                                    /* watch dog timer clear */
    }
}

/******************************************************************************
* Function Name : ctrl_ui
* Description   : User interface control routine
* Arguments     : none
* Return Value  : none
******************************************************************************/
void ctrl_ui(void)
{
    int16_t s2_temp;

    /*======================*/
    /*     init             */
    /*======================*/

    s2_temp = 0;

    /*======================*/
    /*     mode control     */
    /*======================*/

    g_u1_motor_status = R_MTR_GetStatus();                              /* get status of motor control system */

    if (g_u1_motor_status == MTR_MODE_STOP)                             /* check motor status */
    {
        /* check stop_req */
        if (g_u1_stop_req == REQ_CLR)
        {
            R_PreDrv_Set_MotEn(REQ_MOT_EN_ON);

            R_MTR_ExecEvent(MTR_EVENT_RUN);                             /* execute run event */
        }
    }
    else if (g_u1_motor_status == MTR_MODE_RUN)                         /* check motor status */
    {
        /* check stop_req */
        if (g_u1_stop_req == REQ_SET)
        {
            R_MTR_ExecEvent(MTR_EVENT_STOP);                            /* execute stop event */
        }
    }
    else if (g_u1_motor_status == MTR_MODE_ERROR)                       /* check motor status */
    {
        /* check error recovery req */
        if (g_u1_err_recovery_req == REQ_SET)
        {
            /* error recovery */
            R_PREDRV_ErrorRecoverySequence(g_u1_store_alarm_sts1, g_u1_store_alarm_sts2);

            /* check ALMSTS1, ALMSTS2 */
            R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &g_u1_get_alarm_sts1);
            R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &g_u1_get_alarm_sts2);

            g_u1_get_alarm_sts1 &= ALMSTS1_VGT_UVP_MASK;                /* ALARM factor(VGT_UVP) mask */

            if ((g_u1_get_alarm_sts1 != ALMSTS1_NO_ERROR) || (g_u1_get_alarm_sts2 != ALMSTS2_NO_ERROR))
            {
                /* store alarm sts1, alarm sts2 */
                g_u1_store_alarm_sts1 = g_u1_get_alarm_sts1;
                g_u1_store_alarm_sts2 = g_u1_get_alarm_sts2;

                g_u1_alarm_sts1 = g_u1_get_alarm_sts1;
                g_u1_alarm_sts2 = g_u1_get_alarm_sts2;
            }
            else
            {
                R_MTR_ExecEvent(MTR_EVENT_RESET);                       /* execute reset event */
                g_u1_motor_status = R_MTR_GetStatus();                  /* get status of motor control system */
                g_u1_err_recovery_req = REQ_CLR;

                g_u1_alarm_sts1 = ALMSTS1_NO_ERROR;
                g_u1_alarm_sts2 = ALMSTS2_NO_ERROR;

                led_off(MTR_LED1);                                      /* LED1 off */
                led_off(MTR_LED5);                                      /* LED5 off */
                led_off(MTR_LED6);                                      /* LED6 off */
            }
        }
        else
        {
            /* check ALMSTS1, ALMSTS2 */
            R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &g_u1_get_alarm_sts1);
            R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &g_u1_get_alarm_sts2);

            g_u1_get_alarm_sts1 &= ALMSTS1_VGT_UVP_MASK;                /* ALARM factor(VGT_UVP) mask */

            if ((g_u1_get_alarm_sts1 != ALMSTS1_NO_ERROR) || (g_u1_get_alarm_sts2 != ALMSTS2_NO_ERROR))
            {
                /* store alarm sts1, alarm sts2 */
                g_u1_store_alarm_sts1 = g_u1_get_alarm_sts1;
                g_u1_store_alarm_sts2 = g_u1_get_alarm_sts2;

                g_u1_alarm_sts1 = g_u1_get_alarm_sts1;
                g_u1_alarm_sts2 = g_u1_get_alarm_sts2;
            }

            led_on(MTR_LED1);                                           /* LED1 on */

            if ((g_u1_error_status & MTR_HW_ERROR) != 0)
            {
                led_on(MTR_LED5);                                       /* LED5 on */
            }

            if ((g_u1_error_status & MTR_FW_ERROR) != 0)
            {
                led_on(MTR_LED6);                                       /* LED6 on */
            }
        }
    }
    else
    {
        /* Do Nothing */
    }

    /*=============================*/
    /*     set speed reference     */
    /*=============================*/

    s2_temp = get_vr1();
                                                                        /* read speed reference from VR1 */
                                                                        /* 0 <= (speed reference) <= 1023 */

    s2_temp = MTR_VR1_ADC_MAX - s2_temp;

    if (s2_temp < 0)
    {
        s2_temp = 0;
    }

    if (g_u2_bemf_limit_vr_ad_cnt < MTR_BEMF_LIMIT_VRAD_CNT_LIMIT)
    {
        if (g_u1_bemf_limit_vr_ad_flg == 1)
        {
            g_u1_bemf_limit_vr_ad = (g_u2_bemf_limit_vr_ad_cnt * MTR_BEMF_LIMIT_VRAD_COEF1) / MTR_BEMF_LIMIT_VRAD_COEF2;

            if (s2_temp > g_u1_bemf_limit_vr_ad)
            {
                s2_temp = g_u1_bemf_limit_vr_ad;
            }
        }
    }

    DI();                                                               /* interrupt disable */

    g_s2_pwm_duty = (uint16_t)(R_FIX_mul32_int16(s2_temp, (int16_t)MTR_PWM_DUTY_CALC_COEF1) / MTR_PWM_DUTY_CALC_COEF2);
    s2_temp = (uint16_t)((R_FIX_mul32_int16(s2_temp, MTR_RPM_CALC_COEF1) - MTR_RPM_CALC_COEF2) / MTR_RPM_CALC_COEF3);

    EI();                                                               /* interrupt enable */

    if (s2_temp < 0)
    {
        s2_temp = 0;
    }

    g_s2_ref_speed_rpm_vr1 = s2_temp;

    s2_temp = R_FIX_limit_int16(s2_temp, g_s2_max_speed);               /* higher limit is g_s2_max_speed (both CW and CCW) */

    /* s2_temp [rpm] */
    /**********************************************************
    * speed check                                             *
    * 600     <= s2_temp     : start rotation (CW or CCW)     *
    * s2_temp <= 550         : stop                           *
    * others                 : rotation with constant speed   *
    **********************************************************/

    if (s2_temp >= g_s2_min_speed)
    {
        if (MTR_GET_ROT_DIR_REQ == REQ_ROT_CW)
        {
            /* CW */
            g_u1_stop_req = REQ_CLR;                                    /* clear stop request flag */
            g_u1_rot_dir = M_CW;                                        /* rotational direction is CW */
        }
        else if (MTR_GET_ROT_DIR_REQ == REQ_ROT_CCW)
        {
            /* CCW */
            g_u1_stop_req = REQ_CLR;                                    /* clear stop request flag */
            g_u1_rot_dir = M_CCW;                                       /* rotational direction is CCW */
        }
        else
        {
            /* Do Nothing */
        }
    }
    else if (s2_temp <= g_s2_margin_min_speed)
    {
        if (g_u1_motor_status == MTR_MODE_ERROR)
        {
            g_u1_err_recovery_req = REQ_SET;                            /* set error recovery request flag */
        }
        else
        {
            g_u1_stop_req = REQ_SET;                                    /* set stop request flag */
        }
    }
    else
    {
        if (g_u1_motor_status == MTR_MODE_RUN)                          /* check motor status */
        {
            s2_temp = g_s2_min_speed;                                   /* min speed (constant) */
        }
    }

    /* rotation direction */
    if (R_MTR_GetDir() != g_u1_rot_dir)
    {
        R_MTR_ExecEvent(MTR_EVENT_STOP);                                /* execute stop event */
    }

    R_MTR_SetDir(g_u1_rot_dir);                                         /* set rotational direction */
    g_s2_ref_speed = s2_temp;                                           /* speed reference [rpm] (mechanical angle) */
    R_MTR_SetSpeed(g_s2_ref_speed);                                     /* set speed reference */
}

/******************************************************************************
* Function Name : ics_ui
* Description   : User interface using ICS
* Arguments     : none
* Return Value  : none
******************************************************************************/
void ics_ui(void)
{
    int16_t s2_temp;

    /*============================*/
    /*      get ICS value         */
    /*============================*/

    /***** limit variable *****/
    if (com_s2_ref_speed_rpm < g_s2_min_speed)
    {
        com_s2_ref_speed_rpm = g_s2_min_speed;
    }
    else if (com_s2_ref_speed_rpm > g_s2_max_speed)
    {
        com_s2_ref_speed_rpm = g_s2_max_speed;
    }
    else
    {
        /* Do Nothing */
    }

    /***** When com_s2_enable_sw and g_s2_enable_sw are same value, rewrite enable. *****/
    if (com_s2_enable_write == g_s2_enable_write)
    {
        /* reference speed */
        ics_input.s2_ref_speed = com_s2_ref_speed_rpm;

        /* rotation direction */
        if (R_MTR_GetDir() != com_s2_direction)
        {
            if (com_s2_direction > M_CCW)
            {
                com_s2_direction = M_CCW;
            }

            R_MTR_ExecEvent(MTR_EVENT_STOP);
            g_s2_mode_system = MTR_MODE_STOP;
        }

        ics_input.s2_direction = com_s2_direction;

        /* motor speed LPF parameter */
        ics_input.s2_speed_lpf_k = com_s2_speed_lpf_k;

        R_MTR_IcsInput(&ics_input);

        g_s2_enable_write ^= 1;                                         /* change every time 0 and 1 */
    }

#ifdef ICS_USE
    if (com_u1_pdrvreg_ctrl == REQ_SET)
    {
        ics_predrv_reg_ctrl();
    }
#endif

    g_s2_pwm_duty = com_s2_pwm_duty;

    /*==============================*/
    /*        execute event         */
    /*==============================*/

    s2_temp = com_s2_mode_system;

    if (g_s2_mode_system != s2_temp)
    {
        if (s2_temp == MTR_EVENT_RUN)
        {
            g_s2_mode_system = s2_temp;
            R_MTR_ExecEvent((uint8_t)g_s2_mode_system);
        }
        else
        {
            g_s2_mode_system = s2_temp;
            R_MTR_ExecEvent((uint8_t)g_s2_mode_system);
        }
    }

    g_u1_motor_status = R_MTR_GetStatus();                              /* get status of motor control system */

    if (g_s2_mode_system == MTR_EVENT_RESET)
    {
        if (g_u1_motor_status == MTR_MODE_STOP)
        {
            software_init();                                            /* initialize private global variables for reset event */
        }
        else if (g_u1_motor_status == MTR_MODE_ERROR)
        {
            g_s2_mode_system = MTR_MODE_ERROR;
            com_s2_mode_system = MTR_MODE_ERROR;
        }
        else
        {
            /* Do Nothing */
        }
    }

    if (g_u1_motor_status == MTR_MODE_ERROR)
    {
        /* check ALMSTS1, ALMSTS2 */
        R_PreDrvReg_Read(PREDRVREG_ALMSTS1, &g_u1_get_alarm_sts1);
        R_PreDrvReg_Read(PREDRVREG_ALMSTS2, &g_u1_get_alarm_sts2);

        g_u1_get_alarm_sts1 &= ALMSTS1_VGT_UVP_MASK;                    /* ALARM factor(VGT_UVP) mask */

        if ((g_u1_get_alarm_sts1 != ALMSTS1_NO_ERROR) || (g_u1_get_alarm_sts2 != ALMSTS2_NO_ERROR))
        {
            /* store alarm sts1, alarm sts2 */
            g_u1_store_alarm_sts1 = g_u1_get_alarm_sts1;
            g_u1_store_alarm_sts2 = g_u1_get_alarm_sts2;

            g_u1_alarm_sts1 = g_u1_get_alarm_sts1;
            g_u1_alarm_sts2 = g_u1_get_alarm_sts2;

            led_on(MTR_LED1);                                           /* LED1 on */

            /* error recovery */
            R_PREDRV_ErrorRecoverySequence(g_u1_store_alarm_sts1, g_u1_store_alarm_sts2);
        }
        else
        {
            g_u1_alarm_sts1 = ALMSTS1_NO_ERROR;
            g_u1_alarm_sts2 = ALMSTS2_NO_ERROR;

            led_off(MTR_LED1);                                          /* LED1 off */
        }
    }
}

/******************************************************************************
* Function Name : ctrl_sensorless
* Description   : 
* Arguments     : none
* Return Value  : none
******************************************************************************/
void ctrl_sensorless(void)
{
    if (g_u1_sensorless_mode == MTR_SENLESS_INDSEN)
    {
        st_is_prm_t st_is_prm;

        uint16_t u2_specified_time_out;
        uint16_t u2_specified_time_off;

        uint16_t u2_is_trial_num;

        is_result_t is_result;

        uint8_t u1_kick_pos;

        g_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;                /* set pwm duty */

        mtr_set_predrv_isense();                                        /* set Isense */

        TRDMK0 = 1;                                                     /* INTTRD0 interrupt servicing disabled */
        TRDMK1 = 1;                                                     /* INTTRD1 interrupt servicing disabled */

        st_is_prm.u1_direction = g_u1_direction;

        st_is_prm.u2_section_time_a = IS_TIME_RESOLUTION * IS_SECTION_TIME_A;
        st_is_prm.u2_section_time_b = IS_TIME_RESOLUTION * IS_SECTION_TIME_B;
        st_is_prm.u2_section_time_c = IS_TIME_RESOLUTION * IS_SECTION_TIME_C;

        u2_specified_time_out = IS_TIME_RESOLUTION * IS_PWM_OUT_TIME;
        u2_specified_time_off = IS_TIME_RESOLUTION * IS_PWM_OFF_TIME;

        u2_is_trial_num = 0;

        while (1)
        {
            if (IS_STATE_NORMAL == R_IS_Main(st_is_prm, &is_result))
            {
                u1_kick_pos = (uint8_t)is_result + IS_KICK_NUM;

                if (IS_PATTERN_MAX <= u1_kick_pos)
                {
                    u1_kick_pos -= (uint8_t)IS_PATTERN_MAX;
                }

                /* set voltage pattern */
                mtr_pattern_set(mtr_convert_indsenpat_to_pwmpat(u1_kick_pos));

                if (IS_TRIAL_NUM > u2_is_trial_num)
                {
                    r_isw_wait(u2_specified_time_out);

                    r_isw_set_port(IS_PORT_N_ON);                       /* set voltage pattern (low side on) */

                    r_isw_wait(u2_specified_time_off);

                    r_isw_set_port(IS_PORT_OFF);                        /* set voltage pattern (all off) */

                    r_isw_wait(u2_specified_time_off);

                    u2_is_trial_num++;
                }
                else
                {
                    mtr_set_predrv_bemf();                              /* set BEMF */

                    g_u1_sensorless_mode = MTR_SENLESS_BEMF;            /* set sensorless BEMF mode */
                    g_u1_bemf_mode = MTR_BEMF_PRE;                      /* set BEMF mode */
                    g_u1_bemf_seq = MTR_BEMF_SEQ_INIT;                  /* set BEMF sequence */

                    TRDMK1 = 0;                                         /* INTTRD1 interrupt servicing enabled */
                    TRDMK0 = 0;                                         /* INTTRD0 interrupt servicing enabled */

                    break;
                }
            }
            else
            {
#ifdef FW_ERR_DET_LESS
                R_MTR_ExecEvent(MTR_EVENT_ERROR);                       /* execute reset event */

                DI();

                g_u1_error_status |= MTR_UNKNOWN_ERROR;                 /* error status update : unknown error */

                EI();
#endif
                break;
            }
        }
    }

    if (g_u1_bemf_req_moten_on == 1)
    {
        R_PreDrv_Set_MotEn(REQ_MOT_EN_ON);

        g_u1_bemf_req_moten_on = 0;
    }

    if (g_u1_bemf_req_moten_off == 1)
    {
        R_PreDrv_Set_MotEn(REQ_MOT_EN_OFF);

        g_u1_bemf_req_moten_off = 0;
    }
}

/******************************************************************************
* Function Name : ctrl_led
* Description   : 
* Arguments     : none
* Return Value  : none
******************************************************************************/
void ctrl_led(void)
{
    if (g_s2_ref_speed > LED_ON_2ND_SPEED)
    {
        led_on(MTR_LED2);                                               /* LED2 on */
        led_on(MTR_LED3);                                               /* LED3 on */
        led_on(MTR_LED4);                                               /* LED4 on */
    }
    else if (g_s2_ref_speed > LED_ON_1ST_SPEED)
    {
        led_on(MTR_LED2);                                               /* LED2 on */
        led_on(MTR_LED3);                                               /* LED3 on */
        led_off(MTR_LED4);                                              /* LED4 off */
    }
    else
    {
        led_off(MTR_LED2);                                              /* LED2 off */
        led_off(MTR_LED3);                                              /* LED3 off */
        led_off(MTR_LED4);                                              /* LED4 off */
    }
}

#ifdef ICS_USE
/******************************************************************************
* Function Name : ics_predrv_reg_ctrl
* Description   : 
* Arguments     : none
* Return Value  : none
******************************************************************************/
void ics_predrv_reg_ctrl(void)
{
    /* read */
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_PS_ALL    , &com_u1_pdrvreg_ps_all_pre    );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_PS        , &com_u1_pdrvreg_ps_pre        );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_SW_RESET  , &com_u1_pdrvreg_sw_reset_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ADC_SEL   , &com_u1_pdrvreg_adc_sel_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_SELSIG_U  , &com_u1_pdrvreg_selsig_u_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_SELSIG_V  , &com_u1_pdrvreg_selsig_v_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_SELSIG_W  , &com_u1_pdrvreg_selsig_w_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_HALL_SIG  , &com_u1_pdrvreg_hall_sig_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMSTS1   , &com_u1_pdrvreg_almsts1_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMOPE1   , &com_u1_pdrvreg_almope1_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMOUT1   , &com_u1_pdrvreg_almout1_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMSTS2   , &com_u1_pdrvreg_almsts2_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CS_SET2   , &com_u1_pdrvreg_cs_set2_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMOUT2   , &com_u1_pdrvreg_almout2_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ERROR_WAIT, &com_u1_pdrvreg_error_wait_pre);

    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CS_SET1   , &com_u1_pdrvreg_cs_set1_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_HAIC_TH   , &com_u1_pdrvreg_haic_th_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_PDDSTS    , &com_u1_pdrvreg_pddsts_pre    );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_LD_WAIT   , &com_u1_pdrvreg_ld_wait_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_DRIVE_SET , &com_u1_pdrvreg_drive_set_pre );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_DI_TIME   , &com_u1_pdrvreg_di_time_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_IDRCNT_H  , &com_u1_pdrvreg_idrcnt_h_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_IDRCNT_L  , &com_u1_pdrvreg_idrcnt_l_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_TRCNT_P   , &com_u1_pdrvreg_trcnt_p_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CPSET1    , &com_u1_pdrvreg_cpset1_pre    );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CPSET2    , &com_u1_pdrvreg_cpset2_pre    );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CP_TRIM   , &com_u1_pdrvreg_cp_trim_pre   );

    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_VREG5_TRIM, &com_u1_pdrvreg_vreg5_trim_pre);
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_CSAMP_TRIM, &com_u1_pdrvreg_csamp_trim_pre);
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_ALMRAW1   , &com_u1_pdrvreg_almraw1_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_TOIN_MONI , &com_u1_pdrvreg_toin_moni_pre );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_WHO_AM_I  , &com_u1_pdrvreg_who_am_i_pre  );

    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_TRIM_PT   , &com_u1_pdrvreg_trim_pt_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_TRIM_EN   , &com_u1_pdrvreg_trim_en_pre   );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_BGR_TRIM  , &com_u1_pdrvreg_bgr_trim_pre  );
    g_u1_pdrv_status = R_PreDrvReg_Read(PREDRVREG_BFAMP_TRIM, &com_u1_pdrvreg_bfamp_trim_pre);

    /* write */
    if (com_u1_pdrvreg_ps_all_pre != com_u1_pdrvreg_ps_all_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_PS_ALL, com_u1_pdrvreg_ps_all_now);
    }
    if (com_u1_pdrvreg_ps_pre != com_u1_pdrvreg_ps_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_PS, com_u1_pdrvreg_ps_now);
    }
    if (com_u1_pdrvreg_sw_reset_pre != com_u1_pdrvreg_sw_reset_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_SW_RESET, com_u1_pdrvreg_sw_reset_now);
    }
    if (com_u1_pdrvreg_adc_sel_pre != com_u1_pdrvreg_adc_sel_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_ADC_SEL, com_u1_pdrvreg_adc_sel_now);
    }
    if (com_u1_pdrvreg_selsig_u_pre != com_u1_pdrvreg_selsig_u_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_SELSIG_U, com_u1_pdrvreg_selsig_u_now);
    }
    if (com_u1_pdrvreg_selsig_v_pre != com_u1_pdrvreg_selsig_v_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_SELSIG_V, com_u1_pdrvreg_selsig_v_now);
    }
    if (com_u1_pdrvreg_selsig_w_pre != com_u1_pdrvreg_selsig_w_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_SELSIG_W, com_u1_pdrvreg_selsig_w_now);
    }
    if (com_u1_pdrvreg_hall_sig_pre != com_u1_pdrvreg_hall_sig_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_HALL_SIG, com_u1_pdrvreg_hall_sig_now);
    }
    if (com_u1_pdrvreg_almope1_pre != com_u1_pdrvreg_almope1_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_ALMOPE1, com_u1_pdrvreg_almope1_now);
    }
    if (com_u1_pdrvreg_almout1_pre != com_u1_pdrvreg_almout1_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_ALMOUT1, com_u1_pdrvreg_almout1_now);
    }
    if (com_u1_pdrvreg_cs_set2_pre != com_u1_pdrvreg_cs_set2_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CS_SET2, com_u1_pdrvreg_cs_set2_now);
    }
    if (com_u1_pdrvreg_almout2_pre != com_u1_pdrvreg_almout2_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_ALMOUT2, com_u1_pdrvreg_almout2_now);
    }
    if (com_u1_pdrvreg_error_wait_pre != com_u1_pdrvreg_error_wait_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_ERROR_WAIT, com_u1_pdrvreg_error_wait_now);
    }

    if (com_u1_pdrvreg_cs_set1_pre != com_u1_pdrvreg_cs_set1_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CS_SET1, com_u1_pdrvreg_cs_set1_now);
    }
    if (com_u1_pdrvreg_haic_th_pre != com_u1_pdrvreg_haic_th_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_HAIC_TH, com_u1_pdrvreg_haic_th_now);
    }
    if (com_u1_pdrvreg_ld_wait_pre != com_u1_pdrvreg_ld_wait_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_LD_WAIT, com_u1_pdrvreg_ld_wait_now);
    }
    if (com_u1_pdrvreg_drive_set_pre != com_u1_pdrvreg_drive_set_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_DRIVE_SET, com_u1_pdrvreg_drive_set_now);
    }
    if (com_u1_pdrvreg_di_time_pre != com_u1_pdrvreg_di_time_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_DI_TIME, com_u1_pdrvreg_di_time_now);
    }
    if (com_u1_pdrvreg_idrcnt_h_pre != com_u1_pdrvreg_idrcnt_h_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_IDRCNT_H, com_u1_pdrvreg_idrcnt_h_now);
    }
    if (com_u1_pdrvreg_idrcnt_l_pre != com_u1_pdrvreg_idrcnt_l_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_IDRCNT_L, com_u1_pdrvreg_idrcnt_l_now);
    }
    if (com_u1_pdrvreg_trcnt_p_pre != com_u1_pdrvreg_trcnt_p_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_TRCNT_P, com_u1_pdrvreg_trcnt_p_now);
    }
    if (com_u1_pdrvreg_cpset1_pre != com_u1_pdrvreg_cpset1_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CPSET1, com_u1_pdrvreg_cpset1_now);
    }
    if (com_u1_pdrvreg_cpset2_pre != com_u1_pdrvreg_cpset2_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CPSET2, com_u1_pdrvreg_cpset2_now);
    }
    if (com_u1_pdrvreg_cp_trim_pre != com_u1_pdrvreg_cp_trim_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CP_TRIM, com_u1_pdrvreg_cp_trim_now);
    }

    if (com_u1_pdrvreg_vreg5_trim_pre != com_u1_pdrvreg_vreg5_trim_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_VREG5_TRIM, com_u1_pdrvreg_vreg5_trim_now);
    }
    if (com_u1_pdrvreg_csamp_trim_pre != com_u1_pdrvreg_csamp_trim_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_CSAMP_TRIM, com_u1_pdrvreg_csamp_trim_now);
    }

    if (com_u1_pdrvreg_trim_pt_pre != com_u1_pdrvreg_trim_pt_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_TRIM_PT, com_u1_pdrvreg_trim_pt_now);
    }
    if (com_u1_pdrvreg_trim_en_pre != com_u1_pdrvreg_trim_en_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_TRIM_EN, com_u1_pdrvreg_trim_en_now);
    }
    if (com_u1_pdrvreg_bgr_trim_pre != com_u1_pdrvreg_bgr_trim_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_BGR_TRIM, com_u1_pdrvreg_bgr_trim_now);
    }
    if (com_u1_pdrvreg_bfamp_trim_pre != com_u1_pdrvreg_bfamp_trim_now)
    {
        g_u1_pdrv_status = R_PreDrvReg_Write(PREDRVREG_BFAMP_TRIM, com_u1_pdrvreg_bfamp_trim_now);
    }
}
#endif

/******************************************************************************
* Function Name : mcu_sw_init
* Description   : Initialize software
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mcu_sw_init(void)
{
    DI();                                                               /* interrupt disable */

    R_IS_Init();                                                        /* initialization of library */

    software_init();                                                    /* initialize private global variables */

    R_MTR_InitSequence();                                               /* initialize sequence */

    RPECTL = 0x80U;

#ifdef ICS_USE
/****** for ICS ******/
    ics2_init(0xFE00, ICS_SCI1_P02_P03, 2, 0x40, 15, 0);
/*********************/
#endif

    R_TMRD0_Start();                                                    /* Start PWM output */

    EI();                                                               /* interrupt enable */

    R_MTR_ExecEvent(MTR_EVENT_RESET);                                   /* execute reset event */

    if (R_PreDrv_Get_ErrorCode() == PREDRV_ALARM_ERROR)
    {
        R_MTR_ExecEvent(MTR_EVENT_ERROR);                               /* execute error event */

        g_u1_error_status |= MTR_ALARM_ERROR;                           /* error status update : alarm error */
    }
}

/******************************************************************************
* Function Name : software_init
* Description   : Initialize private global variables
* Arguments     : none
* Return Value  : none
******************************************************************************/
void software_init(void)
{
    g_u2_fw_revision = FW_REVISION;                                     /* set FW Revision */

    R_IS_GetRevision((is_revision_t*)&g_u2_is_lib_revision);            /* get library revision */

    g_s2_max_speed = MAX_SPEED;                                         /* set MAX_SPEED (constant) */
    g_s2_min_speed = MIN_SPEED;                                         /* set MIN_SPEED (constant) */
    g_s2_margin_min_speed = MARGIN_MIN_SPEED;                           /* set MARGIN_MIN_SPEED (constant) */
    g_s2_ref_speed = 0;
    g_u2_speed_rpm = 0;
    g_u1_rot_dir = 0;
    g_u1_motor_status = 0;
    g_u1_stop_req = REQ_SET;                                            /* set REQ_SET */

    g_u1_err_recovery_req = REQ_CLR;                                    /* set REQ_CLR */
    g_u1_get_alarm_sts1 = 0xFF;
    g_u1_store_alarm_sts1 = 0xFF;
    g_u1_get_alarm_sts2 = 0xFF;
    g_u1_store_alarm_sts2 = 0xFF;

    com_s2_sw_userif = BOARD_UI;                                        /* set BOARD_UI */
    g_s2_sw_userif = BOARD_UI;                                          /* set BOARD_UI */

    com_s2_mode_system = 0;
    g_s2_mode_system = 0;
    com_s2_direction = 0;
    com_s2_ref_speed_rpm = 0;
    com_s2_enable_write = 0;
    g_s2_enable_write = 0;
    com_s2_speed_lpf_k = SPEED_LPF_K;                                   /* set SPEED_LPF_K */

    ics_input.s2_ref_speed = 0;
    ics_input.s2_direction = 0;
    ics_input.s2_speed_lpf_k = com_s2_speed_lpf_k;                      /* set SPEED_LPF_K */

#ifdef ICS_USE
    g_u1_pdrv_status = MD_OK;

    com_u1_pdrvreg_ctrl           = REQ_CLR;
    com_u1_pdrvreg_ps_all_pre     = 0;
    com_u1_pdrvreg_ps_all_now     = INIT_ICS_PS_ALL;
    com_u1_pdrvreg_ps_pre         = 0;
    com_u1_pdrvreg_ps_now         = INIT_ICS_PS;
    com_u1_pdrvreg_sw_reset_pre   = 0;
    com_u1_pdrvreg_sw_reset_now   = INIT_ICS_SW_RESET;
    com_u1_pdrvreg_adc_sel_pre    = 0;
    com_u1_pdrvreg_adc_sel_now    = INIT_ICS_ADC_SEL;
    com_u1_pdrvreg_selsig_u_pre   = 0;
    com_u1_pdrvreg_selsig_u_now   = INIT_ICS_SELSIG_U;
    com_u1_pdrvreg_selsig_v_pre   = 0;
    com_u1_pdrvreg_selsig_v_now   = INIT_ICS_SELSIG_V;
    com_u1_pdrvreg_selsig_w_pre   = 0;
    com_u1_pdrvreg_selsig_w_now   = INIT_ICS_SELSIG_W;
    com_u1_pdrvreg_hall_sig_pre   = 0;
    com_u1_pdrvreg_hall_sig_now   = INIT_ICS_HALL_SIG;
    com_u1_pdrvreg_almsts1_pre    = 0;
    com_u1_pdrvreg_almsts1_now    = INIT_ICS_ALMSTS1;
    com_u1_pdrvreg_almope1_pre    = 0;
    com_u1_pdrvreg_almope1_now    = INIT_ICS_ALMOPE1;
    com_u1_pdrvreg_almout1_pre    = 0;
    com_u1_pdrvreg_almout1_now    = INIT_ICS_ALMOUT1;
    com_u1_pdrvreg_almsts2_pre    = 0;
    com_u1_pdrvreg_almsts2_now    = INIT_ICS_ALMSTS2;
    com_u1_pdrvreg_cs_set2_pre    = 0;
    com_u1_pdrvreg_cs_set2_now    = INIT_ICS_CS_SET2;
    com_u1_pdrvreg_almout2_pre    = 0;
    com_u1_pdrvreg_almout2_now    = INIT_ICS_ALMOUT2;
    com_u1_pdrvreg_error_wait_pre = 0;
    com_u1_pdrvreg_error_wait_now = INIT_ICS_ERROR_WAIT;

    com_u1_pdrvreg_cs_set1_pre    = 0;
    com_u1_pdrvreg_cs_set1_now    = INIT_ICS_CS_SET1;
    com_u1_pdrvreg_haic_th_pre    = 0;
    com_u1_pdrvreg_haic_th_now    = INIT_ICS_HAIC_TH;
    com_u1_pdrvreg_pddsts_pre     = 0;
    com_u1_pdrvreg_pddsts_now     = INIT_ICS_PDDSTS;
    com_u1_pdrvreg_ld_wait_pre    = 0;
    com_u1_pdrvreg_ld_wait_now    = INIT_ICS_LD_WAIT;
    com_u1_pdrvreg_drive_set_pre  = 0;
    com_u1_pdrvreg_drive_set_now  = INIT_ICS_DRIVE_SET;
    com_u1_pdrvreg_di_time_pre    = 0;
    com_u1_pdrvreg_di_time_now    = INIT_ICS_DI_TIME;
    com_u1_pdrvreg_idrcnt_h_pre   = 0;
    com_u1_pdrvreg_idrcnt_h_now   = INIT_ICS_IDRCNT_H;
    com_u1_pdrvreg_idrcnt_l_pre   = 0;
    com_u1_pdrvreg_idrcnt_l_now   = INIT_ICS_IDRCNT_L;
    com_u1_pdrvreg_trcnt_p_pre    = 0;
    com_u1_pdrvreg_trcnt_p_now    = INIT_ICS_TRCNT_P;
    com_u1_pdrvreg_cpset1_pre     = 0;
    com_u1_pdrvreg_cpset1_now     = INIT_ICS_CPSET1;
    com_u1_pdrvreg_cpset2_pre     = 0;
    com_u1_pdrvreg_cpset2_now     = INIT_ICS_CPSET2;
    com_u1_pdrvreg_cp_trim_pre    = 0;
    com_u1_pdrvreg_cp_trim_now    = INIT_ICS_CP_TRIM;

    com_u1_pdrvreg_vreg5_trim_pre = 0;
 /* com_u1_pdrvreg_vreg5_trim_now = INIT_ICS_VREG5_TRIM; */
    com_u1_pdrvreg_csamp_trim_pre = 0;
 /* com_u1_pdrvreg_csamp_trim_now = INIT_ICS_CSAMP_TRIM; */
    com_u1_pdrvreg_almraw1_pre    = 0;
    com_u1_pdrvreg_almraw1_now    = INIT_ICS_ALMRAW1;
    com_u1_pdrvreg_toin_moni_pre  = 0;
    com_u1_pdrvreg_toin_moni_now  = INIT_ICS_TOIN_MONI;
    com_u1_pdrvreg_who_am_i_pre   = 0;
    com_u1_pdrvreg_who_am_i_now   = INIT_ICS_WHO_AM_I;

    com_u1_pdrvreg_trim_pt_pre    = 0;
    com_u1_pdrvreg_trim_pt_now    = INIT_ICS_TRIM_PT;
    com_u1_pdrvreg_trim_en_pre    = 0;
 /* com_u1_pdrvreg_trim_en_now    = INIT_ICS_TRIM_EN; */
    com_u1_pdrvreg_bgr_trim_pre   = 0;
 /* com_u1_pdrvreg_bgr_trim_now   = INIT_ICS_BGR_TRIM; */
    com_u1_pdrvreg_bfamp_trim_pre = 0;
 /* com_u1_pdrvreg_bfamp_trim_now = INIT_ICS_BFAMP_TRIM; */
#endif

    g_s2_ref_speed_rpm_vr1 = 0;
    g_u1_alarm_sts1 = ALMSTS1_NO_ERROR;
    g_u1_alarm_sts2 = ALMSTS2_NO_ERROR;

    com_s2_pwm_duty = (uint16_t)MTR_START_CARRIER_SET;
}

