/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : r_less_120_isw.h
* Description : A definitions of the processing of a H/W control layer
******************************************************************************/
/******************************************************************************
* History : 27.06.2019  Ver.1.01
******************************************************************************/

/* guard against multiple inclusion */
#ifndef R_LESS_120_ISW_H
#define R_LESS_120_ISW_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define IS_PORT_OFF             (0x00U)                                 /* all OFF("Low") */
#define IS_PORT_P_ON            (0x2CU)                                 /* Up, Vp, Wp = ON("High") */
#define IS_PORT_N_ON            (0x13U)                                 /* Un, Vn, Wn = ON("High") */

/******************************************************************************
* Typedef definitions
******************************************************************************/
typedef enum
{
    ISW_STATE_INIT = 0,                                                 /* initial value */
    ISW_STATE_NORMAL,                                                   /* normal */
    ISW_STATE_ERROR,                                                    /* error */

    ISW_STATE_MAX
} e_isw_status_t;                                                       /* return value */

/******************************************************************************
* Global function definitions
******************************************************************************/
extern e_isw_status_t r_isw_set_port(const uint8_t);                    /* set general purpose port */
extern e_isw_status_t r_isw_wait(const uint16_t);                       /* wait for a specified time */
extern e_isw_status_t r_isw_clear_wdt(void);                            /* clear watchdog timer */
extern e_isw_status_t r_isw_get_adc(int16_t*);                          /* get A/D conversion result */

#endif /* R_LESS_120_ISW_H */
