/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_rl78g1f_t2001.c
* Description : The processing of a H/W control layer(MCU dependance)
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "mtr_spm_less_120_cpm.h"

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile int16_t    g_s2_pwm_duty;                               /* PWM duty (setting value of PWM timer compare register) */

/* for sequence control */
extern volatile uint8_t    g_u1_error_status;                           /* error status */

extern volatile uint8_t    g_u1_sensorless_mode;                        /* sensorless mode */
extern volatile uint8_t    g_u1_bemf_pwm_phase;                         /* type of voltage pattern */
extern volatile uint8_t    g_u1_bemf_th_type;                           /* type of BEMF */
extern volatile uint8_t    g_u1_bemf_chk_hiz_status;                    /* status of Hi-Z detection */
extern volatile int16_t    g_s2_bemf_fix_pwm_duty_cnt;                  /* counts for pwm duty fixed */
extern volatile int16_t    g_s2_bemf_timer_cnt_ave;                     /* counts for 360 degrees */
extern volatile uint8_t    g_u1_bemf_limit_pwm_duty_flg;                /* pwm duty clamp flag */
extern volatile uint8_t    g_u1_bemf_limit_vr_ad_flg;                   /* VR1 ADC limit flag */

/******************************************************************************
* Function Name : mtr_ctrl_start
* Description   : motor control start (empty function)
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_start(void)
{
    /* Do Nothing */
}

/******************************************************************************
* Function Name : mtr_ctrl_stop
* Description   : TRD output disable, other interrupt settings,
*                 motor output port is on/off for motor control stop
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_stop(void)
{
    TRDOER1 = 0xFF;                                                     /* TRD output disable */

    MTR_PORT_UP = 0;                                                    /* Up = OFF("Low") */
    MTR_PORT_UN = 1;                                                    /* Un = ON("High") */
    MTR_PORT_VP = 0;                                                    /* Vp = OFF("Low") */
    MTR_PORT_VN = 1;                                                    /* Vn = ON("High") */
    MTR_PORT_WP = 0;                                                    /* Wp = OFF("Low") */
    MTR_PORT_WN = 1;                                                    /* Wn = ON("High") */

    TRDMK0 = 0;                                                         /* INTTRD0 interrupt servicing enabled */
    TRDMK1 = 1;                                                         /* INTTRD1 interrupt servicing disabled */
}

/******************************************************************************
* Function Name : mtr_change_pattern
* Description   : Change voltage pattern
* Arguments     : pattern - Voltage pattern
* Return Value  : none
******************************************************************************/
void mtr_change_pattern(uint8_t pattern)
{
    int16_t s2_pwm_duty = g_s2_pwm_duty;
    int16_t s2_duty = 0;

    /*========================*/
    /*     stop PWM timer     */
    /*========================*/

    TRDSTR = 0x0C;                                                      /* stop TRD0 */

    if (g_u1_sensorless_mode == MTR_SENLESS_BEMF)
    {
        if (g_s2_bemf_fix_pwm_duty_cnt < MTR_BEMF_FIX_PWMDUTY_CNT_LIMIT)
        {
            s2_pwm_duty = (int16_t)((MTR_CARRIER_SET * MTR_BEMF_FIX_PWMDUTY) / 100);

            if (g_u1_bemf_chk_hiz_status == 1)
            {
                g_s2_bemf_fix_pwm_duty_cnt++;

                if (g_s2_bemf_fix_pwm_duty_cnt >= MTR_BEMF_FIX_PWMDUTY_CNT_LIMIT)
                {
                    g_u1_bemf_limit_vr_ad_flg = 1;
                }
            }
        }

        if (g_s2_bemf_timer_cnt_ave != 0)
        {
            if (g_u1_bemf_limit_pwm_duty_flg == 0)
            {
                if (g_s2_bemf_timer_cnt_ave <= MTR_BEMF_LIMIT_PWMDUTY_ELEROT_TIME)
                {
                    g_u1_bemf_limit_pwm_duty_flg = 1;
                }
            }
            else
            {
                s2_duty = (int16_t)(R_FIX_mul32_int16(s2_pwm_duty, 100) / (int16_t)MTR_CARRIER_SET);

                if (s2_duty < MTR_BEMF_LIMIT_PWMDUTY)
                {
                    g_u1_bemf_limit_pwm_duty_flg = 0;
                }

                if (g_s2_bemf_timer_cnt_ave > MTR_BEMF_LIMIT_PWMDUTY_ELEROT_TIME)
                {
                    g_u1_bemf_limit_pwm_duty_flg = 0;
                }
            }

            if (g_u1_bemf_limit_pwm_duty_flg == 1)
            {
                s2_pwm_duty = (int16_t)((MTR_CARRIER_SET * MTR_BEMF_LIMIT_PWMDUTY) / 100);
            }
        }

        s2_duty = (int16_t)(R_FIX_mul32_int16(s2_pwm_duty, 100) / (int16_t)MTR_CARRIER_SET);
    }

    /*======================*/
    /*     set register     */
    /*======================*/

    switch (pattern)
    {
        case MTR_VP_ON_W_PWM:
            TRDOER1 = 0x5F;                                             /* Wn : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 1;                                            /* Vp = ON ("High") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 1;                                        /* BEMF measure(U) ON */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        case MTR_W_PWM_VN_ON:
            TRDOER1 = 0x5F;                                             /* Wp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 1;                                            /* Vn = ON ("High") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 1;                                        /* BEMF measure(U) ON */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_WP_ON_U_PWM:
            TRDOER1 = 0xF5;                                             /* Un : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 1;                                            /* Wp = ON ("High") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 1;                                        /* BEMF measure(V) ON */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        case MTR_U_PWM_WN_ON:
            TRDOER1 = 0xF5;                                             /* Up : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 1;                                            /* Wn = ON ("High") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 1;                                        /* BEMF measure(V) ON */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_UP_ON_V_PWM:
            TRDOER1 = 0xAF;                                             /* Vn : PWM output enable */
            MTR_PORT_UP = 1;                                            /* Up = ON ("High") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 1;                                        /* BEMF measure(W) ON */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        case MTR_V_PWM_UN_ON:
            TRDOER1 = 0xAF;                                             /* Vp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 1;                                            /* Un = ON ("High") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 1;                                        /* BEMF measure(W) ON */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_U_PWM_VN_ON:
            TRDOER1 = 0xF5;                                             /* Up : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 1;                                            /* Vn = ON ("High") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 1;                                        /* BEMF measure(W) ON */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_VP_ON_U_PWM:
            TRDOER1 = 0xF5;                                             /* Un : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 1;                                            /* Vp = ON ("High") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 1;                                        /* BEMF measure(W) ON */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        case MTR_V_PWM_WN_ON:
            TRDOER1 = 0xAF;                                             /* Vp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 1;                                            /* Wn = ON ("High") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 1;                                        /* BEMF measure(U) ON */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_WP_ON_V_PWM:
            TRDOER1 = 0xAF;                                             /* Vn : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 1;                                            /* Wp = ON ("High") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 1;                                        /* BEMF measure(U) ON */
            MTR_PORT_BEMF_V = 0;                                        /* BEMF measure(V) OFF */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        case MTR_W_PWM_UN_ON:
            TRDOER1 = 0x5F;                                             /* Wp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 1;                                            /* Un = ON ("High") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - s2_pwm_duty;           /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 1;                                        /* BEMF measure(V) ON */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_N;                      /* regeneration : low side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_LSIDE;                  /* BEMF : regeneration (low side) */
            }
        break;

        case MTR_UP_ON_W_PWM:
            TRDOER1 = 0x5F;                                             /* Wn : PWM output enable */
            MTR_PORT_UP = 1;                                            /* Up = ON ("High") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRC1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRD1 = s2_pwm_duty;                                      /* duty of W phase */
            TRDGRB0 = s2_pwm_duty;                                      /* duty of U phase */
            TRDGRA1 = s2_pwm_duty;                                      /* duty of V phase */
            TRDGRB1 = s2_pwm_duty;                                      /* duty of W phase */
            MTR_PORT_BEMF_U = 0;                                        /* BEMF measure(U) OFF */
            MTR_PORT_BEMF_V = 1;                                        /* BEMF measure(V) ON */
            MTR_PORT_BEMF_W = 0;                                        /* BEMF measure(W) OFF */
            g_u1_bemf_pwm_phase = MTR_PWM_PHASE_P;                      /* regeneration : high side */

            /* check type of BEMF */
            if (s2_duty >= MTR_BEMF_TIMING_TH)
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HIZ;                    /* BEMF : current */
            }
            else
            {
                g_u1_bemf_th_type = MTR_BEMF_TH_HSIDE;                  /* BEMF : regeneration (high side) */
            }
        break;

        /* the default case is intentionally combined */
        case MTR_PATTERN_ERROR:
        default:
#ifdef FW_ERR_DET_LESS
            R_MTR_ExecEvent(MTR_EVENT_ERROR);                           /* execute reset event */

            DI();

            g_u1_error_status |= MTR_LESS_ERROR;                        /* error status update : less error */

            EI();
#endif
        break;
    }

    /*===========================*/
    /*     restart PWM timer     */
    /*===========================*/

    TRD0 = (uint16_t)MTR_DEADTIME_SET;
    TRD1 = 0;

    TRDSTR = 0x0F;                                                      /* start TRD0 */
}

/******************************************************************************
* Function Name : mtr_get_adc
* Description   : A/D conversion
* Arguments     : ad_ch - A/D converter channel number
* Return Value  : A/D converted value
******************************************************************************/
int16_t mtr_get_adc(uint8_t ad_ch)
{
    int16_t s2_temp;

    ADS = ad_ch;                                                        /* set converted channel */

    ADCS = 1;

    while (ADIF == 0)
    {
        /* waiting for A/D converter */
    }

    s2_temp = ((ADCR >> 6) & 0x03FF);                                   /* get A/D converted value (10bit) */

    ADIF = 0;

    return (s2_temp);
}

