/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : r_less_120_isw.c
* Description : The processing of a H/W control layer
******************************************************************************/
/******************************************************************************
* History : 25.06.2019  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_stdint.h"
#include "r_less_120_isw.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"

/******************************************************************************
* Function Name : r_isw_set_port
* Description   : set general purpose port
*                 bit5 ... Up
*                 bit4 ... Un
*                 bit3 ... Vp
*                 bit2 ... Wp
*                 bit1 ... Vn
*                 bit0 ... Wn
* Arguments     : general port setting (Up, Un, Vp, Vn, Wp, Wn)
* Return Value  : processing result
******************************************************************************/
e_isw_status_t r_isw_set_port(const uint8_t u1_input)
{
    TRDOER1 = 0xFF;

    P1 = (P1 & (uint8_t)~0x3FU) | (u1_input & 0x3FU);

    return (ISW_STATE_NORMAL);
}

/******************************************************************************
* Function Name : r_isw_wait
* Description   : wait for a specified time
* Arguments     : specified time [us]
* Return Value  : processing result
******************************************************************************/
e_isw_status_t r_isw_wait(const uint16_t u2_input)
{
    uint16_t u2_temp;

    u2_temp = MTR_TAU1_CNT;

    while ((u2_temp - MTR_TAU1_CNT) < u2_input);

    return (ISW_STATE_NORMAL);
}

/******************************************************************************
* Function Name : r_isw_clear_wdt
* Description   : clear watchdog timer
* Arguments     : none
* Return Value  : processing result
******************************************************************************/
e_isw_status_t r_isw_clear_wdt(void)
{
    clear_wdt();

    return (ISW_STATE_NORMAL);
}

/******************************************************************************
* Function Name : r_isw_get_adc
* Description   : get A/D conversion result
* Arguments     : A/D conversion result (current)
* Return Value  : processing result
******************************************************************************/
e_isw_status_t r_isw_get_adc(int16_t* p_s2_output)
{
    *p_s2_output = mtr_get_adc(MTR_ADCCH_PDRV);

    return (ISW_STATE_NORMAL);
}

